@@ -17,10 +17,10 @@
 #include "evas_private.h"
 #include "Evas.h"
 
-static int _threads_max = 0;
-
 #ifdef BUILD_ASYNC_PRELOAD
 
+static int _threads_max = 0;
+
 typedef struct _Evas_Preload_Pthread_Worker Evas_Preload_Pthread_Worker;
 typedef struct _Evas_Preload_Pthread_Data Evas_Preload_Pthread_Data;
 
@@ -45,7 +45,7 @@ struct _Evas_Preload_Pthread_Data
 static int _threads_count = 0;
 static Evas_Preload_Pthread_Worker *_workers = NULL;
 
-static LK(_mutex) = PTHREAD_MUTEX_INITIALIZER;
+static LK(_mutex);
 
 static void
 _evas_preload_thread_end(void *data)
@@ -53,7 +53,9 @@ _evas_preload_thread_end(void *data)
    Evas_Preload_Pthread_Data *pth = data;
    Evas_Preload_Pthread_Data *p = NULL;
 
-   if (pthread_join(pth->thread, (void **)&p) != 0) free(p);
+   if (pthread_join(pth->thread, (void **)&p) == 0) free(p);
+   else return;
+   eina_threads_shutdown();
 }
 
 static void
@@ -66,6 +68,7 @@ _evas_preload_thread_done(void *target __UNUSED__, Evas_Callback_Type type __UNU
      }
    else
       work->func_end(work->data);
+
    free(work);
 }
 
@@ -74,7 +77,7 @@ _evas_preload_thread_worker(void *data)
 {
    Evas_Preload_Pthread_Data *pth = data;
    Evas_Preload_Pthread_Worker *work;
-   
+
    eina_sched_prio_drop();
    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
@@ -125,8 +128,12 @@ on_error:
 void
 _evas_preload_thread_init(void)
 {
+#ifdef BUILD_ASYNC_PRELOAD
    _threads_max = eina_cpu_count();
    if (_threads_max < 1) _threads_max = 1;
+
+   LKI(_mutex);
+#endif
 }
 
 void
@@ -149,6 +156,8 @@ _evas_preload_thread_shutdown(void)
 	free(work);
      }
    LKU(_mutex);
+
+   LKD(_mutex);
 #endif
 }
 
@@ -187,7 +196,9 @@ evas_preload_thread_run(void (*func_heavy) (void *data),
    /* One more thread could be created. */
    pth = malloc(sizeof(Evas_Preload_Pthread_Data));
    if (!pth) goto on_error;
-   
+
+   eina_threads_init();
+
    if (pthread_create(&pth->thread, NULL, _evas_preload_thread_worker, pth) == 0)
      {
 	LKL(_mutex);
@@ -196,6 +207,8 @@ evas_preload_thread_run(void (*func_heavy) (void *data),
 	return (Evas_Preload_Pthread*)work;
      }
 
+   eina_threads_shutdown();
+
  on_error:
    LKL(_mutex);
    if (_threads_count == 0)
@@ -212,6 +225,7 @@ evas_preload_thread_run(void (*func_heavy) (void *data),
     If no thread and as we don't want to break app that rely on this
     facility, we will lock the interface until we are done.
     */
+   (void)func_cancel;
    func_heavy((void *)data);
    func_end((void *)data);
    return (void *)1;
@@ -240,12 +254,13 @@ evas_preload_thread_cancel(Evas_Preload_Pthread *thread)
           }
      }
    LKU(_mutex);
-   
+
    /* Delay the destruction */
    work = (Evas_Preload_Pthread_Worker *)thread;
    work->cancel = EINA_TRUE;
    return EINA_FALSE;
 #else
+   (void) thread;
    return EINA_TRUE;
 #endif
 }
