@@ -12,9 +12,9 @@ evas_object_inject(Evas_Object *obj, Evas *e)
    lay = evas_layer_find(e, obj->cur.layer);
    if (!lay)
      {
-	lay = evas_layer_new(e);
-	lay->layer = obj->cur.layer;
-	evas_layer_add(lay);
+        lay = evas_layer_new(e);
+        lay->layer = obj->cur.layer;
+        evas_layer_add(lay);
      }
    lay->objects = (Evas_Object *)eina_inlist_append(EINA_INLIST_GET(lay->objects), EINA_INLIST_GET(obj));
    lay->usage++;
@@ -30,11 +30,11 @@ evas_object_release(Evas_Object *obj, int clean_layer)
    obj->layer->usage--;
    if (clean_layer)
      {
-	if (obj->layer->usage <= 0)
-	  {
-	     evas_layer_del(obj->layer);
-	     _evas_layer_free(obj->layer);
-	  }
+        if (obj->layer->usage <= 0)
+          {
+             evas_layer_del(obj->layer);
+             _evas_layer_free(obj->layer);
+          }
      }
    obj->layer = NULL;
    obj->in_layer = 0;
@@ -64,8 +64,8 @@ evas_layer_pre_free(Evas_Layer *lay)
 
    EINA_INLIST_FOREACH(lay->objects, obj)
      {
-	if ((!obj->smart.parent) && (!obj->delete_me))
-	  evas_object_del(obj);
+        if ((!obj->smart.parent) && (!obj->delete_me))
+          evas_object_del(obj);
      }
 }
 
@@ -74,10 +74,10 @@ evas_layer_free_objects(Evas_Layer *lay)
 {
    while (lay->objects)
      {
-	Evas_Object *obj;
+        Evas_Object *obj;
 
-	obj = (Evas_Object *)lay->objects;
-	evas_object_free(obj, 0);
+        obj = (Evas_Object *)lay->objects;
+        evas_object_free(obj, 0);
      }
 }
 
@@ -88,9 +88,9 @@ evas_layer_clean(Evas *e)
 
    while (e->layers)
      {
-	tmp = e->layers;
-	evas_layer_del(tmp);
-	_evas_layer_free(tmp);
+        tmp = e->layers;
+        evas_layer_del(tmp);
+        _evas_layer_free(tmp);
      }
 }
 
@@ -101,7 +101,7 @@ evas_layer_find(Evas *e, short layer_num)
 
    EINA_INLIST_FOREACH(e->layers, layer)
      {
-	if (layer->layer == layer_num) return layer;
+        if (layer->layer == layer_num) return layer;
      }
    return NULL;
 }
@@ -113,13 +113,13 @@ evas_layer_add(Evas_Layer *lay)
 
    EINA_INLIST_FOREACH(lay->evas->layers, layer)
      {
-	if (layer->layer > lay->layer)
-	  {
-	     lay->evas->layers = (Evas_Layer *)eina_inlist_prepend_relative(EINA_INLIST_GET(lay->evas->layers),
-									    EINA_INLIST_GET(lay),
-									    EINA_INLIST_GET(layer));
-	     return;
-	  }
+        if (layer->layer > lay->layer)
+          {
+             lay->evas->layers = (Evas_Layer *)eina_inlist_prepend_relative(EINA_INLIST_GET(lay->evas->layers),
+                                                                            EINA_INLIST_GET(lay),
+                                                                            EINA_INLIST_GET(layer));
+             return;
+          }
      }
    lay->evas->layers = (Evas_Layer *)eina_inlist_append(EINA_INLIST_GET(lay->evas->layers), EINA_INLIST_GET(lay));
 }
@@ -135,20 +135,6 @@ evas_layer_del(Evas_Layer *lay)
 
 /* public functions */
 
-/**
- * @addtogroup Evas_Object_Group_Basic
- * @{
- */
-
-/**
- * Sets the layer of the evas that the given object will be part of.
- *
- * It is not possible to change the layer of a smart object's child.
- *
- * @param   obj The given evas object.
- * @param   l   The number of the layer to place the object on.
- *          Between #EVAS_LAYER_MIN and #EVAS_LAYER_MAX.
- */
 EAPI void
 evas_object_layer_set(Evas_Object *obj, short l)
 {
@@ -157,12 +143,13 @@ evas_object_layer_set(Evas_Object *obj, short l)
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
+   if (obj->delete_me) return;
    if (evas_object_intercept_call_layer_set(obj, l)) return;
    if (obj->smart.parent) return;
    if (obj->cur.layer == l)
      {
-	evas_object_raise(obj);
-	return;
+        evas_object_raise(obj);
+        return;
      }
    e = obj->layer->evas;
    evas_object_release(obj, 1);
@@ -172,36 +159,26 @@ evas_object_layer_set(Evas_Object *obj, short l)
    evas_object_change(obj);
    if (obj->clip.clipees)
      {
-	evas_object_inform_call_restack(obj);
-	return;
+        evas_object_inform_call_restack(obj);
+        return;
      }
    evas_object_change(obj);
    if (!obj->smart.smart)
      {
-	if (evas_object_is_in_output_rect(obj,
-					  obj->layer->evas->pointer.x,
-					  obj->layer->evas->pointer.y, 1, 1) &&
-	    obj->cur.visible)
-	  if (eina_list_data_find(obj->layer->evas->pointer.object.in, obj))
-	    evas_event_feed_mouse_move(obj->layer->evas,
-				       obj->layer->evas->pointer.x,
-				       obj->layer->evas->pointer.y,
-				       obj->layer->evas->last_timestamp,
-				       NULL);
+        if (evas_object_is_in_output_rect(obj,
+                                          obj->layer->evas->pointer.x,
+                                          obj->layer->evas->pointer.y, 1, 1) &&
+            obj->cur.visible)
+          if (eina_list_data_find(obj->layer->evas->pointer.object.in, obj))
+            evas_event_feed_mouse_move(obj->layer->evas,
+                                       obj->layer->evas->pointer.x,
+                                       obj->layer->evas->pointer.y,
+                                       obj->layer->evas->last_timestamp,
+                                       NULL);
      }
    evas_object_inform_call_restack(obj);
 }
 
-/**
- * Retrieves the layer of the evas that the given object is part of.
- *
- * Be carefull, it doesn't make sense to change the layer of smart object's
- * child. So the returned value could be wrong in some case. Don't rely on
- * it's accuracy.
- *
- * @param   obj The given evas object.
- * @return  Number of the layer.
- */
 EAPI short
 evas_object_layer_get(const Evas_Object *obj)
 {
@@ -214,7 +191,3 @@ evas_object_layer_get(const Evas_Object *obj)
      }
    return obj->cur.layer;
 }
-
-/**
- * @}
- */
