@@ -1,5 +1,9 @@
 #include "evas_common.h"
 
+#ifdef _WIN32_WCE
+# undef remove
+#endif
+
 typedef struct _Evas_Object_Box_Iterator Evas_Object_Box_Iterator;
 typedef struct _Evas_Object_Box_Accessor Evas_Object_Box_Accessor;
 
@@ -19,11 +23,6 @@ struct _Evas_Object_Box_Accessor
    const Evas_Object *box;
 };
 
-/**
- * @addtogroup Evas_Object_Box
- * @{
- */
-
 #define _evas_object_box_type "Evas_Object_Box"
 #define SIG_CHILD_ADDED "child,added"
 #define SIG_CHILD_REMOVED "child,removed"
@@ -92,11 +91,11 @@ _evas_object_box_iterator_free(Evas_Object_Box_Iterator *it)
 }
 
 static Eina_Bool
-_evas_object_box_accessor_get_at(Evas_Object_Box_Accessor *it, unsigned int index, void **data)
+_evas_object_box_accessor_get_at(Evas_Object_Box_Accessor *it, unsigned int idx, void **data)
 {
    Evas_Object_Box_Option *opt = NULL;
 
-   if (!eina_accessor_data_get(it->real_accessor, index, (void *)&opt))
+   if (!eina_accessor_data_get(it->real_accessor, idx, (void *)&opt))
      return EINA_FALSE;
    if (data) *data = opt->obj;
    return EINA_TRUE;
@@ -544,25 +543,12 @@ _evas_object_box_smart_set_user(Evas_Object_Box_Api *api)
    api->option_free = _evas_object_box_option_free_default;
 }
 
-/**
- * Create a new box.
- *
- * Its layout function must be set via evas_object_box_layout_set()
- * (defaults to evas_object_box_layout_horizontal()).  The other
- * properties of the box must be set/retrieved via
- * evas_object_box_{h,v}_{align,padding}_{get,set)().
- */
 EAPI Evas_Object *
 evas_object_box_add(Evas *evas)
 {
    return evas_object_smart_add(evas, _evas_object_box_smart_class_new());
 }
 
-/**
- * Create a box that is child of a given element @a parent.
- *
- * @see evas_object_box_add()
- */
 EAPI Evas_Object *
 evas_object_box_add_to(Evas_Object *parent)
 {
@@ -575,10 +561,6 @@ evas_object_box_add_to(Evas_Object *parent)
    return o;
 }
 
-/**
- * Set the default box @a api struct (Evas_Object_Box_Api)
- * with the default values. May be used to extend that API.
- */
 EAPI void
 evas_object_box_smart_set(Evas_Object_Box_Api *api)
 {
@@ -587,9 +569,6 @@ evas_object_box_smart_set(Evas_Object_Box_Api *api)
    _evas_object_box_smart_set(api);
 }
 
-/**
- * Get Box Smart Class for inheritance purposes
- */
 EAPI const Evas_Object_Box_Api *
 evas_object_box_smart_class_get(void)
 {
@@ -606,11 +585,6 @@ evas_object_box_smart_class_get(void)
    return class;
 }
 
-/**
- * Set a 'calculate' callback (@a cb) to the @a o box's smart class,
- * which here defines its genre (horizontal, vertical, homogeneous,
- * etc.).
- */
 EAPI void
 evas_object_box_layout_set(Evas_Object *o, Evas_Object_Box_Layout cb, const void *data, void (*free_data)(void *data))
 {
@@ -717,7 +691,7 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static int
-_evas_object_box_layout_horizontal_weight_apply(Evas_Object_Box_Data *priv, Evas_Object_Box_Option **objects, int n_objects, int remaining, int weight_total)
+_evas_object_box_layout_horizontal_weight_apply(Evas_Object_Box_Data *priv, Evas_Object_Box_Option **objects, int n_objects, int remaining, double weight_total)
 {
    int rem_diff = 0;
    int i;
@@ -770,55 +744,6 @@ _evas_object_box_layout_horizontal_weight_apply(Evas_Object_Box_Data *priv, Evas
    return remaining - rem_diff;
 }
 
-/**
- * Layout function which sets the box @a o to a (basic) horizontal
- * box.  @a priv must be the smart data of the box.
- *
- * The object's overall behavior is controlled by its properties,
- * which are set by the evas_object_box_{h,v}_{align,padding}_set()
- * family of functions.  The properties of the elements in the box --
- * set by evas_object_size_hint_{align,padding,weight}_set() functions
- * -- also control the way this function works.
- *
- * \par box's properties:
- * @c align_h controls the horizontal alignment of the child objects
- * relative to the containing box. When set to 0, children are aligned
- * to the left. A value of 1 lets them aligned to the right border.
- * Values in between align them proportionally.  Note that if the size
- * required by the children, which is given by their widths and the @c
- * padding_h property of the box, is bigger than the container width,
- * the children will be displayed out of its bounds.  A negative value
- * of @c align_h makes the box to *justify* its children. The padding
- * between them, in this case, is corrected so that the leftmost one
- * touches the left border and the rightmost one touches the right
- * border (even if they must overlap).  The @c align_v and @c
- * padding_v properties of the box don't contribute to its behaviour
- * when this layout is chosen.
- *
- * \par Child element's properties:
- * @c align_x does not influence the box's behavior.  @c padding_l and
- * @c padding_r sum up to the container's horizontal padding between
- * elements.  The child's @c padding_t, @c padding_b and @c align_y
- * properties apply for padding/positioning relative to the overall
- * height of the box. Finally, there is the @c weight_x property,
- * which, if set to a non-zero value, tells the container that the
- * child width is not pre-defined.  If the container can't accommodate
- * all its children, it sets the widths of the children *with weights*
- * to sizes as small as they can all fit into it.  If the size
- * required by the children is less than the available, the box
- * increases its children's (which have weights) widths as to fit the
- * remaining space.  The @c weight_x property, besides telling the
- * element is resizable, gives a *weight* for the resizing process.
- * The parent box will try to distribute (or take off) widths
- * accordingly to the *normalized* list of weigths: most weighted
- * children remain/get larger in this process than the the least ones.
- * @c weight_y does not influence the layout.
- *
- * If one desires that, besides having weights, child elements must be
- * resized bounded to a minimum or maximum size, their size hint
- * properties must be set (by the
- * evas_object_size_hint_{min,max}_set() functions.
- */
 EAPI void
 evas_object_box_layout_horizontal(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -924,7 +849,7 @@ evas_object_box_layout_horizontal(Evas_Object *o, Evas_Object_Box_Data *priv, vo
 }
 
 static int
-_evas_object_box_layout_vertical_weight_apply(Evas_Object_Box_Data *priv, Evas_Object_Box_Option **objects, int n_objects, int remaining, int weight_total)
+_evas_object_box_layout_vertical_weight_apply(Evas_Object_Box_Data *priv, Evas_Object_Box_Option **objects, int n_objects, int remaining, double weight_total)
 {
    int rem_diff = 0;
    int i;
@@ -977,14 +902,6 @@ _evas_object_box_layout_vertical_weight_apply(Evas_Object_Box_Data *priv, Evas_O
    return remaining - rem_diff;
 }
 
-/**
- * Layout function which sets the box @a o to a (basic) vertical box.
- * @a priv must be the smart data of the box.
- *
- * This function behaves analogously to
- * evas_object_box_layout_horizontal().  The description of its
- * behaviour can be derived from that function's documentation.
- */
 EAPI void
 evas_object_box_layout_vertical(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1090,39 +1007,6 @@ evas_object_box_layout_vertical(Evas_Object *o, Evas_Object_Box_Data *priv, void
    evas_object_size_hint_min_set(o, top_w, req_h);
 }
 
-/**
- * Layout function which sets the box @a o to a *homogeneous*
- * horizontal box.  @a priv must be the smart data of the box.
- *
- * In a homogeneous horizontal box, its width is divided equally
- * between the contained objects.  The box's overall behavior is
- * controlled by its properties, which are set by the
- * evas_object_box_{h,v}_{align,padding}_set() family of functions.
- * The properties of the elements in the box -- set by
- * evas_object_size_hint_{align,padding,weight}_set() functions --
- * also control the way this function works.
- *
- * \par box's properties:
- * @c align_h has no influence on the box for this layout.  @c
- * padding_h tells the box to draw empty spaces of that size, in
- * pixels, between the (still equal) child objects's cells.  The @c
- * align_v and @c padding_v properties of the box don't contribute to
- * its behaviour when this layout is chosen.
- *
- * \par Child element's properties:
- * @c padding_l and @c padding_r sum up to the required width of the
- * child element.  The @c align_x property tells the relative position
- * of this overall child width in its allocated cell (0 to extreme
- * left, 1 to extreme right).  A value of -1.0 to @c align_x makes the
- * box try to resize this child element to the exact width of its cell
- * (respecting the min and max hints on the child's width *and*
- * accounting its horizontal padding properties).  The child's @c
- * padding_t, @c padding_b and @c align_y properties apply for
- * padding/positioning relative to the overall height of the box. A
- * value of -1.0 to @c align_y makes the box try to resize this child
- * element to the exact height of its parent (respecting the max hint
- * on the child's height).
- */
 EAPI void
 evas_object_box_layout_homogeneous_horizontal(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1184,14 +1068,6 @@ evas_object_box_layout_homogeneous_horizontal(Evas_Object *o, Evas_Object_Box_Da
    evas_object_size_hint_min_set(o, w, h);
 }
 
-/**
- * Layout function which sets the box @a o to a *homogeneous* vertical
- * box.  @a priv must be the smart data of the box.
- *
- * This function behaves analogously to
- * evas_object_box_layout_homogeneous_horizontal().  The description
- * of its behaviour can be derived from that function's documentation.
- */
 EAPI void
 evas_object_box_layout_homogeneous_vertical(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1252,47 +1128,6 @@ evas_object_box_layout_homogeneous_vertical(Evas_Object *o, Evas_Object_Box_Data
    evas_object_size_hint_min_set(o, w, h);
 }
 
-/**
- * Layout function which sets the box @a o to a *max size-homogeneous*
- * horizontal box.  @a priv must be the smart data of the box.
- *
- * In a max size-homogeneous horizontal box, the equal sized cells
- * reserved for the child objects have the width of the space required
- * by the largest child (in width). The box's overall behavior is
- * controlled by its properties, which are set by the
- * evas_object_box_{h,v}_{align,padding}_set() family of functions.
- * The properties of the elements in the box -- set by
- * evas_object_size_hint_{align,padding,weight}_set() functions --
- * also control the way this function works.
- *
- * \par box's properties:
- * @c padding_h tells the box to draw empty spaces of that size, in
- * pixels, between the child objects's cells.  @c align_h controls the
- * horizontal alignment of the child objects relative to the
- * containing box. When set to 0, children are aligned to the left. A
- * value of 1 lets them aligned to the right border.  Values in
- * between align them proportionally. A negative value of @c align_h
- * makes the box to *justify* its children cells. The padding between
- * them, in this case, is corrected so that the leftmost one touches
- * the left border and the rightmost one touches the right border
- * (even if they must overlap).  The @c align_v and @c padding_v
- * properties of the box don't contribute to its behaviour when this
- * layout is chosen.
- *
- * \par Child element's properties:
- * @c padding_l and @c padding_r sum up to the required width of the
- * child element. The @c align_x property tells the relative position
- * of this overall child width in its allocated cell (0 to extreme
- * left, 1 to extreme right).  A value of -1.0 to @c align_x makes the
- * box try to resize this child element to the exact width of its cell
- * (respecting the min and max hints on the child's width *and*
- * accounting its horizontal padding properties).  The child's @c
- * padding_t, @c padding_b and @c align_y properties apply for
- * padding/positioning relative to the overall height of the box. A
- * value of -1.0 to @c align_y makes the box try to resize this child
- * element to the exact height of its parent (respecting the max hint
- * on the child's height).
- */
 EAPI void
 evas_object_box_layout_homogeneous_max_size_horizontal(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1377,15 +1212,6 @@ evas_object_box_layout_homogeneous_max_size_horizontal(Evas_Object *o, Evas_Obje
    evas_object_size_hint_min_set(o, x, top_h);
 }
 
-/**
- * Layout function which sets the box @a o to a *max size-homogeneous*
- * vertical box.  @a priv must be the smart data of the box.
- *
- * This function behaves analogously to
- * evas_object_box_layout_homogeneous_max_size_horizontal().  The
- * description of its behaviour can be derived from that function's
- * documentation.
- */
 EAPI void
 evas_object_box_layout_homogeneous_max_size_vertical(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1534,45 +1360,10 @@ _evas_object_box_layout_flow_horizontal_row_info_collect(Evas_Object_Box_Data *p
    *max_h_ret = max_h;
 }
 
-/**
- * Layout function which sets the box @a o to a *flow* horizontal box.
- * @a priv must be the smart data of the box.
- *
- * In a flow horizontal box, the box's child elements are placed in
- * rows (think of text as an analogy). A row has as much elements as
- * can fit into the box's width.  The box's overall behavior is
- * controlled by its properties, which are set by the
- * evas_object_box_{h,v}_{align,padding}_set() family of functions.
- * The properties of the elements in the box -- set by
- * evas_object_size_hint_{align,padding,weight}_set() functions --
- * also control the way this function works.
- *
- * \par box's properties:
- * @c padding_h tells the box to draw empty spaces of that size, in
- * pixels, between the child objects's cells.  @c align_h dictates the
- * horizontal alignment of the rows (0 to left align them, 1 to right
- * align).  A value of -1.0 to @c align_h lets the rows *justified*
- * horizontally.  @c align_v controls the vertical alignment of the
- * entire set of rows (0 to top, 1 to bottom).  A value of -1.0 to @c
- * align_v makes the box to *justify* the rows vertically. The padding
- * between them, in this case, is corrected so that the first row
- * touches the top border and the last one touches the bottom border
- * (even if they must overlap). @c padding_v has no influence on the
- * layout.
- *
- * \par Child element's properties:
- * @c padding_l and @c padding_r sum up to the required width of the
- * child element.  The @c align_x property has no influence on the
- * layout. The child's @c padding_t and @c padding_b sum up to the
- * required height of the child element and is the only means (besides
- * row justifying) of setting space between rows.  Note, however, that
- * @c align_y dictates positioning relative to the *largest height*
- * required by a child object in the actual row.
- */
 EAPI void
 evas_object_box_layout_flow_horizontal(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
-   int n_children, v_justify;
+   int n_children;
    int r, row_count = 0;
    int min_w = 0, min_h = 0;
    int max_h, inc_y;
@@ -1582,7 +1373,7 @@ evas_object_box_layout_flow_horizontal(Evas_Object *o, Evas_Object_Box_Data *pri
    int *row_max_h;
    int *row_break;
    int *row_width;
-   int off_y;
+   int offset_y;
 
    n_children = eina_list_count(priv->children);
    if (!n_children)
@@ -1604,11 +1395,10 @@ evas_object_box_layout_flow_horizontal(Evas_Object *o, Evas_Object_Box_Data *pri
    evas_object_geometry_get(o, &x, &y, &w, &h);
 
    _evas_object_box_layout_flow_horizontal_row_info_collect
-     (priv, w, &row_count, row_max_h, row_break, row_width, &off_y, &max_h);
+     (priv, w, &row_count, row_max_h, row_break, row_width, &offset_y, &max_h);
 
    inc_y = 0;
-   v_justify = 0;
-   remain_y = h - (off_y + max_h);
+   remain_y = h - (offset_y + max_h);
 
    if (remain_y > 0)
      {
@@ -1750,14 +1540,6 @@ _evas_object_box_layout_flow_vertical_col_info_collect(Evas_Object_Box_Data *pri
    *max_w_ret = max_w;
 }
 
-/**
- * Layout function which sets the box @a o to a *flow* vertical box.
- * @a priv must be the smart data of the box.
- *
- * This function behaves analogously to
- * evas_object_box_layout_flow_horizontal().  The description of its
- * behaviour can be derived from that function's documentation.
- */
 EAPI void
 evas_object_box_layout_flow_vertical(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1771,7 +1553,7 @@ evas_object_box_layout_flow_vertical(Evas_Object *o, Evas_Object_Box_Data *priv,
    int *col_max_w;
    int *col_break;
    int *col_height;
-   int off_x;
+   int offset_x;
 
    n_children = eina_list_count(priv->children);
    if (!n_children)
@@ -1793,10 +1575,10 @@ evas_object_box_layout_flow_vertical(Evas_Object *o, Evas_Object_Box_Data *priv,
    evas_object_geometry_get(o, &x, &y, &w, &h);
 
    _evas_object_box_layout_flow_vertical_col_info_collect
-     (priv, h, &col_count, col_max_w, col_break, col_height, &off_x, &max_w);
+     (priv, h, &col_count, col_max_w, col_break, col_height, &offset_x, &max_w);
 
    inc_x = 0;
-   remain_x = w - (off_x + max_w);
+   remain_x = w - (offset_x + max_w);
 
    if (remain_x > 0)
      {
@@ -1873,27 +1655,6 @@ evas_object_box_layout_flow_vertical(Evas_Object *o, Evas_Object_Box_Data *priv,
    evas_object_size_hint_min_set(o, min_w, min_h);
 }
 
-/**
- * Layout function which sets the box @a o to set all children to the
- * size of the object.  @a priv must be the smart data of the box.
- *
- * In a stack box, all children will be given the same size and they
- * will be stacked on above the other, so the first object will be the
- * bottom most.
- *
- * \par box's properties:
- * No box option is used.
- *
- * \par Child  element's   properties:
- * @c padding_l and @c padding_r sum up to the required width of the
- * child element.  The @c align_x property tells the relative position
- * of this overall child width in its allocated cell (0 to extreme
- * left, 1 to extreme right).  A value of -1.0 to @c align_x makes the
- * box try to resize this child element to the exact width of its cell
- * (respecting the min and max hints on the child's width *and*
- * accounting its horizontal padding properties).  Same applies to
- * vertical axis.
- */
 EAPI void
 evas_object_box_layout_stack(Evas_Object *o, Evas_Object_Box_Data *priv, void *data __UNUSED__)
 {
@@ -1942,9 +1703,6 @@ evas_object_box_layout_stack(Evas_Object *o, Evas_Object_Box_Data *priv, void *d
    evas_object_size_hint_min_set(o, top_w, top_h);
 }
 
-/**
- * Set the alignment of the whole bounding box of contents.
- */
 EAPI void
 evas_object_box_align_set(Evas_Object *o, double horizontal, double vertical)
 {
@@ -1956,9 +1714,6 @@ evas_object_box_align_set(Evas_Object *o, double horizontal, double vertical)
    evas_object_smart_changed(o);
 }
 
-/**
- * Get alignment of the whole bounding box of contents.
- */
 EAPI void
 evas_object_box_align_get(const Evas_Object *o, double *horizontal, double *vertical)
 {
@@ -1975,9 +1730,6 @@ evas_object_box_align_get(const Evas_Object *o, double *horizontal, double *vert
      }
 }
 
-/**
- * Set the space (padding) between cells.
- */
 EAPI void
 evas_object_box_padding_set(Evas_Object *o, Evas_Coord horizontal, Evas_Coord vertical)
 {
@@ -1989,9 +1741,6 @@ evas_object_box_padding_set(Evas_Object *o, Evas_Coord horizontal, Evas_Coord ve
    evas_object_smart_changed(o);
 }
 
-/**
- * Get the (space) padding between cells.
- */
 EAPI void
 evas_object_box_padding_get(const Evas_Object *o, Evas_Coord *horizontal, Evas_Coord *vertical)
 {
@@ -2008,10 +1757,6 @@ evas_object_box_padding_get(const Evas_Object *o, Evas_Coord *horizontal, Evas_C
      }
 }
 
-/**
- * Append a new object @a child to the box @a o. On error, @c NULL is
- * returned.
- */
 EAPI Evas_Object_Box_Option *
 evas_object_box_append(Evas_Object *o, Evas_Object *child)
 {
@@ -2038,10 +1783,6 @@ evas_object_box_append(Evas_Object *o, Evas_Object *child)
    return NULL;
 }
 
-/**
- * Prepend a new object @a child to the box @a o. On error, @c NULL is
- * returned.
- */
 EAPI Evas_Object_Box_Option *
 evas_object_box_prepend(Evas_Object *o, Evas_Object *child)
 {
@@ -2068,11 +1809,6 @@ evas_object_box_prepend(Evas_Object *o, Evas_Object *child)
    return NULL;
 }
 
-/**
- * Prepend a new object @a child to the box @c o relative to element @a
- * reference. If @a reference is not contained in the box or any other
- * error occurs, @c NULL is returned.
- */
 EAPI Evas_Object_Box_Option *
 evas_object_box_insert_before(Evas_Object *o, Evas_Object *child, const Evas_Object *reference)
 {
@@ -2099,11 +1835,6 @@ evas_object_box_insert_before(Evas_Object *o, Evas_Object *child, const Evas_Obj
    return NULL;
 }
 
-/**
- * Append a new object @a child to the box @c o relative to element @a
- * reference. If @a reference is not contained in the box or any other
- * error occurs, @c NULL is returend.
- */
 EAPI Evas_Object_Box_Option *
 evas_object_box_insert_after(Evas_Object *o, Evas_Object *child, const Evas_Object *reference)
 {
@@ -2130,10 +1861,6 @@ evas_object_box_insert_after(Evas_Object *o, Evas_Object *child, const Evas_Obje
    return NULL;
 }
 
-/**
- * Insert a new object @a child to the box @a o at position @a pos. On
- * error, @c NULL is returned.
- */
 EAPI Evas_Object_Box_Option *
 evas_object_box_insert_at(Evas_Object *o, Evas_Object *child, unsigned int pos)
 {
@@ -2160,10 +1887,6 @@ evas_object_box_insert_at(Evas_Object *o, Evas_Object *child, unsigned int pos)
    return NULL;
 }
 
-/**
- * Remove an object @a child from the box @a o. On error, @c 0 is
- * returned.
- */
 EAPI Eina_Bool
 evas_object_box_remove(Evas_Object *o, Evas_Object *child)
 {
@@ -2190,10 +1913,6 @@ evas_object_box_remove(Evas_Object *o, Evas_Object *child)
    return EINA_FALSE;
 }
 
-/**
- * Remove an object from the box @a o which occupies position @a
- * pos. On error, @c 0 is returned.
- */
 EAPI Eina_Bool
 evas_object_box_remove_at(Evas_Object *o, unsigned int pos)
 {
@@ -2217,10 +1936,6 @@ evas_object_box_remove_at(Evas_Object *o, unsigned int pos)
    return EINA_FALSE;
 }
 
-/**
- * Remove all child objects.
- * @return 0 on errors
- */
 EAPI Eina_Bool
 evas_object_box_remove_all(Evas_Object *o, Eina_Bool clear)
 {
@@ -2252,11 +1967,6 @@ evas_object_box_remove_all(Evas_Object *o, Eina_Bool clear)
    return EINA_TRUE;
 }
 
-/**
- * Get an iterator to walk the list of children for the box.
- *
- * @note Do not remove or delete objects while walking the list.
- */
 EAPI Eina_Iterator *
 evas_object_box_iterator_new(const Evas_Object *o)
 {
@@ -2281,11 +1991,6 @@ evas_object_box_iterator_new(const Evas_Object *o)
    return &it->iterator;
 }
 
-/**
- * Get an accessor to get random access to the list of children for the box.
- *
- * @note Do not remove or delete objects while walking the list.
- */
 EAPI Eina_Accessor *
 evas_object_box_accessor_new(const Evas_Object *o)
 {
@@ -2310,14 +2015,6 @@ evas_object_box_accessor_new(const Evas_Object *o)
    return &it->accessor;
 }
 
-/**
- * Get the list of children for the box.
- *
- * @note This is a duplicate of the list kept by the box internally.
- *       It's up to the user to destroy it when it no longer needs it.
- *       It's possible to remove objects from the box when walking this
- *       list, but these removals won't be reflected on it.
- */
 EAPI Eina_List *
 evas_object_box_children_get(const Evas_Object *o)
 {
@@ -2332,10 +2029,6 @@ evas_object_box_children_get(const Evas_Object *o)
    return new_list;
 }
 
-/**
- * Get the name of the property of the child elements of the box @a o
- * whose id is @a property. On error, @c NULL is returned.
- */
 EAPI const char *
 evas_object_box_option_property_name_get(Evas_Object *o, int property)
 {
@@ -2353,10 +2046,6 @@ evas_object_box_option_property_name_get(Evas_Object *o, int property)
    return api->property_name_get(o, property);
 }
 
-/**
- * Get the id of the property of the child elements of the box @a o
- * whose name is @a name. On error, @c -1 is returned.
- */
 EAPI int
 evas_object_box_option_property_id_get(Evas_Object *o, const char *name)
 {
@@ -2374,12 +2063,6 @@ evas_object_box_option_property_id_get(Evas_Object *o, const char *name)
    return api->property_id_get(o, name);
 }
 
-/**
- * Set the property (with id @a property) of the child element of the
- * box @a o whose property struct is @a opt. The property's values
- * must be the last arguments and their type *must* match that of the
- * property itself. On error, @c 0 is returned.
- */
 EAPI Eina_Bool
 evas_object_box_option_property_set(Evas_Object *o, Evas_Object_Box_Option *opt, int property, ...)
 {
@@ -2393,13 +2076,6 @@ evas_object_box_option_property_set(Evas_Object *o, Evas_Object_Box_Option *opt,
    return ret;
 }
 
-/**
- * Set the property (with id @a property) of the child element of the
- * box @a o whose property struct is @a opt. The property's values
- * must be the args which the va_list @a args is initialized with and
- * their type *must* match that of the property itself. On error, @c 0
- * is returned.
- */
 
 EAPI Eina_Bool
 evas_object_box_option_property_vset(Evas_Object *o, Evas_Object_Box_Option *opt, int property, va_list args)
@@ -2421,12 +2097,6 @@ evas_object_box_option_property_vset(Evas_Object *o, Evas_Object_Box_Option *opt
    return EINA_TRUE;
 }
 
-/**
- * Get the property (with id @a property) of the child element of the
- * box @a o whose property struct is @a opt. The last arguments must
- * be addresses of variables with the same type of that property. On
- * error, @c 0 is returned.
- */
 EAPI Eina_Bool
 evas_object_box_option_property_get(Evas_Object *o, Evas_Object_Box_Option *opt, int property, ...)
 {
@@ -2440,12 +2110,6 @@ evas_object_box_option_property_get(Evas_Object *o, Evas_Object_Box_Option *opt,
    return ret;
 }
 
-/**
- * Get the property (with id @a property) of the child element of the
- * box @a o whose property struct is @a opt. The args which the
- * va_list @a args is initialized with must be addresses of variables
- * with the same type of that property. On error, @c 0 is returned.
- */
 EAPI Eina_Bool
 evas_object_box_option_property_vget(Evas_Object *o, Evas_Object_Box_Option *opt, int property, va_list args)
 {
@@ -2461,7 +2125,3 @@ evas_object_box_option_property_vget(Evas_Object *o, Evas_Object_Box_Option *opt
 
    return api->property_get(o, opt, property, args);
 }
-
-/**
- * @}
- */
