@@ -12,10 +12,10 @@ struct _Evas_Object_Line
    DATA32            magic;
    struct {
       struct {
-	 int         x1, y1, x2, y2;
-	 struct {
-	    Evas_Coord w, h;
-	 } object;
+         int         x1, y1, x2, y2;
+         struct {
+            Evas_Coord w, h;
+         } object;
       } cache;
       Evas_Coord         x1, y1, x2, y2;
    } cur, prev;
@@ -72,19 +72,8 @@ static const Evas_Object_Func object_func =
 /* the actual api call to add a rect */
 /* it has no other api calls as all properties are standard */
 
-
-/**
- * @addtogroup Evas_Line_Group
- * @{
- */
-
 EVAS_MEMPOOL(_mp_obj);
 
-/**
- * Adds a new evas line object to the given evas.
- * @param   e The given evas.
- * @return  The new evas line object.
- */
 EAPI Evas_Object *
 evas_object_line_add(Evas *e)
 {
@@ -99,14 +88,6 @@ evas_object_line_add(Evas *e)
    return obj;
 }
 
-/**
- * Sets the coordinates of the end points of the given evas line object.
- * @param   obj The given evas line object.
- * @param   x1  The X coordinate of the first point.
- * @param   y1  The Y coordinate of the first point.
- * @param   x2  The X coordinate of the second point.
- * @param   y2  The Y coordinate of the second point.
- */
 EAPI void
 evas_object_line_xy_set(Evas_Object *obj, Evas_Coord x1, Evas_Coord y1, Evas_Coord x2, Evas_Coord y2)
 {
@@ -125,30 +106,32 @@ evas_object_line_xy_set(Evas_Object *obj, Evas_Coord x1, Evas_Coord y1, Evas_Coo
        (x2 == o->cur.x2) && (y2 == o->cur.y2)) return;
    if (obj->layer->evas->events_frozen <= 0)
      {
-	if (!evas_event_passes_through(obj))
-	  was = evas_object_is_in_output_rect(obj,
-					      obj->layer->evas->pointer.x,
-					      obj->layer->evas->pointer.y, 1, 1);
+        if (!evas_event_passes_through(obj) &&
+            !evas_event_freezes_through(obj))
+          was = evas_object_is_in_output_rect(obj,
+                                              obj->layer->evas->pointer.x,
+                                              obj->layer->evas->pointer.y,
+                                              1, 1);
      }
    if (x1 < x2)
      {
-	min_x = x1;
-	max_x = x2;
+        min_x = x1;
+        max_x = x2;
      }
    else
      {
-	min_x = x2;
-	max_x = x1;
+        min_x = x2;
+        max_x = x1;
      }
    if (y1 < y2)
      {
-	min_y = y1;
-	max_y = y2;
+        min_y = y1;
+        max_y = y2;
      }
    else
      {
-	min_y = y2;
-	max_y = y1;
+        min_y = y2;
+        max_y = y1;
      }
    obj->cur.geometry.x = min_x;
    obj->cur.geometry.y = min_y;
@@ -165,35 +148,24 @@ evas_object_line_xy_set(Evas_Object *obj, Evas_Coord x1, Evas_Coord y1, Evas_Coo
    evas_object_clip_dirty(obj);
    if (obj->layer->evas->events_frozen <= 0)
      {
-	is = evas_object_is_in_output_rect(obj,
-					   obj->layer->evas->pointer.x,
-					   obj->layer->evas->pointer.y, 1, 1);
-	if (!evas_event_passes_through(obj))
-	  {
-	     if ((is ^ was) && obj->cur.visible)
-	       evas_event_feed_mouse_move(obj->layer->evas,
-					  obj->layer->evas->pointer.x,
-					  obj->layer->evas->pointer.y,
-					  obj->layer->evas->last_timestamp,
-					  NULL);
-	  }
+        is = evas_object_is_in_output_rect(obj,
+                                           obj->layer->evas->pointer.x,
+                                           obj->layer->evas->pointer.y, 1, 1);
+        if (!evas_event_passes_through(obj) &&
+            !evas_event_freezes_through(obj))
+          {
+             if ((is ^ was) && obj->cur.visible)
+               evas_event_feed_mouse_move(obj->layer->evas,
+                                          obj->layer->evas->pointer.x,
+                                          obj->layer->evas->pointer.y,
+                                          obj->layer->evas->last_timestamp,
+                                          NULL);
+          }
      }
    evas_object_inform_call_move(obj);
    evas_object_inform_call_resize(obj);
 }
 
-/**
- * Retrieves the coordinates of the end points of the given evas line object.
- * @param obj The given line object.
- * @param x1  Pointer to an integer in which to store the X coordinate of the
- *            first end point.
- * @param y1  Pointer to an integer in which to store the Y coordinate of the
- *            first end point.
- * @param x2  Pointer to an integer in which to store the X coordinate of the
- *            second end point.
- * @param y2  Pointer to an integer in which to store the Y coordinate of the
- *            second end point.
- */
 EAPI void
 evas_object_line_xy_get(const Evas_Object *obj, Evas_Coord *x1, Evas_Coord *y1, Evas_Coord *x2, Evas_Coord *y2)
 {
@@ -220,10 +192,6 @@ evas_object_line_xy_get(const Evas_Object *obj, Evas_Coord *x1, Evas_Coord *y1,
    if (y2) *y2 = obj->cur.geometry.y + o->cur.y2;
 }
 
-/**
- * @}
- */
-
 /* all nice and private */
 static void
 evas_object_line_init(Evas_Object *obj)
@@ -280,7 +248,7 @@ evas_object_line_free(Evas_Object *obj)
    MAGIC_CHECK_END();
    /* free obj */
    o->magic = 0;
-   free(o);
+   EVAS_MEMPOOL_FREE(_mp_obj, o);
 }
 
 static void
