@@ -4,6 +4,8 @@
 /* private magic number for rectangle objects */
 static const char o_type[] = "rectangle";
 
+const char *o_rect_type = o_type;
+
 /* private struct for rectangle object internal data */
 typedef struct _Evas_Object_Rectangle      Evas_Object_Rectangle;
 
@@ -66,19 +68,8 @@ static const Evas_Object_Func object_func =
 /* the actual api call to add a rect */
 /* it has no other api calls as all properties are standard */
 
-
-/**
- * @addtogroup Evas_Object_Rectangle
- * @{
- */
-
 EVAS_MEMPOOL(_mp_obj);
 
-/**
- * Adds a rectangle to the given evas.
- * @param   e The given evas.
- * @return  The new rectangle object.
- */
 EAPI Evas_Object *
 evas_object_rectangle_add(Evas *e)
 {
@@ -93,10 +84,6 @@ evas_object_rectangle_add(Evas *e)
    return obj;
 }
 
-/**
- * @}
- */
-
 /* all nice and private */
 static void
 evas_object_rectangle_init(Evas_Object *obj)
@@ -201,8 +188,7 @@ evas_object_rectangle_render_pre(Evas_Object *obj)
    /* if it just became visible or invisible */
    is_v = evas_object_is_visible(obj);
    was_v = evas_object_was_visible(obj);
-/* POST 1.0 -> enable */   
-/*   if (!(is_v | was_v)) goto done;*/
+   if (!(is_v | was_v)) goto done;
    if (is_v != was_v)
      {
 	evas_object_render_pre_visible_change(&obj->layer->evas->clip_changes, obj, is_v, was_v);
