@@ -1,9 +1,11 @@
 #ifndef _EVAS_FONT_PRIVATE_H
 # define _EVAS_FONT_PRIVATE_H
+#include "evas_font_ot.h"
 
 #ifdef BUILD_PTHREAD
 extern LK(lock_font_draw); // for freetype2 API calls
 extern LK(lock_bidi); // for fribidi API calls
+extern LK(lock_ot); // for harfbuzz calls
 #endif
 
 # if defined(EVAS_FRAME_QUEUING) || defined(BUILD_PIPE_RENDER)
@@ -12,12 +14,19 @@ extern LK(lock_bidi); // for fribidi API calls
 
 #  define BIDILOCK() LKL(lock_bidi)
 #  define BIDIUNLOCK() LKU(lock_bidi)
-# else
-#  define FTLOCK(x) 
-#  define FTUNLOCK(x) 
 
-#  define BIDILOCK() 
-#  define BIDIUNLOCK() 
+#  define OTLOCK() LKL(lock_ot)
+#  define OTUNLOCK() LKU(lock_ot)
+# else
+#  define FTLOCK(x)
+#  define FTUNLOCK(x)
+
+#  define BIDILOCK()
+#  define BIDIUNLOCK()
+
+/* Macros for text walking */
+#  define OTLOCK()
+#  define OTUNLOCK()
 # endif
 
 void evas_common_font_source_unload(RGBA_Font_Source *fs);
@@ -29,4 +38,15 @@ void evas_common_font_int_use_trim(void);
 void evas_common_font_int_unload(RGBA_Font_Int *fi);
 void evas_common_font_int_reload(RGBA_Font_Int *fi);
 
+/* 6th bit is on is the same as frac part >= 0.5 */
+# define EVAS_FONT_ROUND_26_6_TO_INT(x) \
+   (((x + 0x20) & -0x40) >> 6)
+
+# define EVAS_FONT_CHARACTER_IS_INVISIBLE(x) ( \
+      ((0x200C <= (x)) && ((x) <= 0x200D)) || /* ZWNJ..ZWH */ \
+      ((0x200E <= (x)) && ((x) <= 0x200F)) || /* BIDI stuff */ \
+      ((0x202A <= (x)) && ((x) <= 0x202E)) /* BIDI stuff */ \
+      )
+
+# include "evas_font_default_walk.x"
 #endif /* !_EVAS_FONT_PRIVATE_H */
