@@ -1,40 +1,130 @@
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include "evas_common.h"
 #include "evas_private.h"
 #include "evas_cs.h"
 
 struct ext_loader_s
 {
+   unsigned int length;
    const char *extension;
    const char *loader;
 };
 
+#define MATCHING(Ext, Module)                   \
+  { sizeof (Ext), Ext, Module }
+
 static const struct ext_loader_s loaders[] =
-{
-   { "png", "png" },
-   { "jpg", "jpeg" },
-   { "jpeg", "jpeg" },
-   { "jfif", "jpeg" },
-   { "eet", "eet" },
-   { "edj", "eet" },
-   { "eap", "eet" },
-   { "edb", "edb" },
-   { "xpm", "xpm" },
-   { "tiff", "tiff" },
-   { "tif", "tiff" },
-   { "svg", "svg" },
-   { "svgz", "svg" },
-   { "gif", "gif" },
-   { "pbm", "pmaps" },
-   { "pgm", "pmaps" },
-   { "ppm", "pmaps" },
-   { "pnm", "pmaps" },
-   { "bmp", "bmp" },
-   { "tga", "tga" }
+{ /* map extensions to loaders to use for good first-guess tries */
+   MATCHING(".png", "png"),
+   MATCHING(".jpg", "jpeg"),
+   MATCHING(".jpeg", "jpeg"),
+   MATCHING(".jfif", "jpeg"),
+   MATCHING(".eet", "eet"),
+   MATCHING(".edj", "eet"),
+   MATCHING(".eap", "eet"),
+   MATCHING(".edb", "edb"),
+   MATCHING(".xpm", "xpm"),
+   MATCHING(".tiff", "tiff"),
+   MATCHING(".tif", "tiff"),
+   MATCHING(".svg", "svg"),
+   MATCHING(".svgz", "svg"),
+   MATCHING(".svg.gz", "svg"),
+   MATCHING(".gif", "gif"),
+   MATCHING(".pbm", "pmaps"),
+   MATCHING(".pgm", "pmaps"),
+   MATCHING(".ppm", "pmaps"),
+   MATCHING(".pnm", "pmaps"),
+   MATCHING(".bmp", "bmp"),
+   MATCHING(".tga", "tga"),
+   MATCHING(".wbmp", "wbmp"),
+   MATCHING(".ico", "ico"),
+   MATCHING(".cur", "ico"),
+   MATCHING(".psd", "psd"),
+   MATCHING(".pdf", "generic"),
+   MATCHING(".ps", "generic"),
+   MATCHING(".xcf", "generic"),
+   MATCHING(".xcf.gz", "generic"),
+   /* RAW */
+   MATCHING(".arw", "generic"),
+   MATCHING(".cr2", "generic"),
+   MATCHING(".crw", "generic"),
+   MATCHING(".dcr", "generic"),
+   MATCHING(".dng", "generic"),
+   MATCHING(".k25", "generic"),
+   MATCHING(".kdc", "generic"),
+   MATCHING(".erf", "generic"),
+   MATCHING(".mrw", "generic"),
+   MATCHING(".nef", "generic"),
+   MATCHING(".nrf", "generic"),
+   MATCHING(".nrw", "generic"),
+   MATCHING(".orf", "generic"),
+   MATCHING(".raw", "generic"),
+   MATCHING(".rw2", "generic"),
+   MATCHING(".pef", "generic"),
+   MATCHING(".raf", "generic"),
+   MATCHING(".sr2", "generic"),
+   MATCHING(".srf", "generic"),
+   MATCHING(".x3f", "generic"),
+   /* video */
+   MATCHING(".264", "generic"),
+   MATCHING(".3g2", "generic"),
+   MATCHING(".3gp", "generic"),
+   MATCHING(".3gp2", "generic"),
+   MATCHING(".3gpp", "generic"),
+   MATCHING(".3gpp2", "generic"),
+   MATCHING(".3p2", "generic"),
+   MATCHING(".asf", "generic"),
+   MATCHING(".avi", "generic"),
+   MATCHING(".bdm", "generic"),
+   MATCHING(".bdmv", "generic"),
+   MATCHING(".clpi", "generic"),
+   MATCHING(".clp", "generic"),
+   MATCHING(".fla", "generic"),
+   MATCHING(".flv", "generic"),
+   MATCHING(".m1v", "generic"),
+   MATCHING(".m2v", "generic"),
+   MATCHING(".m2t", "generic"),
+   MATCHING(".m4v", "generic"),
+   MATCHING(".mkv", "generic"),
+   MATCHING(".mov", "generic"),
+   MATCHING(".mp2", "generic"),
+   MATCHING(".mp2ts", "generic"),
+   MATCHING(".mp4", "generic"),
+   MATCHING(".mpe", "generic"),
+   MATCHING(".mpeg", "generic"),
+   MATCHING(".mpg", "generic"),
+   MATCHING(".mpl", "generic"),
+   MATCHING(".mpls", "generic"),
+   MATCHING(".mts", "generic"),
+   MATCHING(".mxf", "generic"),
+   MATCHING(".nut", "generic"),
+   MATCHING(".nuv", "generic"),
+   MATCHING(".ogg", "generic"),
+   MATCHING(".ogm", "generic"),
+   MATCHING(".ogv", "generic"),
+   MATCHING(".rm", "generic"),
+   MATCHING(".rmj", "generic"),
+   MATCHING(".rmm", "generic"),
+   MATCHING(".rms", "generic"),
+   MATCHING(".rmx", "generic"),
+   MATCHING(".rmvb", "generic"),
+   MATCHING(".swf", "generic"),
+   MATCHING(".ts", "generic"),
+   MATCHING(".weba", "generic"),
+   MATCHING(".webm", "generic"),
+   MATCHING(".wmv", "generic")
 };
 
 static const char *loaders_name[] =
-{
-  "png", "jpeg", "eet", "xpm", "tiff", "gif", "svg", "pmaps", "edb", "bmp", "tga"
+{ /* in order of most likely needed */
+  "png", "jpeg", "eet", "xpm", "tiff", "gif", "svg", "pmaps", "bmp", "tga", "wbmp", "ico", "psd", "edb", "generic"
 };
 
 struct evas_image_foreach_loader_data
@@ -72,11 +162,11 @@ EAPI int
 evas_common_load_rgba_image_module_from_file(Image_Entry *ie)
 {
    Evas_Image_Load_Func *evas_image_load_func = NULL;
-   const char           *loader = NULL;
+   const char           *loader = NULL, *end;
    Evas_Module          *em;
-   char                 *dot;
+   struct stat		 st;
    unsigned int          i;
-   int                   ret = EVAS_LOAD_ERROR_NONE;
+   int                   len, ret = EVAS_LOAD_ERROR_NONE;
    struct evas_image_foreach_loader_data fdata;
 
 
@@ -92,20 +182,26 @@ evas_common_load_rgba_image_module_from_file(Image_Entry *ie)
              return EVAS_LOAD_ERROR_NONE;
           }
      }
-#endif   
-   dot = strrchr (ie->file, '.');
-   if (dot)
+#endif
+   if (stat(ie->file, &st) != 0 || S_ISDIR(st.st_mode))
      {
-	for (i = 0, ++dot; i < (sizeof (loaders) / sizeof (struct ext_loader_s)); ++i)
-	  {
-	     if (!strcasecmp(dot, loaders[i].extension))
-	       {
-		  loader = loaders[i].loader;
-		  DBG("known loader '%s' handles extension '%s' of file '%s'",
-		      loader, dot, ie->file);
-		  break;
-	       }
-	  }
+        DBG("trying to open directory '%s' !", ie->file);
+        return EVAS_LOAD_ERROR_DOES_NOT_EXIST;
+     }
+
+   len = strlen(ie->file);
+   end = ie->file + len;
+   for (i = 0; i < (sizeof (loaders) / sizeof(struct ext_loader_s)); i++)
+     {
+        int len2 = strlen(loaders[i].extension);
+        if (len2 > len) continue;
+        if (!strcasecmp(end - len2, loaders[i].extension))
+          {
+             loader = loaders[i].loader;
+             DBG("known loader '%s' handles extension '%s' of file '%s'",
+                 loader, end - len2, ie->file);
+             break;
+          }
      }
 
    if (loader)
@@ -126,7 +222,7 @@ evas_common_load_rgba_image_module_from_file(Image_Entry *ie)
 		       goto end;
 		    }
 		  evas_module_unload(em);
-		  DBG("failed to load file head using module '%s' (%p): "
+		  INF("failed to load file head using module '%s' (%p): "
 		      "%s (%s)",
 		      loader, em, ie->file, evas_load_error_str(ret));
 	       }
@@ -134,7 +230,7 @@ evas_common_load_rgba_image_module_from_file(Image_Entry *ie)
 	       WRN("failed to load module '%s' (%p)", loader, em);
 	  }
 	else
-	  DBG("image loader '%s' is not enabled or missing!", loader);
+	  INF("image loader '%s' is not enabled or missing!", loader);
      }
 
    fdata.ie = ie;
@@ -160,25 +256,25 @@ evas_common_load_rgba_image_module_from_file(Image_Entry *ie)
 		  ret = EVAS_LOAD_ERROR_NONE;
 		  if (evas_image_load_func->file_head(ie, ie->file, ie->key, &ret))
 		    {
-		       DBG("brute force loader '%s' (%p) worked on %s",
+                       DBG("brute force loader '%s' (%p) worked on %s",
 			   loaders_name[i], em, ie->file);
 		       goto end;
 		    }
 		  else
-		    DBG("brute force loader '%s' (%p) failed on %s (%s)",
+		    INF("brute force loader '%s' (%p) failed on %s (%s)",
 			loaders_name[i], em, ie->file,
 			evas_load_error_str(ret));
 
 		  evas_module_unload(em);
 	       }
 	     else
-	       WRN("failed to load module '%s' (%p)", loaders_name[i], em);
+	       INF("failed to load module '%s' (%p)", loaders_name[i], em);
 	  }
 	else
 	  DBG("could not find module '%s'", loaders_name[i]);
      }
 
-   DBG("exhausted all means to load image '%s'", ie->file);
+   INF("exhausted all means to load image '%s'", ie->file);
    return EVAS_LOAD_ERROR_UNKNOWN_FORMAT;
 
    end:
@@ -217,7 +313,7 @@ evas_common_load_rgba_image_data_from_file(Image_Entry *ie)
    Evas_Image_Load_Func *evas_image_load_func = NULL;
    int ret = EVAS_LOAD_ERROR_NONE;
 
-   if (ie->flags.loaded) return EVAS_LOAD_ERROR_GENERIC;
+   if ((ie->flags.loaded) && (!ie->flags.animated)) return EVAS_LOAD_ERROR_GENERIC;
 
 #ifdef EVAS_CSERVE
    if (ie->data1)
@@ -253,3 +349,37 @@ evas_common_load_rgba_image_data_from_file(Image_Entry *ie)
 
    return EVAS_LOAD_ERROR_NONE;
 }
+
+EAPI double
+evas_common_load_rgba_image_frame_duration_from_file(Image_Entry *ie, const int start, const int frame_num)
+{
+   Evas_Image_Load_Func *evas_image_load_func = NULL;
+
+   if (!ie->info.module) return -1;
+
+   evas_image_load_func = ie->info.loader;
+   evas_module_use((Evas_Module*) ie->info.module);
+   if (evas_image_load_func->frame_duration)
+     return evas_image_load_func->frame_duration(ie, ie->file, start, frame_num);
+   return -1;
+}
+
+EAPI Eina_Bool
+evas_common_extension_can_load_get(const char *file)
+{
+   unsigned int length;
+   unsigned int i;
+
+   length = eina_stringshare_strlen(file) + 1;
+   if (length < 5) return EINA_FALSE;
+
+   for (i = 0; i < sizeof (loaders) / sizeof (struct ext_loader_s); ++i)
+     {
+        if (loaders[i].length > length) continue;
+
+        if (!strcasecmp(loaders[i].extension, file + length - loaders[i].length))
+          return EINA_TRUE;
+     }
+
+   return EINA_FALSE;
+}
