@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"  /* so that EAPI in Eet.h is correctly defined */
 #endif
@@ -14,11 +10,14 @@
 #include "evas_private.h"
 #include "evas_image_private.h"
 #include "evas_convert_yuv.h"
+#include "evas_cs.h"
 
 #ifdef HAVE_VALGRIND
 # include <memcheck.h>
 #endif
 
+//#define SURFDBG 1
+
 static Evas_Cache_Image * eci = NULL;
 static int                reference = 0;
 
@@ -36,13 +35,13 @@ static int                reference = 0;
 static Image_Entry      *_evas_common_rgba_image_new(void);
 static void              _evas_common_rgba_image_delete(Image_Entry *ie);
 
-static int               _evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h);
+static int               _evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned int h);
 static void              _evas_common_rgba_image_surface_delete(Image_Entry *ie);
 static DATA32           *_evas_common_rgba_image_surface_pixels(Image_Entry *ie);
 
 static void              _evas_common_rgba_image_unload(Image_Entry *im);
 
-static void              _evas_common_rgba_image_dirty_region(Image_Entry *im, int x, int y, int w, int h);
+static void              _evas_common_rgba_image_dirty_region(Image_Entry *im, unsigned int x, unsigned int y, unsigned int w, unsigned int h);
 
 static int               _evas_common_rgba_image_ram_usage(Image_Entry *ie);
 
@@ -50,11 +49,13 @@ static int               _evas_common_rgba_image_ram_usage(Image_Entry *ie);
 /* The destination surface does have a surface, but no allocated pixel data. */
 static int               _evas_common_rgba_image_dirty(Image_Entry* dst, const Image_Entry* src);
 
+#if 0
 static void
 _evas_common_rgba_image_debug(const char* context, Image_Entry *eim)
 {
   DBG("%p = [%s] {%s,%s} %i [%i|%i]", eim, context, eim->file, eim->key, eim->references, eim->w, eim->h);
 }
+#endif
 
 static const Evas_Cache_Image_Func      _evas_common_image_func =
 {
@@ -122,6 +123,13 @@ evas_common_image_shutdown(void)
    evas_common_scalecache_shutdown();
 }
 
+EAPI void
+evas_common_image_image_all_unload(void)
+{
+   evas_common_rgba_image_scalecache_dump();
+   evas_cache_image_unload_all(eci);
+}
+
 static Image_Entry *
 _evas_common_rgba_image_new(void)
 {
@@ -131,18 +139,31 @@ _evas_common_rgba_image_new(void)
    if (!im) return NULL;
    im->flags = RGBA_IMAGE_NOTHING;
    im->ref = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKI(im->cache_entry.ref_fq_add);
+   LKI(im->cache_entry.ref_fq_del);
+   eina_condition_new(&(im->cache_entry.cond_fq_del),
+		      &(im->cache_entry.ref_fq_del));
+#endif
+
    evas_common_rgba_image_scalecache_init(&im->cache_entry);
+
    return &im->cache_entry;
 }
 
 static void
 _evas_common_rgba_image_delete(Image_Entry *ie)
 {
-   RGBA_Image   *im = (RGBA_Image *) ie;
+   RGBA_Image *im = (RGBA_Image *)ie;
 
 #ifdef BUILD_PIPE_RENDER
    evas_common_pipe_free(im);
-#endif   
+# ifdef EVAS_FRAME_QUEUING
+   LKD(im->cache_entry.ref_fq_add);
+   LKD(im->cache_entry.ref_fq_del);
+   eina_condition_free(&(im->cache_entry.cond_fq_del));
+# endif
+#endif
    evas_common_rgba_image_scalecache_shutdown(&im->cache_entry);
    if (ie->info.module) evas_module_unref((Evas_Module *)ie->info.module);
    /* memset the image to 0x99 because i recently saw a segv where an
@@ -154,7 +175,35 @@ _evas_common_rgba_image_delete(Image_Entry *ie)
 //   memset(im, 0x99, sizeof(im));
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
-#endif   
+#endif
+/*
+ * FIXME: This doesn't seem to be needed... But I'm not sure why.
+ *	 -- nash
+     {
+        Filtered_Image *fi;
+
+        EINA_LIST_FREE(im->filtered, fi)
+          {
+             free(fi->key);
+             _evas_common_rgba_image_delete((Image_Entry *)(fi->image));
+             free(fi);
+          }
+     }
+*/
+   if (ie->frames)
+     {
+        Eina_List *l;
+        Image_Entry_Frame *frame;
+        EINA_LIST_FOREACH(ie->frames, l, frame)
+          {
+           if (frame)
+             {
+                if (frame->data) free(frame->data);
+                if (frame->info) free(frame->info);
+                free (frame);
+             }
+          }
+     }
    free(im);
 }
 
@@ -165,13 +214,35 @@ evas_common_rgba_image_free(Image_Entry *ie)
    _evas_common_rgba_image_delete(ie);
 }
 
+#ifdef SURFDBG
+static Eina_List *surfs = NULL;
+
+static void
+surf_debug(void)
+{
+   Eina_List *l;
+   Image_Entry *ie;
+   RGBA_Image *im;
+   int i = 0;
+   
+   printf("----SURFS----\n");
+   EINA_LIST_FOREACH(surfs, l, ie)
+     {
+        im = ie;
+        printf("%i - %p - %ix%i  [%s][%s]\n", 
+               i, im->image.data, ie->allocated.w, ie->allocated.h,
+               ie->file, ie->key
+              );
+        i++;
+     }
+}
+#endif
+
 EAPI void
 evas_common_rgba_image_unload(Image_Entry *ie)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
 
-   evas_cache_image_preload_cancel(ie, NULL);
-   
    if (!ie->flags.loaded) return;
    if ((!ie->info.module) && (!ie->data1)) return;
    if (!ie->file) return;
@@ -198,26 +269,82 @@ evas_common_rgba_image_unload(Image_Entry *ie)
         im->image.data = NULL;
         ie->allocated.w = 0;
         ie->allocated.h = 0;
+        ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+        ie->flags.preload_done = 0;
+#endif
         return;
      }
-#endif   
-   
+#endif
+
    if (im->image.data && !im->image.no_free)
-     free(im->image.data);
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
    im->image.data = NULL;
    ie->allocated.w = 0;
    ie->allocated.h = 0;
+   ie->flags.loaded = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+   ie->flags.preload_done = 0;
+#endif
+#ifdef SURFDBG
+   surf_debug();
+#endif   
+}
+
+void
+_evas_common_rgba_image_post_surface(Image_Entry *ie)
+{
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   RGBA_Image *im = (RGBA_Image *)ie;
+
+   if (im->pixman.im) pixman_image_unref(im->pixman.im);
+   if (im->cache_entry.flags.alpha)
+     {
+        im->pixman.im = pixman_image_create_bits
+        (
+// FIXME: endianess determines this
+            PIXMAN_a8r8g8b8,
+//            PIXMAN_b8g8r8a8,
+            im->cache_entry.w, im->cache_entry.h,
+            im->image.data,
+            im->cache_entry.w * 4
+        );
+     }
+   else
+     {
+        im->pixman.im = pixman_image_create_bits
+        (
+// FIXME: endianess determines this
+            PIXMAN_x8r8g8b8,
+//            PIXMAN_b8g8r8x8,
+            im->cache_entry.w, im->cache_entry.h,
+            im->image.data,
+            im->cache_entry.w * 4
+        );
+     }
+# else
+   (void)ie;
+# endif
+#else
+   (void)ie;
+#endif
 }
 
 static int
-_evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h)
+_evas_common_rgba_image_surface_alloc(Image_Entry *ie, unsigned int w, unsigned int h)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
    size_t        siz = 0;
 
 #ifdef EVAS_CSERVE
    if (ie->data1) return 0;
-#endif   
+#endif
    if (im->image.no_free) return 0;
 
    if (im->flags & RGBA_IMAGE_ALPHA_ONLY)
@@ -225,10 +352,20 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h)
    else
      siz = w * h * sizeof(DATA32);
 
-   if (im->image.data) free(im->image.data);
+   if (im->image.data)
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
    im->image.data = malloc(siz);
-   if (im->image.data == NULL) return -1;
-
+   if (!im->image.data) return -1;
+   ie->allocated.w = w;
+   ie->allocated.h = h;
+#ifdef SURFDBG
+   surfs = eina_list_append(surfs, ie);
+#endif   
 #ifdef HAVE_VALGRIND
 # ifdef VALGRIND_MAKE_READABLE
    VALGRIND_MAKE_READABLE(im->image.data, siz);
@@ -238,7 +375,10 @@ _evas_common_rgba_image_surface_alloc(Image_Entry *ie, int w, int h)
 #  endif
 # endif
 #endif
-
+   _evas_common_rgba_image_post_surface(ie);
+#ifdef SURFDBG
+   surf_debug();
+#endif
    return 0;
 }
 
@@ -247,6 +387,17 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
 
+#ifdef HAVE_PIXMAN
+# ifdef PIXMAN_IMAGE   
+   if (im->pixman.im)
+     {
+        pixman_image_unref(im->pixman.im);
+        im->pixman.im = NULL;
+     }
+# endif   
+#endif
+   if (ie->file)
+     DBG("unload: [%p] %s %s", ie, ie->file, ie->key);
    if ((im->cs.data) && (im->image.data))
      {
 	if (im->cs.data != im->image.data)
@@ -261,28 +412,46 @@ _evas_common_rgba_image_surface_delete(Image_Entry *ie)
    im->cs.data = NULL;
 
    if (im->image.data && !im->image.no_free)
-     free(im->image.data);
+     {
+        free(im->image.data);
+#ifdef SURFDBG
+        surfs = eina_list_remove(surfs, ie);
+#endif        
+     }
 #ifdef EVAS_CSERVE
    else if (ie->data1)
      evas_cserve_image_free(ie);
-#endif   
+#endif
+
    im->image.data = NULL;
+   ie->allocated.w = 0;
+   ie->allocated.h = 0;
+#ifdef BUILD_ASYNC_PRELOAD
+   ie->flags.preload_done = 0;
+#endif
+   ie->flags.loaded = 0;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
+#ifdef SURFDBG
+   surf_debug();
+#endif   
 }
 
 static void
-_evas_common_rgba_image_unload(Image_Entry* im __UNUSED__)
+_evas_common_rgba_image_unload(Image_Entry *im)
 {
+//   DBG("unload: [%p] %s %s", im, im->file, im->key);
+   evas_common_rgba_image_scalecache_dirty(im);
+   evas_common_rgba_image_unload(im);
 }
 
 static void
-_evas_common_rgba_image_dirty_region(Image_Entry* ie, int x __UNUSED__, int y __UNUSED__, int w __UNUSED__, int h __UNUSED__)
+_evas_common_rgba_image_dirty_region(Image_Entry* ie, unsigned int x __UNUSED__, unsigned int y __UNUSED__, unsigned int w __UNUSED__, unsigned int h __UNUSED__)
 {
    RGBA_Image   *im = (RGBA_Image *) ie;
 
 #ifdef EVAS_CSERVE
    if (ie->data1) evas_cserve_image_free(ie);
-#endif   
+#endif
    im->flags |= RGBA_IMAGE_IS_DIRTY;
    evas_common_rgba_image_scalecache_dirty(&im->cache_entry);
 }
@@ -294,21 +463,21 @@ _evas_common_rgba_image_dirty(Image_Entry *ie_dst, const Image_Entry *ie_src)
    RGBA_Image   *dst = (RGBA_Image *) ie_dst;
    RGBA_Image   *src = (RGBA_Image *) ie_src;
 
-   evas_common_rgba_image_scalecache_dirty(ie_src);
+   evas_common_rgba_image_scalecache_dirty((Image_Entry *)ie_src);
    evas_common_rgba_image_scalecache_dirty(ie_dst);
    evas_cache_image_load_data(&src->cache_entry);
    if (_evas_common_rgba_image_surface_alloc(&dst->cache_entry,
                                              src->cache_entry.w, src->cache_entry.h))
      {
 #ifdef EVAS_CSERVE
-        if (ie_src->data1) evas_cserve_image_free(ie_src);
-#endif        
+        if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
+#endif
         return 1;
      }
 
 #ifdef EVAS_CSERVE
-   if (ie_src->data1) evas_cserve_image_free(ie_src);
-#endif   
+   if (ie_src->data1) evas_cserve_image_free((Image_Entry*) ie_src);
+#endif
    evas_common_image_colorspace_normalize(src);
    evas_common_image_colorspace_normalize(dst);
 /*    evas_common_blit_rectangle(src, dst, 0, 0, src->cache_entry.w, src->cache_entry.h, 0, 0); */
@@ -320,19 +489,20 @@ _evas_common_rgba_image_dirty(Image_Entry *ie_dst, const Image_Entry *ie_src)
 static int
 _evas_common_rgba_image_ram_usage(Image_Entry *ie)
 {
-   RGBA_Image   *im = (RGBA_Image *) ie;
-   int size = 0;
+   RGBA_Image *im = (RGBA_Image *)ie;
+   int size = sizeof(struct _RGBA_Image);
+
+   if (ie->cache_key) size += strlen(ie->cache_key);
+   if (ie->file) size += strlen(ie->file);
+   if (ie->key) size += strlen(ie->key);
 
-//   ram += sizeof(struct _RGBA_Image);
-//   if (im->info.real_file) ram += strlen(im->info.real_file);
-//   if (im->info.comment) ram += strlen(im->info.comment);
    if (im->image.data)
      {
 #ifdef EVAS_CSERVE
         if ((!im->image.no_free) || (ie->data1))
 #else
         if ((!im->image.no_free))
-#endif          
+#endif
           size += im->cache_entry.w * im->cache_entry.h * sizeof(DATA32);
      }
    size += evas_common_rgba_image_scalecache_usage_get(&im->cache_entry);
@@ -424,7 +594,7 @@ evas_common_image_surface_alpha_tiles_calc(RGBA_Surface *is, int tsize)
 /* } */
 
 static RGBA_Image *
-evas_common_image_create(int w, int h)
+evas_common_image_create(unsigned int w, unsigned int h)
 {
    RGBA_Image *im;
 
@@ -442,7 +612,7 @@ evas_common_image_create(int w, int h)
 }
 
 EAPI RGBA_Image *
-evas_common_image_alpha_create(int w, int h)
+evas_common_image_alpha_create(unsigned int w, unsigned int h)
 {
    RGBA_Image   *im;
 
@@ -461,7 +631,7 @@ evas_common_image_alpha_create(int w, int h)
 }
 
 EAPI RGBA_Image *
-evas_common_image_new(int w, int h, int alpha)
+evas_common_image_new(unsigned int w, unsigned int h, unsigned int alpha)
 {
    if (alpha)
      return evas_common_image_alpha_create(w, h);
@@ -479,9 +649,17 @@ evas_common_image_colorspace_normalize(RGBA_Image *im)
 	if (im->image.data != im->cs.data)
 	  {
 #ifdef EVAS_CSERVE
-             if (((Image_Entry *)im)->data1) evas_cserve_image_free(im);
-#endif             
-	     if (!im->image.no_free) free(im->image.data);
+             if (((Image_Entry *)im)->data1) evas_cserve_image_free(&im->cache_entry);
+#endif
+	     if (!im->image.no_free)
+               {
+                  free(im->image.data);
+#ifdef SURFDBG
+                  surfs = eina_list_remove(surfs, im);
+#endif                  
+                  ((Image_Entry *)im)->allocated.w = 0;
+                  ((Image_Entry *)im)->allocated.h = 0;
+               }
 	     im->image.data = im->cs.data;
 	     im->cs.no_free = im->image.no_free;
 	  }
@@ -493,10 +671,34 @@ evas_common_image_colorspace_normalize(RGBA_Image *im)
 						im->cache_entry.w, im->cache_entry.h);
 #endif
 	break;
+      case EVAS_COLORSPACE_YCBCR422601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_422_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                               im->cache_entry.w, im->cache_entry.h);
+#endif
+        break;
+      case EVAS_COLORSPACE_YCBCR420NV12601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_420_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                               im->cache_entry.w, im->cache_entry.h);
+#endif
+        break;
+      case EVAS_COLORSPACE_YCBCR420TM12601_PL:
+#ifdef BUILD_CONVERT_YUV
+        if ((im->image.data) && (*((unsigned char **)im->cs.data)))
+          evas_common_convert_yuv_420T_601_rgba(im->cs.data, (DATA8*) im->image.data,
+                                                im->cache_entry.w, im->cache_entry.h);
+#endif
+         break;
       default:
 	break;
      }
    im->cs.dirty = 0;
+#ifdef SURFDBG
+   surf_debug();
+#endif   
 }
 
 EAPI void
@@ -507,9 +709,9 @@ evas_common_image_colorspace_dirty(RGBA_Image *im)
 }
 
 EAPI void
-evas_common_image_set_cache(int size)
+evas_common_image_set_cache(unsigned int size)
 {
-   if (eci != NULL)
+   if (eci)
      evas_cache_image_set(eci, size);
 }
 
@@ -520,12 +722,14 @@ evas_common_image_get_cache(void)
 }
 
 EAPI RGBA_Image *
-evas_common_load_image_from_file(const char *file, const char *key, RGBA_Image_Loadopts *lo)
+evas_common_load_image_from_file(const char *file, const char *key, RGBA_Image_Loadopts *lo, int *error)
 {
-   int           error;
-
-   if (file == NULL) return NULL;
-   return (RGBA_Image *) evas_cache_image_request(eci, file, key, lo, &error);
+   if (!file)
+     {
+	*error = EVAS_LOAD_ERROR_GENERIC;
+	return NULL;
+     }
+   return (RGBA_Image *) evas_cache_image_request(eci, file, key, lo, error);
 }
 
 EAPI void
