@@ -1,7 +1,3 @@
-/*
- * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
- */
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -36,10 +32,11 @@ struct _Scaleitem
    unsigned long long usage;
    unsigned long long usage_count;
    RGBA_Image *im, *parent_im;
-   int src_x, src_y, src_w, src_h;
-   int dst_w, dst_h;
-   int flop;
-   int size_adjust;
+   int src_x, src_y;
+   unsigned int src_w, src_h;
+   unsigned int dst_w, dst_h;
+   unsigned int flop;
+   unsigned int size_adjust;
 #ifdef EVAS_FRAME_QUEUING
    RWLK(lock);
 #endif
@@ -51,18 +48,16 @@ struct _Scaleitem
 #ifdef SCALECACHE
 static unsigned long long use_counter = 0;
 
-#ifdef BUILD_PTHREAD
 static LK(cache_lock);
-#endif
 static Eina_Inlist *cache_list = NULL;
-static int cache_size = 0;
+static unsigned int cache_size = 0;
 static int init = 0;
 
-static int max_cache_size = SCALE_CACHE_SIZE;
-static int max_dimension = MAX_SCALECACHE_DIM;
-static int max_flop_count = MAX_FLOP_COUNT;
-static int max_scale_items = MAX_SCALEITEMS;
-static int min_scale_uses = MIN_SCALE_USES;
+static unsigned int max_cache_size = SCALE_CACHE_SIZE;
+static unsigned int max_dimension = MAX_SCALECACHE_DIM;
+static unsigned int max_flop_count = MAX_FLOP_COUNT;
+static unsigned int max_scale_items = MAX_SCALEITEMS;
+static unsigned int min_scale_uses = MIN_SCALE_USES;
 #endif
 
 void
@@ -103,6 +98,7 @@ evas_common_rgba_image_scalecache_init(Image_Entry *ie)
 {
 #ifdef SCALECACHE
    RGBA_Image *im = (RGBA_Image *)ie;
+   // NOTE: this conflicts with evas image cache init and del of lock
    LKI(im->cache.lock);
 #endif
 }
@@ -113,6 +109,7 @@ evas_common_rgba_image_scalecache_shutdown(Image_Entry *ie)
 #ifdef SCALECACHE
    RGBA_Image *im = (RGBA_Image *)ie;
    evas_common_rgba_image_scalecache_dirty(ie);
+   // NOTE: this conflicts with evas image cache init and del of lock
    LKD(im->cache.lock);
 #endif
 }
@@ -194,7 +191,7 @@ _sci_fix_newest(RGBA_Image *im)
 {
    Eina_List *l;
    Scaleitem *sci;
-   
+
    im->cache.newest_usage = 0;
    im->cache.newest_usage_count = 0;
    EINA_LIST_FOREACH(im->cache.list, l, sci)
@@ -211,8 +208,8 @@ static Scaleitem *
 _sci_find(RGBA_Image *im,
           RGBA_Draw_Context *dc __UNUSED__, int smooth,
           int src_region_x, int src_region_y,
-          int src_region_w, int src_region_h,
-          int dst_region_w, int dst_region_h)
+          unsigned int src_region_w, unsigned int src_region_h,
+          unsigned int dst_region_w, unsigned int dst_region_h)
 {
    Eina_List *l;
    Scaleitem *sci;
@@ -269,8 +266,7 @@ _sci_find(RGBA_Image *im,
 
         if (eina_list_count(im->cache.list) > (max_scale_items - 1))
           return NULL;
-        sci = malloc(sizeof(Scaleitem));
-        memset(sci, 0, sizeof(Eina_Inlist));
+        sci = calloc(1, sizeof(Scaleitem));
         sci->parent_im = im;
 #ifdef EVAS_FRAME_QUEUING
         RWLKI(sci->lock);
@@ -336,7 +332,7 @@ _cache_prune(Scaleitem *notsci, Eina_Bool copies_only)
 #endif
 
 EAPI void
-evas_common_rgba_image_scalecache_size_set(int size)
+evas_common_rgba_image_scalecache_size_set(unsigned int size)
 {
 #ifdef SCALECACHE
    LKL(cache_lock);
@@ -349,7 +345,7 @@ evas_common_rgba_image_scalecache_size_set(int size)
 #endif   
 }
 
-EAPI int
+EAPI unsigned int
 evas_common_rgba_image_scalecache_size_get(void)
 {
 #ifdef SCALECACHE
@@ -363,6 +359,20 @@ evas_common_rgba_image_scalecache_size_get(void)
 #endif   
 }
 
+EAPI void
+evas_common_rgba_image_scalecache_dump(void)
+{
+#ifdef SCALECACHE
+   int t;
+   LKL(cache_lock);
+   t = max_cache_size;
+   max_cache_size = 0;
+   _cache_prune(NULL, 0);
+   max_cache_size = t;
+   LKU(cache_lock);
+#endif   
+}
+
 EAPI void
 evas_common_rgba_image_scalecache_flush(void)
 {
@@ -386,19 +396,63 @@ evas_common_rgba_image_scalecache_prepare(Image_Entry *ie, RGBA_Image *dst __UNU
                                           int dst_region_w, int dst_region_h)
 {
 #ifdef SCALECACHE
+   int locked = 0;
+   Eina_Lock_Result ret;
    RGBA_Image *im = (RGBA_Image *)ie;
    Scaleitem *sci;
    if (!im->image.data) return;
    if ((dst_region_w == 0) || (dst_region_h == 0) ||
        (src_region_w == 0) || (src_region_h == 0)) return;
-   LKL(im->cache.lock);
+   // was having major lock issues here - LKL was deadlocking. what was
+   // going on? it may have been an eina treads badness but this will stay here
+   // for now for debug
+#if 1
+   ret = LKT(im->cache.lock);
+   if (ret == EINA_FALSE) /* can't get image lock */
+     {
+        useconds_t slp = 1, slpt = 0;
+        
+        while (slpt < 500000)
+          {
+#ifdef _WIN32
+             Sleep(slp / 1000);
+#else
+             usleep(slp);
+#endif
+             slpt += slp;
+             slp++;
+             ret = LKT(im->cache.lock);
+             if (ret == EINA_LOCK_DEADLOCK)
+               {
+                  printf("WARNING: DEADLOCK on image %p (%s)\n", im, ie->file);
+               }
+             else
+               {
+                  locked = 1;
+                  break;
+               }
+          }
+        if (ret == EINA_FALSE)
+          {
+             printf("WARNING: lock still there after %i usec\n", slpt);
+             printf("WARNING: stucklock on image %p (%s)\n", im, ie->file);
+             LKDBG(im->cache.lock);
+          }
+     }
+   else if (ret == EINA_LOCK_DEADLOCK)
+     {
+        printf("WARNING: DEADLOCK on image %p (%s)\n", im, ie->file);
+     }
+   else locked = 1;
+#endif   
+   if (!locked) { LKL(im->cache.lock); locked = 1; }
    use_counter++;
    if ((src_region_w == dst_region_w) && (src_region_h == dst_region_h))
      {
         // 1:1 scale.
         im->cache.orig_usage++;
         im->cache.usage_count = use_counter;
-        LKU(im->cache.lock);
+        if (locked) LKU(im->cache.lock);
         return;
      }
    if ((!im->cache_entry.flags.alpha) && (!smooth))
@@ -407,7 +461,7 @@ evas_common_rgba_image_scalecache_prepare(Image_Entry *ie, RGBA_Image *dst __UNU
         // or in some cases faster not cached
         im->cache.orig_usage++;
         im->cache.usage_count = use_counter;
-        LKU(im->cache.lock);
+        if (locked) LKU(im->cache.lock);
         return;
      }
    LKL(cache_lock);
@@ -417,7 +471,7 @@ evas_common_rgba_image_scalecache_prepare(Image_Entry *ie, RGBA_Image *dst __UNU
    if (!sci)
      {
         LKU(cache_lock);
-        LKU(im->cache.lock);
+        if (locked) LKU(im->cache.lock);
         return;
      }
 //   INF("%10i | %4i %4i %4ix%4i -> %4i %4i %4ix%4i | %i",
@@ -454,7 +508,7 @@ evas_common_rgba_image_scalecache_prepare(Image_Entry *ie, RGBA_Image *dst __UNU
    if (sci->usage_count > im->cache.newest_usage_count) 
      im->cache.newest_usage_count = sci->usage_count;
 //   INF("  -------------- used %8i#, %8i@", (int)sci->usage, (int)sci->usage_count);
-   LKU(im->cache.lock);
+   if (locked) LKU(im->cache.lock);
 #endif
 }
 
@@ -506,18 +560,11 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
         LKU(im->cache.lock);
         if (im->image.data)
           {
-             if (smooth)
-               evas_common_scale_rgba_in_to_out_clip_smooth(im, dst, dc,
-                                                            src_region_x, src_region_y, 
-                                                            src_region_w, src_region_h,
-                                                            dst_region_x, dst_region_y, 
-                                                            dst_region_w, dst_region_h);
-             else
-               evas_common_scale_rgba_in_to_out_clip_sample(im, dst, dc,
-                                                            src_region_x, src_region_y, 
-                                                            src_region_w, src_region_h,
-                                                            dst_region_x, dst_region_y, 
-                                                            dst_region_w, dst_region_h);
+             evas_common_scale_rgba_in_to_out_clip_sample(im, dst, dc,
+                                                          src_region_x, src_region_y, 
+                                                          src_region_w, src_region_h,
+                                                          dst_region_x, dst_region_y, 
+                                                          dst_region_w, dst_region_h);
           }
         return;
      }
@@ -679,7 +726,7 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
              didpop = 1;
           }
      }
-   if (sci->im)
+   if (sci->im && !ie->flags.animated)
      {
         if (!didpop)
           {
@@ -690,7 +737,7 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
           }
         else
           {
-             if (sci->flop > 0) sci->flop -= FLOP_DEL;
+             if (sci->flop >= FLOP_DEL) sci->flop -= FLOP_DEL;
           }
 //        INF("use cached!");
 #ifdef EVAS_FRAME_QUEUING
@@ -719,19 +766,23 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
          * causes only speed-down side-effect and no memory usage gain;
          * it will be loaded again for the very next rendering for this image.
          */
-        if ((dounload) || 
-            ((im->cache_entry.flags.loaded) && 
-             ((!im->cs.no_free) 
-#ifdef EVAS_CSERVE             
-             || (ie->data1)
-#endif             
-              )  &&
-             (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)))
+        if (ie->scale_hint != EVAS_IMAGE_SCALE_HINT_DYNAMIC)
           {
-             if ((dounload) || (im->cache.orig_usage < 
-                                (im->cache.newest_usage / 20)))
+             if ((dounload) || 
+                 ((im->cache_entry.flags.loaded) && 
+                     ((!im->cs.no_free) 
+#ifdef EVAS_CSERVE             
+                      || (ie->data1)
+#endif             
+                     )  &&
+                     (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)))
                {
-                  evas_common_rgba_image_unload(&im->cache_entry);
+                  if ((dounload) || (im->cache.orig_usage < 
+                                     (im->cache.newest_usage / 20)))
+                    {
+                       //FIXME: imagedataunload - inform owners
+                       evas_common_rgba_image_unload(&im->cache_entry);
+                    }
                }
           }
 #endif
@@ -791,17 +842,3 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
      }
 #endif
 }
-
-#if 0
-// to be done
-void
-evas_common_rgba_image_scalecache_XXX(Image_Entry *ie)
-{
-#ifdef SCALECACHE
-   RGBA_Image *im = (RGBA_Image *)ie;
-   LKL(im->cache.lock);
-   // FIXME: XXX
-   LKU(im->cache.lock);
-#endif
-}
-#endif
