@@ -82,10 +82,10 @@ _limit(Span *s, int c1, int c2, int nocol)
 static void
 _calc_spans(RGBA_Map_Point *p, Line *spans, int ystart, int yend, int cx, int cy __UNUSED__, int cw, int ch __UNUSED__)
 {
-   int i, y, yp;
+   int i, y, yp, yy;
    int py[4];
    int edge[4][4], edge_num, swapped, order[4];
-   FPc uv[4][2], u, v, x, h, t;
+   FPc uv[4][2], u, v, x, h, t, uu, vv;
    DATA32 col[4];
    
 #if 1 // maybe faster on x86?
@@ -169,7 +169,7 @@ _calc_spans(RGBA_Map_Point *p, Line *spans, int ystart, int yend, int cx, int cy
              
              h = (p[e2].y - p[e1].y) >> FP; // height of edge
              if (h < 1) h = 1;
-             t = (((y << FP) + (FP1 - 1)) - p[e1].y) >> FP;
+             t = (((y << FP) + (FP1 / 2) - 1) - p[e1].y) >> FP;
              x = p[e2].x - p[e1].x;
              x = p[e1].x + ((x * t) / h);
 
@@ -220,10 +220,42 @@ _calc_spans(RGBA_Map_Point *p, Line *spans, int ystart, int yend, int cx, int cy
                }
  */
              u = p[e2].u - p[e1].u;
-             u = p[e1].u + ((u * t) / h);
+             uu = u >> FP;
+             if (uu < 0) uu = -uu;
+             if (uu == h)
+               {
+                  yy = ((y << FP) - p[e1].y) >> FP;
+                  if (u > 0)
+                     u = p[e1].u + (yy << FP);
+                  else
+                     u = p[e1].u - (yy << FP) - (FP1 - 1);
+               }
+             else
+               {
+                  if (u >= 0)
+                     u = p[e1].u + ((u * t) / h);
+                  else
+                     u = p[e1].u + (((u * t) - (FP1 / 2)) / h);
+               }
              
              v = p[e2].v - p[e1].v;
-             v = p[e1].v + ((v * t) / h);
+             vv = v >> FP;
+             if (vv < 0) vv = -vv;
+             if (vv == h)
+               {
+                  yy = ((y << FP) - p[e1].y) >> FP;
+                  if (v > 0)
+                     v = p[e1].v + (yy << FP);
+                  else
+                     v = p[e1].v - (yy << FP) - (FP1 - 1);
+               }
+             else
+               {
+                  if (v >= 0)
+                     v = p[e1].v + ((v * t) / h);
+                  else
+                     v = p[e1].v + (((v * t) - (FP1 / 2)) / h);
+               }
 
              // FIXME: 3d accuracy for color too
              t256 = (t << 8) / h; // maybe * 255?
@@ -325,23 +357,23 @@ _calc_spans(RGBA_Map_Point *p, Line *spans, int ystart, int yend, int cx, int cy
 #ifdef BUILD_SCALE_SMOOTH
 # ifdef BUILD_MMX
 #  undef FUNC_NAME
-#  define FUNC_NAME evas_common_map4_rgba_internal_mmx
+#  define FUNC_NAME evas_common_map_rgba_internal_mmx
 #  undef SCALE_USING_MMX
 #  define SCALE_USING_MMX
 #  include "evas_map_image_internal.c"
 # endif
 # ifdef BUILD_C
 #  undef FUNC_NAME
-#  define FUNC_NAME evas_common_map4_rgba_internal
+#  define FUNC_NAME evas_common_map_rgba_internal
 #  undef SCALE_USING_MMX
 #  include "evas_map_image_internal.c"
 # endif
 #endif
 
 EAPI void
-evas_common_map4_rgba(RGBA_Image *src, RGBA_Image *dst,
+evas_common_map_rgba(RGBA_Image *src, RGBA_Image *dst,
                       RGBA_Draw_Context *dc,
-                      RGBA_Map_Point *p, 
+                      int npoints __UNUSED__, RGBA_Map_Point *p,
                       int smooth, int level)
 {
 #ifdef BUILD_MMX
@@ -363,11 +395,11 @@ evas_common_map4_rgba(RGBA_Image *src, RGBA_Image *dst,
      {
 #ifdef BUILD_MMX
         if (mmx)
-          evas_common_map4_rgba_internal_mmx(src, dst, dc, p, smooth, level);
+          evas_common_map_rgba_internal_mmx(src, dst, dc, p, smooth, level);
         else
 #endif
 #ifdef BUILD_C
-          evas_common_map4_rgba_internal(src, dst, dc, p, smooth, level);
+          evas_common_map_rgba_internal(src, dst, dc, p, smooth, level);
 #endif
         return;
      }
@@ -387,11 +419,11 @@ evas_common_map4_rgba(RGBA_Image *src, RGBA_Image *dst,
         evas_common_draw_context_set_clip(dc, r->x, r->y, r->w, r->h);
 #ifdef BUILD_MMX
         if (mmx)
-          evas_common_map4_rgba_internal_mmx(src, dst, dc, p, smooth, level);
+          evas_common_map_rgba_internal_mmx(src, dst, dc, p, smooth, level);
         else
 #endif
 #ifdef BUILD_C
-          evas_common_map4_rgba_internal(src, dst, dc, p, smooth, level);
+          evas_common_map_rgba_internal(src, dst, dc, p, smooth, level);
 #endif        
      }
    evas_common_draw_context_apply_clear_cutouts(rects);
