@@ -1,3 +1,18 @@
+#define NEONDEBUG 0
+
+
+#if NEONDEBUG
+#define DEBUG_FNCOUNT(x)	\
+	do {			\
+	static int _foo = 0;		\
+	if (_foo++%10000 ==0)		\
+		printf("%s %+d %s: %d (%s)\n",__FILE__,__LINE__,__FUNCTION__,\
+				_foo, x " optimised");\
+	} while (0)
+#else
+#define	DEBUG_FNCOUNT(x)	((void)x)
+#endif
+
 
 /* blend mask x color -> dst */
 
@@ -5,9 +20,12 @@
 static void
 _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int l) {
    DATA32 *e;
-   int alpha = 256 - (c >> 24);
+
+   DEBUG_FNCOUNT("");
+
 #define AP "blend_mas_c_dp_"
      asm volatile (
+	".fpu neon						\n\t"
 	"	vdup.i32	q15, %[c]			\n\t"
 	"	vmov.i8		q14,	#1			\n\t"
 
@@ -23,12 +41,12 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	vld1.32		d4[0],	[%[d]]			\n\t"
 	"	vdup.u8		d0,	d0[0]			\n\t"
 	"	vmull.u8	q4,	d0, d30			\n\t"
-	"	vshrn.u16	d12,	q4, #8			\n\t"
+	"	vqrshrn.u16	d12,	q4, #8			\n\t"
 	"	vmvn.u16	d14,	d12			\n\t"
 	"	vshr.u32	d16,	d14, #24		\n\t"
 	"	vmul.u32	d16,	d16, d28		\n\t"
 	"	vmull.u8	q7,	d16, d4			\n\t"
-	"	vshrn.u16	d0,	q7, #8			\n\t"
+	"	vqrshrn.u16	d0,	q7, #8			\n\t"
 	"	vqadd.u8	d0, 	d0, d12			\n\t"
 	"	vst1.32		d0[0],	[%[d]]!			\n\t"
 
@@ -47,12 +65,12 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	vmovl.u8	q0,	d0			\n\t"
 	"	vmul.u32	q0,	q14			\n\t"
 	"	vmull.u8	q4,	d0, d30			\n\t"
-	"	vshrn.u16	d12,	q4, #8			\n\t"
+	"	vqrshrn.u16	d12,	q4, #8			\n\t"
 	"	vmvn.u16	d14,	d12			\n\t"
 	"	vshr.u32	d16,	d14, #24		\n\t"
 	"	vmul.u32	d16,	d16, d28		\n\t"
 	"	vmull.u8	q7,	d16, d4			\n\t"
-	"	vshrn.u16	d0,	q7, #8			\n\t"
+	"	vqrshrn.u16	d0,	q7, #8			\n\t"
 	"	vqadd.u8	q0, 	q0, q6			\n\t"
 	"	vstm		%[d]!,	{d0}			\n\t"
 
@@ -60,6 +78,8 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	sub		%[tmp], %[e], %[d]		\n\t"
 	"	cmp		%[tmp], #16			\n\t"
 	"	blt		"AP"loopout			\n\t"
+
+
 	"	sub		%[tmp], %[e], #15		\n\t"
 
 	"	sub		%[d],	#16			\n\t"
@@ -85,8 +105,8 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	vmull.u8	q5,	d1, d31			\n\t"
 
 	// Shorten
-	"	vshrn.u16	d12,	q4, #8			\n\t"
-	"	vshrn.u16	d13,	q5, #8			\n\t"
+	"	vqrshrn.u16	d12,	q4, #8			\n\t"
+	"	vqrshrn.u16	d13,	q5, #8			\n\t"
 
 	// extract negated alpha
 	"	vmvn.u16	q7,	q6			\n\t"
@@ -97,8 +117,8 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	vmull.u8	q7,	d16, d4			\n\t"
 	"	vmull.u8	q8,	d17, d5			\n\t"
 
-	"	vshrn.u16	d0,	q7, #8			\n\t"
-	"	vshrn.u16	d1,	q8, #8			\n\t"
+	"	vqrshrn.u16	d0,	q7, #8			\n\t"
+	"	vqrshrn.u16	d1,	q8, #8			\n\t"
 
 	// Add
 	"	vqadd.u8	q0, 	q0, q6			\n\t"
@@ -109,6 +129,15 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	bhi		"AP"quadloopint			\n\t"
 
 	AP"loopout:						\n\t"
+#if NEONDEBUG
+		"cmp		%[d], %[e]		\n\t"
+		"ble		"AP"foo		\n\t"
+		"cmp		%[tmp], %[m]	\n\t"
+		"sub		%[x],	%[x]		\n\t"
+		"vst1.32	d0[0], [%[x]]		\n\t"
+	AP"foo: \n\t"
+#endif
+
 	"	cmp		%[d], %[e]			\n\t"
 	"	beq		"AP"done			\n\t"
 	"	sub		%[tmp],%[e], %[d]		\n\t"
@@ -116,19 +145,19 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	beq		"AP"singleout			\n\t"
 
 	AP "dualloop2:					\n\t"
-		"sub		%[tmp],%[e],$0x7	\n\t"
+		"sub		%[tmp],%[e],$0x8	\n\t"
 	"	vld1.16		d0[0],	[%[m]]!			\n\t"
 	"	vldm		%[d],	{d4}			\n\t"
 	"	vmovl.u8	q0,	d0			\n\t"
 	"	vmovl.u8	q0,	d0			\n\t"
 	"	vmul.u32	q0,	q14			\n\t"
 	"	vmull.u8	q4,	d0, d30			\n\t"
-	"	vshrn.u16	d12,	q4, #8			\n\t"
+	"	vqrshrn.u16	d12,	q4, #8			\n\t"
 	"	vmvn.u16	d14,	d12			\n\t"
 	"	vshr.u32	d16,	d14, #24		\n\t"
 	"	vmul.u32	d16,	d16, d28		\n\t"
 	"	vmull.u8	q7,	d16, d4			\n\t"
-	"	vshrn.u16	d0,	q7, #8			\n\t"
+	"	vqrshrn.u16	d0,	q7, #8			\n\t"
 	"	vqadd.u8	q0, 	q0, q6			\n\t"
 	"	vstm		%[d]!,	{d0}			\n\t"
 
@@ -140,17 +169,23 @@ _op_blend_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int
 	"	vld1.32		d4[0],	[%[d]]			\n\t"
 	"	vdup.u8		d0,	d0[0]			\n\t"
 	"	vmull.u8	q4,	d0, d30			\n\t"
-	"	vshrn.u16	d12,	q4, #8			\n\t"
+	"	vqrshrn.u16	d12,	q4, #8			\n\t"
 	"	vmvn.u16	d14,	d12			\n\t"
 	"	vshr.u32	d16,	d14, #24		\n\t"
 	"	vmul.u32	d16,	d16, d28		\n\t"
 	"	vmull.u8	q7,	d16, d4			\n\t"
-	"	vshrn.u16	d0,	q7, #8			\n\t"
+	"	vqrshrn.u16	d0,	q7, #8			\n\t"
 	"	vqadd.u8	q0, 	q0, q6			\n\t"
 	"	vst1.32		d0[0],	[%[d]]!			\n\t"
 
 	AP"done:						\n\t"
-
+#if NEONDEBUG
+		"cmp		%[d], %[e]		\n\t"
+		"beq		"AP"reallydone		\n\t"
+		"sub		%[tmp],	%[tmp]		\n\t"
+		"vst1.32	d0[0], [%[tmp]]		\n\t"
+	AP"reallydone:"
+#endif
 	: // Out
 	:  [e] "r" (d + l), [d] "r" (d), [c] "r" (c),
 		[tmp] "r" (7), [m] "r" (m), [x] "r" (0)
@@ -166,8 +201,12 @@ static void
 _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int l) {
    DATA32 *e,*tmp;
    int alpha;
+
+   DEBUG_FNCOUNT("");
+
 #define AP	"_blend_mas_can_dp_neon_"
      asm volatile (
+		".fpu neon				\n\t"
 		"vdup.u32	q9,	%[c]		\n\t"
 		"vmov.i8	q15,	#1		\n\t"
 		"vmov.i8	q14,	#0		\n\t"
@@ -181,7 +220,7 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	beq		"AP"quadloop		\n\t"
 
 	"	andS		%[tmp], %[d], #4	\n\t"
-	"	beq		"AP"dualloop		\n\t"
+	"	beq		"AP"dualstart		\n\t"
 
 
 	AP"singleloop:					\n\t"
@@ -198,16 +237,21 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	vqmovun.s16	d2, 	q6		\n\t"
 	"	vst1.32		d2[0],	[%[d]]!		\n\t"
 
-	"	andS		%[tmp], %[d],	#15	\n\t"
+	"	andS		%[tmp], %[d],	$0xf	\n\t"
 	"	beq		"AP"quadloop		\n\t"
 
+	AP"dualstart:					\n\t"
+	"	sub		%[tmp], %[e], %[d]	\n\t"
+	"	cmp		%[tmp],	#16		\n\t"
+	"	blt		"AP"loopout		\n\t"
+
 	AP"dualloop:					\n\t"
-	"	vld1.16	d0[0],	[%[m]]!		\n\t"
-	"	vldm		%[d],		{d8}	\n\t"
+	"	vld1.16		d0[0],	[%[m]]!		\n\t"
+	"	vldm		%[d],	{d8}		\n\t"
 	"	vmovl.u8	q0,	d0		\n\t"
 	"	vmovl.u8	q0,	d0		\n\t"
 	"	vmul.u32	d0,	d0,	d30	\n\t"
-	"	vshr.u8	d0,	d0, #1		\n\t"
+	"	vshr.u8		d0,	d0, #1		\n\t"
 	"	vmovl.u8	q0,	d0		\n\t"
 	"	vmovl.u8	q4,	d8		\n\t"
 	"	vsub.s16	q6,	q2, q4		\n\t"
@@ -227,21 +271,23 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	AP"fastloop:					\n\t"
 	"	add		%[d],	#16		\n\t"
 	"	cmp		%[tmp], %[d]		\n\t"
-	"	ble		"AP"loopout		\n\t"
+	"	blt		"AP"loopout		\n\t"
 
 	AP"quadloopint:					\n\t"
 		// Load the mask: 4 bytes: It has d0/d1
 	"	ldr		%[x],	[%[m]]		\n\t"
 	"	add		%[m], #4		\n\t"
+
+		// Check for shortcuts
 	"	cmp		%[x],	#0		\n\t"
 	"	beq		"AP"fastloop		\n\t"
-	"	vmov.32		d0[0],	%[x]		\n\t"
 
-		// Load d into d8/d9 q4
-	"	vldm		%[d],	{d8,d9}		\n\t"
 	"	cmp		%[x],	$0xffffffff	\n\t"
 	"	beq		"AP"quadstore		\n\t"
 
+	"	vmov.32		d0[0],	%[x]		\n\t"
+		// Load d into d8/d9 q4
+	"	vldm		%[d],	{d8,d9}		\n\t"
 
 		// Get the alpha channel ready (m)
 	"	vmovl.u8	q0,	d0		\n\t"
@@ -277,7 +323,7 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	vqmovun.s16	d9,  q7			\n\t"
 	"	vqmovun.s16	d8,  q6			\n\t"
 
-	"	vstm		%[d]!,	{d8,d9}	\n\t"
+	"	vstm		%[d]!,	{d8,d9}		\n\t"
 
 	"	cmp		%[tmp], %[d]		\n\t"
 	"	bhi		"AP"quadloopint		\n\t"
@@ -288,8 +334,14 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	cmp		%[tmp], %[d]		\n\t"
 	"	bhi		"AP"quadloopint		\n\t"
 
-
 	AP"loopout:					\n\t"
+#if NEONDEBUG
+		"cmp		%[d], %[e]		\n\t"
+		"ble		"AP"foo		\n\t"
+		"sub		%[tmp],	%[tmp]		\n\t"
+		"vst1.32	d0[0], [%[tmp]]		\n\t"
+	AP"foo: \n\t"
+#endif
 
 	"	cmp		%[e], %[d]		\n\t"
 	"	beq		"AP"done		\n\t"
@@ -300,7 +352,7 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	blt		"AP"onebyte		\n\t"
 
 		// Load the mask: 2 bytes: It has d0
-	"	vld1.16	d0[0],	[%[m]]!		\n\t"
+	"	vld1.16		d0[0],	[%[m]]!		\n\t"
 
 		// Load d into d8/d9 q4
 	"	vldm		%[d],		{d8}	\n\t"
@@ -310,7 +362,7 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 	"	vmovl.u8	q0,	d0		\n\t"
 	"	vmul.u32	d0,	d0,	d30	\n\t"
 		// Lop a bit off to prevent overflow
-	"	vshr.u8	d0,	d0, #1		\n\t"
+	"	vshr.u8	d0,	d0, #1			\n\t"
 
 		// Now make it 16 bit
 	"	vmovl.u8	q0,	d0		\n\t"
@@ -338,7 +390,7 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 		"beq		"AP"done		\n\t"
 
 	AP"onebyte:					\n\t"
-		"vld1.8	d0[0],	[%[m]]!			\n\t"
+		"vld1.8		d0[0],	[%[m]]!		\n\t"
 		"vld1.32	d8[0],	[%[d]]		\n\t"
 		"vdup.u8	d0,	d0[0]		\n\t"
 		"vshr.u8	d0,	d0, #1		\n\t"
@@ -351,13 +403,22 @@ _op_blend_mas_can_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, i
 		"vqmovun.s16	d2,  q6			\n\t"
 		"vst1.32	d2[0], [%[d]]!		\n\t"
 
+
 	AP"done:					\n\t"
+#if NEONDEBUG
+		"cmp		%[d], %[e]		\n\t"
+		"beq		"AP"reallydone		\n\t"
+		"sub		%[m],	%[m]		\n\t"
+		"vst1.32	d0[0], [%[m]]		\n\t"
+	AP"reallydone:"
+#endif
+
 
 	  : // output regs
 	  // Input
           :  [e] "r" (e = d + l), [d] "r" (d), [c] "r" (c),
 		[m] "r" (m), [tmp] "r" (7), [x] "r" (33)
-          : "q0", "q1", "q2","q3", "q4","q5","q6", "q7","q14","q15",
+          : "q0", "q1", "q2","q3", "q4","q5","q6", "q7","q9","q14","q15",
 			"memory" // clobbered
 
      );
@@ -431,6 +492,9 @@ static void
 _op_blend_rel_mas_c_dp_neon(DATA32 *s __UNUSED__, DATA8 *m, DATA32 c, DATA32 *d, int l) {
    DATA32 *e;
    int alpha;
+
+   DEBUG_FNCOUNT("not");
+
    UNROLL8_PLD_WHILE(d, l, e,
                      {
                         DATA32 mc = MUL_SYM(*m, c);
