@@ -1,6 +1,7 @@
 // THIS IS DEPRECATED. WILL GO EVENTUALLTY. NO NEED TO SUPPORT ANYMORE
 
 #include "evas_common.h"
+#include <unistd.h>
 
 #ifdef BUILD_PIPE_RENDER
 
@@ -74,7 +75,7 @@ evas_common_frame_add(Evas_FrameQ *frameq, Evas_Frame *frame)
    while ((int)eina_inlist_count(EINA_INLIST_GET(frameq->frames)) >= frameq->frameq_sz)
      {
         /* wait a worker thread finish previous frame */
-        pthread_cond_wait(&(frameq->cond_done), &(frameq->mutex));
+        eina_condition_wait(&(frameq->cond_done));
      }
    frameq->frames = (Evas_Frame *) eina_inlist_append(EINA_INLIST_GET(frameq->frames), EINA_INLIST_GET(frame));
 
@@ -91,7 +92,7 @@ evas_common_frame_add(Evas_FrameQ *frameq, Evas_Frame *frame)
 
    LKU(frameq->mutex);
 
-   pthread_cond_signal(&(frameq->cond_new));
+   eina_condition_signal(&(frameq->cond_new));
 }
 
 EAPI Evas_Surface *
@@ -145,10 +146,10 @@ EAPI void
 evas_common_frameq_init(void)
 {
    gframeq.frames = NULL;
-   pthread_cond_init(&(gframeq.cond_new), NULL);
-   pthread_cond_init(&(gframeq.cond_ready), NULL);
-   pthread_cond_init(&(gframeq.cond_done), NULL);
    LKI(gframeq.mutex);
+   eina_condition_new(&(gframeq.cond_new), &(gframeq.mutex));
+   eina_condition_new(&(gframeq.cond_ready), &(gframeq.mutex));
+   eina_condition_new(&(gframeq.cond_done), &(gframeq.mutex));
    gframeq.initialised = 0;	// worker thread are not created yet
    gframeq.frameq_sz = 1;	// this value ensures the first frame can be enqueued.
 }
@@ -158,13 +159,13 @@ evas_common_frameq_destroy(void)
 {
 #if 0 // let them destroyed indirectly with program exit
    LKL(gframeq.mutex);
-   pthread_cond_destroy(&(gframeq.cond_new));
-   pthread_cond_destroy(&(gframeq.cond_ready));
-   pthread_cond_destroy(&(gframeq.cond_done));
+   eina_condition_free(&(gframeq.cond_new));
+   eina_condition_free(&(gframeq.cond_ready));
+   eina_condition_free(&(gframeq.cond_done));
    LKU(gframeq.mutex);
 #endif
    LKD(gframeq.mutex);
-   
+
    gframeq.frames = NULL;
    gframeq.initialised = 0;
 }
@@ -179,7 +180,7 @@ evas_common_frameq_flush(void)
    while(eina_inlist_count(EINA_INLIST_GET(gframeq.frames)) > 0)
      {
         /* wait a worker thread finish previous frame */
-        pthread_cond_wait(&(gframeq.cond_done), &(gframeq.mutex));
+        eina_condition_wait(&(gframeq.cond_done));
    }
    LKU(gframeq.mutex);
 }
@@ -210,24 +211,24 @@ static void evas_common_pipe_op_free(RGBA_Pipe_Op *op);
 
 /* utils */
 static RGBA_Pipe *
-evas_common_pipe_add(RGBA_Pipe *pipe, RGBA_Pipe_Op **op)
+evas_common_pipe_add(RGBA_Pipe *rpipe, RGBA_Pipe_Op **op)
 {
    RGBA_Pipe *p;
    int first_pipe = 0;
 
-   if (!pipe)
+   if (!rpipe)
      {
         first_pipe = 1;
         p = calloc(1, sizeof(RGBA_Pipe));
         if (!p) return NULL;
-        pipe = (RGBA_Pipe *)eina_inlist_append(EINA_INLIST_GET(pipe), EINA_INLIST_GET(p));
+        rpipe = (RGBA_Pipe *)eina_inlist_append(EINA_INLIST_GET(rpipe), EINA_INLIST_GET(p));
      }
-   p = (RGBA_Pipe *)(EINA_INLIST_GET(pipe))->last;
+   p = (RGBA_Pipe *)(EINA_INLIST_GET(rpipe))->last;
    if (p->op_num == PIPE_LEN)
      {
         p = calloc(1, sizeof(RGBA_Pipe));
         if (!p) return NULL;
-        pipe = (RGBA_Pipe *)eina_inlist_append(EINA_INLIST_GET(pipe), EINA_INLIST_GET(p));
+        rpipe = (RGBA_Pipe *)eina_inlist_append(EINA_INLIST_GET(rpipe), EINA_INLIST_GET(p));
      }
    p->op_num++;
    *op = &(p->op[p->op_num - 1]);
@@ -235,7 +236,7 @@ evas_common_pipe_add(RGBA_Pipe *pipe, RGBA_Pipe_Op **op)
      {
         /* FIXME: PTHREAD init any thread locks etc */
      }
-   return pipe;
+   return rpipe;
 }
 
 static void
@@ -359,7 +360,7 @@ evas_common_frameq_thread(void *data)
                   break;
                }
              pthread_testcancel();
-             pthread_cond_wait(&(frameq->cond_new), &(frameq->mutex));
+             eina_condition_wait(&(frameq->cond_new));
           }
         LKU(frameq->mutex);
 
@@ -398,7 +399,7 @@ evas_common_frameq_thread(void *data)
 
         LKL(frameq->mutex);
         frame->ready = 1;
-        pthread_cond_signal(&frameq->cond_ready);
+        eina_condition_signal(&(frameq->cond_ready));
         LKU(frameq->mutex);
      }
 
@@ -584,7 +585,7 @@ evas_common_frameq_schedule_flush_time(int frameq_sz, int thread_no,
    long long current_time = 0LL;
    long long current_ready_interval = 0LL;
    long long theshold_time = SATISFACTION_THRESHOLD * 1000LL;	// ms -> usec
-   long long reset_time_interval;
+   long long reset_time_interval = 0LL;
    long long sleep_time = 0LL;
    long long saved_ready_time, saved_ready_interval;
    long long time_slept = 0LL;
@@ -734,7 +735,7 @@ evas_common_frameq_thread_post(void *data)
         LKL(frameq->mutex);
         while(!frameq->frames || !frameq->frames->ready)
           {
-             pthread_cond_wait(&(frameq->cond_ready), &(frameq->mutex));
+             eina_condition_wait(&(frameq->cond_ready));
           }
         frame = frameq->frames;
 
@@ -784,7 +785,7 @@ evas_common_frameq_thread_post(void *data)
                   EINA_INLIST_GET(frame));
 
         LKU(frameq->mutex);
-        pthread_cond_broadcast(&frameq->cond_done);
+        eina_condition_broadcast(&frameq->cond_done);
         evas_common_frame_dealloc(frame);
      }
 
@@ -868,6 +869,8 @@ evas_common_frameq_begin(void)
         gframeq.thread_num = cpunum;
         gframeq.frameq_sz = cpunum * FRAMEQ_SZ_PER_THREAD;
 
+	eina_threads_init();
+
         for (i = 0; i < gframeq.thread_num; i++)
           {
 
@@ -937,10 +940,10 @@ evas_common_frameq_finish(void)
    /* 2. send signal to worker threads so that they enter to the thread cancelation cleanup handler */
    for (i = 0; i < gframeq.thread_num; i++)
      {
-        pthread_cond_signal(&(gframeq.cond_new));
+        eina_condition_signal(&(gframeq.cond_new));
      }
    // send signal to post-worker thread
-   pthread_cond_signal(&(gframeq.cond_ready));
+   eina_condition_signal(&(gframeq.cond_ready));
 
    /* all the workers were created and detached before
     *  so don't need to join them here.
@@ -1173,14 +1176,11 @@ evas_common_pipe_op_text_free(RGBA_Pipe_Op *op)
    LKL(op->op.text.font->ref_fq_del);
    op->op.text.font->ref_fq[1]++;
    LKU(op->op.text.font->ref_fq_del);
-   pthread_cond_signal(&(op->op.text.font->cond_fq_del));
+   eina_condition_signal(&(op->op.text.font->cond_fq_del));
 #else
    evas_common_font_free(op->op.text.font);
 #endif
-#ifdef BIDI_SUPPORT
-   evas_bidi_props_clean(&(op->op.text.intl_props));
-#endif   
-   free(op->op.text.text);
+   evas_common_text_props_content_unref(&(op->op.text.intl_props));
    evas_common_pipe_op_free(op);
 }
 
@@ -1196,7 +1196,7 @@ evas_common_pipe_op_text_flush(RGBA_Font *fn)
    LKL(fn->ref_fq_del);
 
    while (fn->ref_fq[0] != fn->ref_fq[1])
-      pthread_cond_wait(&(fn->cond_fq_del), &(fn->ref_fq_del));
+     eina_condition_wait(&(fn->cond_fq_del));
 
    LKU(fn->ref_fq_del);
    LKU(fn->ref_fq_add);
@@ -1218,31 +1218,29 @@ evas_common_pipe_text_draw_do(RGBA_Image *dst, RGBA_Pipe_Op *op, RGBA_Pipe_Threa
 #endif
         evas_common_font_draw(dst, &(context),
                   op->op.text.font, op->op.text.x, op->op.text.y,
-                  op->op.text.text, &op->op.text.intl_props);
+                  &op->op.text.intl_props);
      }
    else
      {
         evas_common_font_draw(dst, &(op->context),
                   op->op.text.font, op->op.text.x, op->op.text.y,
-                  op->op.text.text, &op->op.text.intl_props);
+                  &op->op.text.intl_props);
      }
 }
 
 EAPI void
 evas_common_pipe_text_draw(RGBA_Image *dst, RGBA_Draw_Context *dc,
-                           RGBA_Font *fn, int x, int y, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props)
+               RGBA_Font *fn, int x, int y, const Evas_Text_Props *intl_props)
 {
    RGBA_Pipe_Op *op;
 
-   if ((!fn) || (!text)) return;
+   if (!fn) return;
    dst->cache_entry.pipe = evas_common_pipe_add(dst->cache_entry.pipe, &op);
    if (!dst->cache_entry.pipe) return;
    op->op.text.x = x;
    op->op.text.y = y;
-   op->op.text.text = eina_unicode_strdup(text);
-#ifdef BIDI_SUPPORT
-   evas_bidi_props_copy_and_ref(intl_props, &(op->op.text.intl_props));
-#endif   
+   evas_common_text_props_content_copy_and_ref(&(op->op.text.intl_props),
+         intl_props);
 #ifdef EVAS_FRAME_QUEUING
    LKL(fn->ref_fq_add);
    fn->ref_fq[0]++;
@@ -1264,7 +1262,7 @@ evas_common_pipe_op_image_free(RGBA_Pipe_Op *op)
    LKL(op->op.image.src->cache_entry.ref_fq_del);
    op->op.image.src->cache_entry.ref_fq[1]++;
    LKU(op->op.image.src->cache_entry.ref_fq_del);
-   pthread_cond_signal(&(op->op.image.src->cache_entry.cond_fq_del)); 
+   eina_condition_signal(&(op->op.image.src->cache_entry.cond_fq_del));
 #else
    op->op.image.src->ref--;
    if (op->op.image.src->ref == 0)
@@ -1286,7 +1284,7 @@ evas_common_pipe_op_image_flush(RGBA_Image *im)
    LKL(im->cache_entry.ref_fq_del);
 
    while (im->cache_entry.ref_fq[0] != im->cache_entry.ref_fq[1])
-      pthread_cond_wait(&(im->cache_entry.cond_fq_del), &(im->cache_entry.ref_fq_del));
+     eina_condition_wait(&(im->cache_entry.cond_fq_del));
 
    LKU(im->cache_entry.ref_fq_del);
    LKU(im->cache_entry.ref_fq_add);
@@ -1443,23 +1441,23 @@ evas_common_pipe_image_draw(RGBA_Image *src, RGBA_Image *dst,
 }
 
 static void
-evas_common_pipe_op_map4_free(RGBA_Pipe_Op *op)
+evas_common_pipe_op_map_free(RGBA_Pipe_Op *op)
 {
 #ifdef EVAS_FRAME_QUEUING
    LKL(op->op.image.src->cache_entry.ref_fq_del);
    op->op.image.src->cache_entry.ref_fq[1]++;
    LKU(op->op.image.src->cache_entry.ref_fq_del);
 #else
-   op->op.map4.src->ref--;
-   if (op->op.map4.src->ref == 0)
-     evas_cache_image_drop(&op->op.map4.src->cache_entry);
+   op->op.map.src->ref--;
+   if (op->op.map.src->ref == 0)
+     evas_cache_image_drop(&op->op.map.src->cache_entry);
 #endif
-   free(op->op.map4.p);
+   free(op->op.map.p);
    evas_common_pipe_op_free(op);
 }
 
 static void
-evas_common_pipe_map4_draw_do(RGBA_Image *dst, RGBA_Pipe_Op *op, RGBA_Pipe_Thread_Info *info)
+evas_common_pipe_map_draw_do(RGBA_Image *dst, RGBA_Pipe_Op *op, RGBA_Pipe_Thread_Info *info)
 {
    if (info)
      {
@@ -1472,22 +1470,22 @@ evas_common_pipe_map4_draw_do(RGBA_Image *dst, RGBA_Pipe_Op *op, RGBA_Pipe_Threa
 	evas_common_draw_context_clip_clip(&(context), info->x, info->y, info->w, info->h);
 #endif
 
-	evas_common_map4_rgba(op->op.map4.src, dst,
-			      &context, op->op.map4.p,
-			      op->op.map4.smooth, op->op.map4.level);
+	evas_common_map_rgba(op->op.map.src, dst,
+                             &context, op->op.map.npoints, op->op.map.p,
+                             op->op.map.smooth, op->op.map.level);
      }
    else
      {
-	evas_common_map4_rgba(op->op.map4.src, dst,
-			      &(op->context), op->op.map4.p,
-			      op->op.map4.smooth, op->op.map4.level);
+	evas_common_map_rgba(op->op.map.src, dst,
+                             &(op->context), op->op.map.npoints, op->op.map.p,
+                             op->op.map.smooth, op->op.map.level);
      }
 }
 
 EAPI void
-evas_common_pipe_map4_draw(RGBA_Image *src, RGBA_Image *dst,
-			   RGBA_Draw_Context *dc, RGBA_Map_Point *p,
-			   int smooth, int level)
+evas_common_pipe_map_draw(RGBA_Image *src, RGBA_Image *dst,
+                          RGBA_Draw_Context *dc, int npoints, RGBA_Map_Point *p,
+                          int smooth, int level)
 {
    RGBA_Pipe_Op *op;
    RGBA_Map_Point *pts_copy;
@@ -1506,8 +1504,9 @@ evas_common_pipe_map4_draw(RGBA_Image *src, RGBA_Image *dst,
    for (i = 0; i < 4; ++i)
      pts_copy[i] = p[i];
 
-   op->op.map4.smooth = smooth;
-   op->op.map4.level = level;
+   op->op.map.npoints = npoints;
+   op->op.map.smooth = smooth;
+   op->op.map.level = level;
 #ifdef EVAS_FRAME_QUEUING
    LKL(src->cache_entry.ref_fq_add);
    src->cache_entry.ref_fq[0]++;
@@ -1515,10 +1514,10 @@ evas_common_pipe_map4_draw(RGBA_Image *src, RGBA_Image *dst,
 #else
    src->ref++;
 #endif
-   op->op.map4.src = src;
-   op->op.map4.p = pts_copy;
-   op->op_func = evas_common_pipe_map4_draw_do;
-   op->free_func = evas_common_pipe_op_map4_free;
+   op->op.map.src = src;
+   op->op.map.p = pts_copy;
+   op->op_func = evas_common_pipe_map_draw_do;
+   op->free_func = evas_common_pipe_op_map_free;
    evas_common_pipe_draw_context_copy(dc, op);
 
 #ifdef EVAS_FRAME_QUEUING
@@ -1536,7 +1535,7 @@ evas_common_pipe_map4_draw(RGBA_Image *src, RGBA_Image *dst,
 }
 
 static void
-evas_common_pipe_map4_render(RGBA_Image *root)
+evas_common_pipe_map_render(RGBA_Image *root)
 {
   RGBA_Pipe *p;
   int i;
@@ -1546,15 +1545,15 @@ evas_common_pipe_map4_render(RGBA_Image *root)
     {
       for (i = 0; i < p->op_num; i++) 
 	{
-	  if (p->op[i].op_func == evas_common_pipe_map4_draw_do)
+	  if (p->op[i].op_func == evas_common_pipe_map_draw_do)
 	    {
-	      if (p->op[i].op.map4.src->cache_entry.pipe)
-		evas_common_pipe_map4_render(p->op[i].op.map4.src);
+	      if (p->op[i].op.map.src->cache_entry.pipe)
+		evas_common_pipe_map_render(p->op[i].op.map.src);
 	    }
 	  else if (p->op[i].op_func == evas_common_pipe_image_draw_do)
 	    {
 	      if (p->op[i].op.image.src->cache_entry.pipe)
-		evas_common_pipe_map4_render(p->op[i].op.image.src);
+		evas_common_pipe_map_render(p->op[i].op.image.src);
 	    }
 	}
     }
@@ -1567,20 +1566,20 @@ evas_common_pipe_map4_render(RGBA_Image *root)
 static Eina_List *task = NULL;
 static Thinfo task_thinfo[TH_MAX];
 static pthread_barrier_t task_thbarrier[2];
-static LK(task_mutext) = PTHREAD_MUTEX_INITIALIZER;
+static LK(task_mutext);
 #endif
 
 #ifdef BUILD_PTHREAD
 static void*
 evas_common_pipe_load(void *data)
 {
-  Thinfo *thinfo;
+  Thinfo *tinfo;
 
-  thinfo = data;
+  tinfo = data;
   for (;;)
     {
       /* wait for start signal */
-      pthread_barrier_wait(&(thinfo->barrier[0]));
+      pthread_barrier_wait(&(tinfo->barrier[0]));
 
       while (task)
 	{
@@ -1602,7 +1601,7 @@ evas_common_pipe_load(void *data)
 	}
 
       /* send finished signal */    
-      pthread_barrier_wait(&(thinfo->barrier[1]));
+      pthread_barrier_wait(&(tinfo->barrier[1]));
     }
 
   return NULL;
@@ -1638,6 +1637,10 @@ evas_common_pipe_init(void)
 // waiting onm pthread barriers for async rendering on a single core!
 //	if (thread_num == 1) return EINA_FALSE;
 
+	eina_threads_init();
+
+        LKI(task_mutext);
+
 	pthread_barrier_init(&(thbarrier[0]), NULL, thread_num + 1);
 	pthread_barrier_init(&(thbarrier[1]), NULL, thread_num + 1);
 	for (i = 0; i < thread_num; i++)
@@ -1677,11 +1680,8 @@ evas_common_pipe_init(void)
 			    evas_common_pipe_load, &(task_thinfo[i]));
 	     pthread_attr_destroy(&attr);
 	  }
-
-#if defined(METRIC_CACHE) || defined(WORD_CACHE)
-	eina_threads_init();
-#endif
      }
+
    if (thread_num == 1) return EINA_FALSE;
    return EINA_TRUE;
 #endif
@@ -1709,7 +1709,7 @@ evas_common_pipe_image_load(RGBA_Image *im)
 }
 
 EAPI void
-evas_common_pipe_map4_begin(RGBA_Image *root)
+evas_common_pipe_map_begin(RGBA_Image *root)
 {
   if (!evas_common_pipe_init())
     {
@@ -1727,7 +1727,7 @@ evas_common_pipe_map4_begin(RGBA_Image *root)
 
   evas_common_pipe_image_load_do();
 
-  evas_common_pipe_map4_render(root);
+  evas_common_pipe_map_render(root);
 }
 
 #endif
