@@ -1,6 +1,8 @@
 #ifndef _EVAS_PIPE_H
 #define _EVAS_PIPE_H
-#include "evas_bidi_utils.h"
+
+#include <sys/time.h>
+#include "language/evas_bidi_utils.h"
 
 #ifdef BUILD_PTHREAD
 typedef struct _Thinfo
@@ -46,9 +48,9 @@ struct _Evas_FrameQ
 {
    int initialised;
    Evas_Frame   *frames;
-   pthread_cond_t cond_new;
-   pthread_cond_t cond_ready;
-   pthread_cond_t cond_done;
+   Eina_Condition cond_new;
+   Eina_Condition cond_ready;
+   Eina_Condition cond_done;
    LK(mutex);
 
    int             thread_num;
@@ -92,12 +94,12 @@ EAPI void evas_common_pipe_free(RGBA_Image *im);
 EAPI void evas_common_pipe_rectangle_draw(RGBA_Image *dst, RGBA_Draw_Context *dc, int x, int y, int w, int h);
 EAPI void evas_common_pipe_line_draw(RGBA_Image *dst, RGBA_Draw_Context *dc, int x0, int y0, int x1, int y1);
 EAPI void evas_common_pipe_poly_draw(RGBA_Image *dst, RGBA_Draw_Context *dc, RGBA_Polygon_Point *points, int x, int y);
-EAPI void evas_common_pipe_text_draw(RGBA_Image *dst, RGBA_Draw_Context *dc, RGBA_Font *fn, int x, int y, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
+EAPI void evas_common_pipe_text_draw(RGBA_Image *dst, RGBA_Draw_Context *dc, RGBA_Font *fn, int x, int y, const Evas_Text_Props *intl_props);
 EAPI void evas_common_pipe_image_load(RGBA_Image *im);
 EAPI void evas_common_pipe_image_draw(RGBA_Image *src, RGBA_Image *dst, RGBA_Draw_Context *dc, int smooth, int src_region_x, int src_region_y, int src_region_w, int src_region_h, int dst_region_x, int dst_region_y, int dst_region_w, int dst_region_h);
-EAPI void evas_common_pipe_map4_begin(RGBA_Image *root);
-EAPI void evas_common_pipe_map4_draw(RGBA_Image *src, RGBA_Image *dst,
-				     RGBA_Draw_Context *dc, RGBA_Map_Point *p,
+EAPI void evas_common_pipe_map_begin(RGBA_Image *root);
+EAPI void evas_common_pipe_map_draw(RGBA_Image *src, RGBA_Image *dst,
+				     RGBA_Draw_Context *dc, int npoints, RGBA_Map_Point *p,
 				     int smooth, int level);
 EAPI void evas_common_pipe_flush(RGBA_Image *im);
 
