@@ -8,15 +8,6 @@
 #include <evas_private.h>
 #include <evas_module.h>
 
-#ifdef _WIN32
-# ifdef open
-#  undef open
-# endif
-# ifdef close
-#  undef close
-# endif
-#endif
-
 
 static Eina_Hash *evas_modules[4] = {
   NULL,
@@ -91,14 +82,12 @@ evas_module_paths_init(void)
   { evas_##Tn##_##Name##_init, evas_##Tn##_##Name##_shutdown }
 
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, buffer);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, cairo_x11);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, direct3d);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, directfb);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, fb);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, gl_glew);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, gl_x11);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, gl_sdl);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, quartz);
+EVAS_EINA_STATIC_MODULE_DEFINE(engine, psl1ght);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_ddraw);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_16_sdl);
@@ -109,21 +98,22 @@ EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_8_x11);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_ddraw);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_gdi);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_generic);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_qtopia);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_sdl);
 EVAS_EINA_STATIC_MODULE_DEFINE(engine, software_x11);
-EVAS_EINA_STATIC_MODULE_DEFINE(engine, xrender_x11);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, xpm);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, bmp);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, tiff);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, svg);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, png);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, pmaps);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, jpeg);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, gif);
-EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, eet);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, edb);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, eet);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, generic);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, gif);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, ico);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, jpeg);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, pmaps);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, png);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, psd);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, svg);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, tga);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, tiff);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, wbmp);
+EVAS_EINA_STATIC_MODULE_DEFINE(image_loader, xpm);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_saver, edb);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_saver, eet);
 EVAS_EINA_STATIC_MODULE_DEFINE(image_saver, jpeg);
@@ -137,9 +127,6 @@ static const struct {
 #ifdef EVAS_STATIC_BUILD_BUFFER
   EVAS_EINA_STATIC_MODULE_USE(engine, buffer),
 #endif
-#ifdef EVAS_STATIC_BUILD_CAIRO_X11
-  EVAS_EINA_STATIC_MODULE_USE(engine, cairo_x11),
-#endif
 #ifdef EVAS_STATIC_BUILD_DIRECT3D
   EVAS_EINA_STATIC_MODULE_USE(engine, direct3d),
 #endif
@@ -149,17 +136,14 @@ static const struct {
 #ifdef EVAS_STATIC_BUILD_FB
   EVAS_EINA_STATIC_MODULE_USE(engine, fb),
 #endif
-#ifdef EVAS_STATIC_BUILD_GL_GLEW
-  EVAS_EINA_STATIC_MODULE_USE(engine, gl_glew),
-#endif
 #ifdef EVAS_STATIC_BUILD_GL_X11
   EVAS_EINA_STATIC_MODULE_USE(engine, gl_x11),
 #endif
 #ifdef EVAS_STATIC_BUILD_GL_SDL
   EVAS_EINA_STATIC_MODULE_USE(engine, gl_sdl),
 #endif
-#ifdef EVAS_STATIC_BUILD_QUARTZ
-  EVAS_EINA_STATIC_MODULE_USE(engine, quartz),
+#ifdef EVAS_STATIC_BUILD_PSL1GHT
+  EVAS_EINA_STATIC_MODULE_USE(engine, psl1ght),
 #endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_16
   EVAS_EINA_STATIC_MODULE_USE(engine, software_16),
@@ -191,58 +175,61 @@ static const struct {
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_GENERIC
   EVAS_EINA_STATIC_MODULE_USE(engine, software_generic),
 #endif
-#ifdef EVAS_STATIC_BUILD_SOFTWARE_QTOPIA
-  EVAS_EINA_STATIC_MODULE_USE(engine, software_qtopia),
-#endif
-#ifdef EVAS_STATIC_BUILD_SOFTWARE_SDL
-  EVAS_EINA_STATIC_MODULE_USE(engine, software_sdl),
-#endif
 #ifdef EVAS_STATIC_BUILD_SOFTWARE_X11
   EVAS_EINA_STATIC_MODULE_USE(engine, software_x11),
 #endif
-#ifdef EVAS_STATIC_BUILD_XRENDER_X11
-  EVAS_EINA_STATIC_MODULE_USE(engine, xrender_x11),
-#endif
-#ifdef EVAS_STATIC_BUILD_XPM
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, xpm),
-#endif
 #ifdef EVAS_STATIC_BUILD_BMP
   EVAS_EINA_STATIC_MODULE_USE(image_loader, bmp),
 #endif
-#ifdef EVAS_STATIC_BUILD_TIFF
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, tiff),
-#endif
-#ifdef EVAS_STATIC_BUILD_SVG
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, svg),
-#endif
-#ifdef EVAS_STATIC_BUILD_PNG
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, png),
-#endif
-#ifdef EVAS_STATIC_BUILD_PMAPS
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, pmaps),
-#endif
-#ifdef EVAS_STATIC_BUILD_JPEG
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, jpeg),
-#endif
-#ifdef EVAS_STATIC_BUILD_GIF
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, gif),
+#ifdef EVAS_STATIC_BUILD_EDB
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, edb),
 #endif
 #ifdef EVAS_STATIC_BUILD_EET
   EVAS_EINA_STATIC_MODULE_USE(image_loader, eet),
 #endif
-#ifdef EVAS_STATIC_BUILD_EDB
-  EVAS_EINA_STATIC_MODULE_USE(image_loader, edb),
+#ifdef EVAS_STATIC_BUILD_GENERIC
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, generic),
+#endif
+#ifdef EVAS_STATIC_BUILD_GIF
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, gif),
+#endif
+#ifdef EVAS_STATIC_BUILD_ICO
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, ico),
+#endif
+#ifdef EVAS_STATIC_BUILD_JPEG
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, jpeg),
+#endif
+#ifdef EVAS_STATIC_BUILD_PMAPS
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, pmaps),
+#endif
+#ifdef EVAS_STATIC_BUILD_PNG
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, png),
+#endif
+#ifdef EVAS_STATIC_BUILD_PSD
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, psd),
+#endif
+#ifdef EVAS_STATIC_BUILD_SVG
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, svg),
 #endif
 #ifdef EVAS_STATIC_BUILD_TGA
   EVAS_EINA_STATIC_MODULE_USE(image_loader, tga),
 #endif
+#ifdef EVAS_STATIC_BUILD_TIFF
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, tiff),
+#endif
+#ifdef EVAS_STATIC_BUILD_WBMP
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, wbmp),
+#endif
+#ifdef EVAS_STATIC_BUILD_XPM
+  EVAS_EINA_STATIC_MODULE_USE(image_loader, xpm),
+#endif
 #ifdef EVAS_STATIC_BUILD_EDB
   EVAS_EINA_STATIC_MODULE_USE(image_saver, edb),
 #endif
 #ifdef EVAS_STATIC_BUILD_EET
   EVAS_EINA_STATIC_MODULE_USE(image_saver, eet),
 #endif
-#ifdef EVAS_STATIC_BUILD_JPEG
+#if defined (EVAS_BUILD_SAVER_JPEG) && defined (EVAS_STATIC_BUILD_JPEG)
   EVAS_EINA_STATIC_MODULE_USE(image_saver, jpeg),
 #endif
 #ifdef EVAS_STATIC_BUILD_PNG
@@ -294,7 +281,7 @@ evas_module_register(const Evas_Module_Api *module, Evas_Module_Type type)
    if (type == EVAS_MODULE_TYPE_ENGINE)
      {
 	eina_array_push(evas_engines, em);
-	em->id_engine = eina_array_count_get(evas_engines);
+	em->id_engine = eina_array_count(evas_engines);
      }
 
    eina_hash_direct_add(evas_modules[type], module->name, em);
@@ -302,6 +289,59 @@ evas_module_register(const Evas_Module_Api *module, Evas_Module_Type type)
    return EINA_TRUE;
 }
 
+Eina_List *
+evas_module_engine_list(void)
+{
+   Evas_Module *em;
+   Eina_List *r = NULL, *l, *ll;
+   Eina_Array_Iterator iterator;
+   Eina_Iterator *it, *it2;
+   unsigned int i;
+   const char *s, *s2;
+   char buf[4096];
+
+   EINA_LIST_FOREACH(evas_module_paths, l, s)
+     {
+        snprintf(buf, sizeof(buf), "%s/engines", s);
+        it = eina_file_direct_ls(buf);
+        if (it)
+          {
+             Eina_File_Direct_Info *fi;
+
+             EINA_ITERATOR_FOREACH(it, fi)
+               {
+                  const char *fname = fi->path + fi->name_start;
+                  snprintf(buf, sizeof(buf), "%s/engines/%s/%s", 
+                           s, fname, MODULE_ARCH);
+                  it2 = eina_file_ls(buf);
+                  if (it2)
+                    {
+                       EINA_LIST_FOREACH(r, ll, s2)
+                         {
+                            if (!strcmp(fname, s2)) break;
+                         }
+                       if (!ll)
+                         r = eina_list_append(r, eina_stringshare_add(fname));
+                       eina_iterator_free(it2);
+                    }
+               }
+             eina_iterator_free(it);
+          }
+     }
+   
+   EINA_ARRAY_ITER_NEXT(evas_engines, i, em, iterator)
+     {
+        EINA_LIST_FOREACH(r, ll, s2)
+          {
+             if (!strcmp(em->definition->name, s2)) break;
+          }
+        if (!ll)
+          r = eina_list_append(r, eina_stringshare_add(em->definition->name));
+     }
+
+   return r;
+}
+
 Eina_Bool
 evas_module_unregister(const Evas_Module_Api *module, Evas_Module_Type type)
 {
@@ -393,7 +433,7 @@ Evas_Module *
 evas_module_engine_get(int render_method)
 {
    if ((render_method <= 0) ||
-       ((unsigned int)render_method > eina_array_count_get(evas_engines)))
+       ((unsigned int)render_method > eina_array_count(evas_engines)))
      return NULL;
    return eina_array_data_get(evas_engines, render_method - 1);
 }
@@ -567,3 +607,15 @@ _evas_module_engine_inherit(Evas_Func *funcs, char *name)
      }
    return 0;
 }
+
+static Eina_Prefix *pfx = NULL;
+
+EAPI const char *
+_evas_module_libdir_get(void)
+{
+   if (!pfx) pfx = eina_prefix_new
+      (NULL, _evas_module_libdir_get, "EVAS", "evas", NULL,
+       PACKAGE_BIN_DIR, PACKAGE_LIB_DIR, PACKAGE_DATA_DIR, PACKAGE_DATA_DIR);
+   if (!pfx) return NULL;
+   return eina_prefix_lib_get(pfx);
+}
