@@ -11,7 +11,9 @@
 
 #include "../file/evas_module.h"
 #include "../file/evas_path.h"
-#include "../engines/common/evas_bidi_utils.h"
+#include "../engines/common/evas_text_utils.h"
+#include "../engines/common/language/evas_bidi_utils.h"
+#include "../engines/common/language/evas_language_utils.h"
 
 #ifdef EVAS_MAGIC_DEBUG
 /* complain when peole pass in wrong object types etc. */
@@ -29,6 +31,7 @@ typedef struct _Evas_Size_Hints             Evas_Size_Hints;
 typedef struct _Evas_Font_Dir               Evas_Font_Dir;
 typedef struct _Evas_Font                   Evas_Font;
 typedef struct _Evas_Font_Alias             Evas_Font_Alias;
+typedef struct _Evas_Font_Description       Evas_Font_Description;
 typedef struct _Evas_Data_Node              Evas_Data_Node;
 typedef struct _Evas_Func_Node              Evas_Func_Node;
 typedef RGBA_Image_Loadopts                 Evas_Image_Load_Opts;
@@ -43,6 +46,62 @@ typedef struct _Evas_Format                 Evas_Format;
 typedef struct _Evas_Map_Point              Evas_Map_Point;
 typedef struct _Evas_Smart_Cb_Description_Array Evas_Smart_Cb_Description_Array;
 typedef struct _Evas_Post_Callback          Evas_Post_Callback;
+typedef struct _Evas_Coord_Touch_Point      Evas_Coord_Touch_Point;
+
+enum _Evas_Font_Style
+{
+   EVAS_FONT_STYLE_SLANT,
+   EVAS_FONT_STYLE_WEIGHT,
+   EVAS_FONT_STYLE_WIDTH
+};
+
+enum _Evas_Font_Slant
+{
+   EVAS_FONT_SLANT_NORMAL,
+   EVAS_FONT_SLANT_OBLIQUE,
+   EVAS_FONT_SLANT_ITALIC
+};
+
+enum _Evas_Font_Weight
+{
+   EVAS_FONT_WEIGHT_NORMAL,
+   EVAS_FONT_WEIGHT_THIN,
+   EVAS_FONT_WEIGHT_ULTRALIGHT,
+   EVAS_FONT_WEIGHT_LIGHT,
+   EVAS_FONT_WEIGHT_BOOK,
+   EVAS_FONT_WEIGHT_MEDIUM,
+   EVAS_FONT_WEIGHT_SEMIBOLD,
+   EVAS_FONT_WEIGHT_BOLD,
+   EVAS_FONT_WEIGHT_ULTRABOLD,
+   EVAS_FONT_WEIGHT_BLACK,
+   EVAS_FONT_WEIGHT_EXTRABLACK
+};
+
+enum _Evas_Font_Width
+{
+   EVAS_FONT_WIDTH_NORMAL,
+   EVAS_FONT_WIDTH_ULTRACONDENSED,
+   EVAS_FONT_WIDTH_EXTRACONDENSED,
+   EVAS_FONT_WIDTH_CONDENSED,
+   EVAS_FONT_WIDTH_SEMICONDENSED,
+   EVAS_FONT_WIDTH_SEMIEXPANDED,
+   EVAS_FONT_WIDTH_EXPANDED,
+   EVAS_FONT_WIDTH_EXTRAEXPANDED,
+   EVAS_FONT_WIDTH_ULTRAEXPANDED
+};
+
+typedef enum _Evas_Font_Style               Evas_Font_Style;
+typedef enum _Evas_Font_Slant               Evas_Font_Slant;
+typedef enum _Evas_Font_Weight              Evas_Font_Weight;
+typedef enum _Evas_Font_Width               Evas_Font_Width;
+
+/* General types - used for script type chceking */
+#define OPAQUE_TYPE(type) struct __##type { int a; }; \
+   typedef struct __##type type
+
+OPAQUE_TYPE(Evas_Font_Set); /* General type for RGBA_Font */
+OPAQUE_TYPE(Evas_Font_Instance); /* General type for RGBA_Font_Int */
+/* End of general types */
 
 #define MAGIC_EVAS                 0x70777770
 #define MAGIC_OBJ                  0x71777770
@@ -57,6 +116,8 @@ typedef struct _Evas_Post_Callback          Evas_Post_Callback;
 #define MAGIC_OBJ_SHAPE            0x72777773
 #define MAGIC_OBJ_CONTAINER        0x72777774
 #define MAGIC_OBJ_CUSTOM           0x72777775
+#define MAGIC_EVAS_GL              0x72777776
+#define MAGIC_MAP                  0x72777777
 
 #ifdef MAGIC_DEBUG
 # define MAGIC_CHECK_FAILED(o, t, m) \
@@ -84,38 +145,45 @@ MAGIC_CHECK_FAILED(o, t, m)
 #define EVAS_OBJECT_IMAGE_FREE_FILE_AND_KEY(o)                              \
    if ((o)->cur.file)                                                       \
      {                                                                      \
-         eina_stringshare_del((o)->cur.file);                               \
-	 if ((o)->prev.file == (o)->cur.file)                               \
-	       (o)->prev.file = NULL;                                       \
-	 (o)->cur.file = NULL;                                              \
+        eina_stringshare_del((o)->cur.file);                                \
+        if ((o)->prev.file == (o)->cur.file)                                \
+          (o)->prev.file = NULL;                                            \
+        (o)->cur.file = NULL;                                               \
      }                                                                      \
    if ((o)->cur.key)                                                        \
      {                                                                      \
-         eina_stringshare_del((o)->cur.key);                                \
-	 if ((o)->prev.key == (o)->cur.key)                                 \
-	       (o)->prev.key = NULL;                                        \
-	 (o)->cur.key = NULL;                                               \
+        eina_stringshare_del((o)->cur.key);                                 \
+        if ((o)->prev.key == (o)->cur.key)                                  \
+          (o)->prev.key = NULL;                                             \
+        (o)->cur.key = NULL;                                                \
      }                                                                      \
    if ((o)->prev.file)                                                      \
      {                                                                      \
-         eina_stringshare_del((o)->prev.file);                              \
-	 (o)->prev.file = NULL;                                             \
+        eina_stringshare_del((o)->prev.file);                               \
+        (o)->prev.file = NULL;                                              \
      }                                                                      \
    if ((o)->prev.key)                                                       \
      {                                                                      \
-         eina_stringshare_del((o)->prev.key);                               \
-	 (o)->prev.key = NULL;                                              \
+        eina_stringshare_del((o)->prev.key);                                \
+        (o)->prev.key = NULL;                                               \
      }
 
+struct _Evas_Coord_Touch_Point
+{
+   Evas_Coord x, y; // point's x, y position
+   int id; // id in order to distinguish each point
+   Evas_Touch_Point_State state;
+};
+
 struct _Evas_Key_Grab
 {
    char               *keyname;
    Evas_Modifier_Mask  modifiers;
    Evas_Modifier_Mask  not_modifiers;
    Evas_Object        *object;
-   unsigned char       exclusive : 1;
-   unsigned char       just_added : 1;
-   unsigned char       delete_me : 1;
+   Eina_Bool           exclusive : 1;
+   Eina_Bool           just_added : 1;
+   Eina_Bool           delete_me : 1;
 };
 
 struct _Evas_Intercept_Func
@@ -233,12 +301,13 @@ struct _Evas
    struct {
       unsigned char  inside : 1;
       int            mouse_grabbed;
+      int            downs;
       DATA32         button;
       Evas_Coord     x, y;
+      int            nogrep;
       struct {
-	 Eina_List *in;
+          Eina_List *in;
       } object;
-
    } pointer;
 
    struct  {
@@ -252,6 +321,12 @@ struct _Evas
       unsigned char  changed : 1;
    } output;
 
+   struct 
+     {
+        Evas_Coord x, y, w, h;
+        Eina_Bool changed : 1;
+     } framespace;
+
    Eina_List        *damages;
    Eina_List        *obscures;
 
@@ -263,14 +338,15 @@ struct _Evas
 
    int               walking_list;
    int               events_frozen;
+   Evas_Event_Flags  default_event_flags;
 
    struct {
       Evas_Module *module;
       Evas_Func *func;
       struct {
-	 void *output;
+         void *output;
 
-	 void *context;
+         void *context;
       } data;
 
       void *info;
@@ -287,6 +363,10 @@ struct _Evas
    Eina_Array     calculate_objects;
    Eina_Array     clip_changes;
 
+   Eina_Clist     calc_list;
+   Eina_Clist     calc_done;
+   Eina_List     *video_objects;
+
    Eina_List     *post_events; // free me on evas_free
 
    Evas_Callbacks *callbacks;
@@ -297,6 +377,9 @@ struct _Evas
 
    Eina_List     *font_path;
 
+   int            in_smart_calc;
+   int            smart_calc_count;
+
    Evas_Object   *focused;
    void          *attach_data;
    Evas_Modifier  modifiers;
@@ -311,6 +394,8 @@ struct _Evas
    unsigned char  invalidate : 1;
    unsigned char  cleanup : 1;
    unsigned char  focus : 1;
+
+   Eina_List     *touch_points;
 };
 
 struct _Evas_Layer
@@ -365,6 +450,7 @@ struct _Evas_Map_Point
 
 struct _Evas_Map
 {
+   DATA32                magic;
    int                   count; // num of points
    Evas_Coord_Rectangle  normal_geometry; // bounding box of map geom actually
    void                 *surface; // surface holding map if needed
@@ -378,6 +464,35 @@ struct _Evas_Map
    Evas_Map_Point        points[]; // actual points
 };
 
+#if 0 // filtering disabled
+/* nash: Split into two bits */
+typedef struct Evas_Filter_Info
+{
+   Evas_Filter filter;
+   Evas_Filter_Mode mode;
+
+   Eina_Bool dirty : 1;
+
+   int datalen;
+   void *data;
+   void (*data_free)(void *);
+
+   uint8_t *key;
+   uint32_t len;
+   Filtered_Image *cached;
+} Evas_Filter_Info;
+
+typedef Eina_Bool (*Evas_Software_Filter_Fn)(Evas_Filter_Info *, RGBA_Image *, RGBA_Image *);
+
+int evas_filter_get_size(Evas_Filter_Info *info, int inw, int inh,
+                     int *outw, int *outh, Eina_Bool inv);
+Eina_Bool evas_filter_always_alpha(Evas_Filter_Info *info);
+uint8_t *evas_filter_key_get(const Evas_Filter_Info *info, uint32_t *lenp);
+// expose for use in engines
+EAPI Evas_Software_Filter_Fn evas_filter_software_get(Evas_Filter_Info *info);
+void evas_filter_free(Evas_Object *o);
+#endif
+
 struct _Evas_Object
 {
    EINA_INLIST;
@@ -390,20 +505,21 @@ struct _Evas_Object
    struct {
       Evas_Map             *map;
       Evas_Object          *clipper;
+      Evas_Object          *mask;
       Evas_Object          *map_parent;
       double                scale;
       Evas_Coord_Rectangle  geometry;
       struct {
-	 struct {
-	    Evas_Coord      x, y, w, h;
-	    unsigned char   r, g, b, a;
-	    Eina_Bool       visible : 1;
-	    Eina_Bool       dirty : 1;
-	 } clip;
+         struct {
+            Evas_Coord      x, y, w, h;
+            unsigned char   r, g, b, a;
+            Eina_Bool       visible : 1;
+            Eina_Bool       dirty : 1;
+         } clip;
       } cache;
       short                 layer;
       struct {
-	 unsigned char      r, g, b, a;
+         unsigned char      r, g, b, a;
       } color;
       Eina_Bool             usemap : 1;
       Eina_Bool             visible : 1;
@@ -438,6 +554,17 @@ struct _Evas_Object
       Evas_Object             *parent;
    } smart;
 
+   struct {
+      Eina_List               *proxies;
+      void                    *surface;
+      int                      w,h;
+      Eina_Bool                redraw;
+   } proxy;
+
+#if 0 // filtering disabled
+   Evas_Filter_Info           *filter;
+#endif
+
    Evas_Size_Hints            *size_hints;
 
    int                         last_mouse_down_counter;
@@ -445,20 +572,31 @@ struct _Evas_Object
    int                         mouse_grabbed;
 
    int                         last_event;
-   
+   Evas_Callback_Type          last_event_type;
+
    struct {
-      int                      in_move, in_resize;
+        int                      in_move, in_resize;
    } doing;
 
+   unsigned int                ref;
+
    unsigned char               delete_me;
 
-   Evas_Object_Pointer_Mode    pointer_mode : 1;
+   unsigned char               recalculate_cycle;
+   Eina_Clist                  calc_entry;
+
+   Evas_Object_Pointer_Mode    pointer_mode : 2;
 
    Eina_Bool                   store : 1;
    Eina_Bool                   pass_events : 1;
-   Eina_Bool                   parent_pass_events : 1;
-   Eina_Bool                   parent_cache_valid : 1;
+   Eina_Bool                   freeze_events : 1;
    Eina_Bool                   repeat_events : 1;
+   struct  {
+      Eina_Bool                pass_events : 1;
+      Eina_Bool                pass_events_valid : 1;
+      Eina_Bool                freeze_events : 1;
+      Eina_Bool                freeze_events_valid : 1;
+   } parent_cache;
    Eina_Bool                   restack : 1;
    Eina_Bool                   is_active : 1;
    Eina_Bool                   precise_is_inside : 1;
@@ -477,6 +615,11 @@ struct _Evas_Object
    Eina_Bool                   changed_move : 1;
    Eina_Bool                   changed_move_only : 1;
    Eina_Bool                   changed_nomove : 1;
+   Eina_Bool                   changed_map : 1;
+   Eina_Bool                   changed_pchange : 1;
+   Eina_Bool                   del_ref : 1;
+
+   Eina_Bool                   is_frame : 1;
 };
 
 struct _Evas_Func_Node
@@ -485,6 +628,7 @@ struct _Evas_Func_Node
    void (*func) ();
    void *data;
    Evas_Callback_Type type;
+   Evas_Callback_Priority priority;
    unsigned char delete_me : 1;
 };
 
@@ -522,6 +666,21 @@ struct _Evas_Font_Alias
    Evas_Font  *fn;
 };
 
+struct _Evas_Font_Description
+{
+   int ref;
+   /* We assume everywhere this is stringshared */
+   const char *name;
+   const char *fallbacks;
+   const char *lang;
+
+   Evas_Font_Slant slant;
+   Evas_Font_Weight weight;
+   Evas_Font_Width width;
+
+   Eina_Bool is_new : 1;
+};
+
 struct _Evas_Object_Func
 {
    void (*free) (Evas_Object *obj);
@@ -580,6 +739,8 @@ struct _Evas_Func
    void (*context_clip_clip)               (void *data, void *context, int x, int y, int w, int h);
    void (*context_clip_unset)              (void *data, void *context);
    int  (*context_clip_get)                (void *data, void *context, int *x, int *y, int *w, int *h);
+   void (*context_mask_set)                (void *data, void *context, void *mask, int x, int y, int w, int h);
+   void (*context_mask_unset)              (void *data, void *context);
    void (*context_color_set)               (void *data, void *context, int r, int g, int b, int a);
    int  (*context_color_get)               (void *data, void *context, int *r, int *g, int *b, int *a);
    void (*context_multiplier_set)          (void *data, void *context, int r, int g, int b, int a);
@@ -610,7 +771,7 @@ struct _Evas_Func
    void *(*image_size_set)                 (void *data, void *image, int w, int h);
    void (*image_stride_get)                (void *data, void *image, int *stride);
    void *(*image_dirty_region)             (void *data, void *image, int x, int y, int w, int h);
-   void *(*image_data_get)                 (void *data, void *image, int to_write, DATA32 **image_data);
+   void *(*image_data_get)                 (void *data, void *image, int to_write, DATA32 **image_data, int *err);
    void *(*image_data_put)                 (void *data, void *image, DATA32 *image_data);
    void  (*image_data_preload_request)     (void *data, void *image, const void *target);
    void  (*image_data_preload_cancel)      (void *data, void *image, const void *target);
@@ -623,6 +784,8 @@ struct _Evas_Func
    char *(*image_format_get)               (void *data, void *image);
    void (*image_colorspace_set)            (void *data, void *image, int cspace);
    int  (*image_colorspace_get)            (void *data, void *image);
+   Eina_Bool (*image_can_region_get)       (void *data, void *image);
+   void (*image_mask_create)               (void *data, void *image);
    void *(*image_native_set)               (void *data, void *image, void *native);
    void *(*image_native_get)               (void *data, void *image);
 
@@ -630,22 +793,22 @@ struct _Evas_Func
    void (*image_cache_set)                 (void *data, int bytes);
    int  (*image_cache_get)                 (void *data);
 
-   void *(*font_load)                      (void *data, const char *name, int size);
-   void *(*font_memory_load)               (void *data, char *name, int size, const void *fdata, int fdata_size);
-   void *(*font_add)                       (void *data, void *font, const char *name, int size);
-   void *(*font_memory_add)                (void *data, void *font, char *name, int size, const void *fdata, int fdata_size);
-   void (*font_free)                       (void *data, void *font);
-   int  (*font_ascent_get)                 (void *data, void *font);
-   int  (*font_descent_get)                (void *data, void *font);
-   int  (*font_max_ascent_get)             (void *data, void *font);
-   int  (*font_max_descent_get)            (void *data, void *font);
-   void (*font_string_size_get)            (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int *w, int *h);
-   int  (*font_inset_get)                  (void *data, void *font, const Eina_Unicode *text);
-   int  (*font_h_advance_get)              (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
-   int  (*font_v_advance_get)              (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
-   int  (*font_char_coords_get)            (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int pos, int *cx, int *cy, int *cw, int *ch);
-   int  (*font_char_at_coords_get)         (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int x, int y, int *cx, int *cy, int *cw, int *ch);
-   void (*font_draw)                       (void *data, void *context, void *surface, void *font, int x, int y, int w, int h, int ow, int oh, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props);
+   Evas_Font_Set *(*font_load)             (void *data, const char *name, int size, Font_Rend_Flags wanted_rend);
+   Evas_Font_Set *(*font_memory_load)      (void *data, char *name, int size, const void *fdata, int fdata_size, Font_Rend_Flags wanted_rend);
+   Evas_Font_Set *(*font_add)              (void *data, Evas_Font_Set *font, const char *name, int size, Font_Rend_Flags wanted_rend);
+   Evas_Font_Set *(*font_memory_add)       (void *data, Evas_Font_Set *font, char *name, int size, const void *fdata, int fdata_size, Font_Rend_Flags wanted_rend);
+   void (*font_free)                       (void *data, Evas_Font_Set *font);
+   int  (*font_ascent_get)                 (void *data, Evas_Font_Set *font);
+   int  (*font_descent_get)                (void *data, Evas_Font_Set *font);
+   int  (*font_max_ascent_get)             (void *data, Evas_Font_Set *font);
+   int  (*font_max_descent_get)            (void *data, Evas_Font_Set *font);
+   void (*font_string_size_get)            (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props, int *w, int *h);
+   int  (*font_inset_get)                  (void *data, Evas_Font_Set *font, const Evas_Text_Props *text_props);
+   int  (*font_h_advance_get)              (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props);
+   int  (*font_v_advance_get)              (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props);
+   int  (*font_char_coords_get)            (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props, int pos, int *cx, int *cy, int *cw, int *ch);
+   int  (*font_char_at_coords_get)         (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props, int x, int y, int *cx, int *cy, int *cw, int *ch);
+   void (*font_draw)                       (void *data, void *context, void *surface, Evas_Font_Set *font, int x, int y, int w, int h, int ow, int oh, const Evas_Text_Props *intl_props);
 
    void (*font_cache_flush)                (void *data);
    void (*font_cache_set)                  (void *data, int bytes);
@@ -653,21 +816,57 @@ struct _Evas_Func
 
    /* Engine functions will over time expand from here */
 
-   void (*font_hinting_set)                (void *data, void *font, int hinting);
+   void (*font_hinting_set)                (void *data, Evas_Font_Set *font, int hinting);
    int  (*font_hinting_can_hint)           (void *data, int hinting);
 
 /*    void (*image_rotation_set)              (void *data, void *image); */
 
    void (*image_scale_hint_set)            (void *data, void *image, int hint);
    int  (*image_scale_hint_get)            (void *data, void *image);
-   int  (*font_last_up_to_pos)             (void *data, void *font, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props, int x, int y);
+   int  (*font_last_up_to_pos)             (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props, int x, int y);
 
-   void (*image_map4_draw)                 (void *data, void *context, void *surface, void *image, RGBA_Map_Point *p, int smooth, int level);
+   void (*image_map_draw)                  (void *data, void *context, void *surface, void *image, int npoints, RGBA_Map_Point *p, int smooth, int level);
    void *(*image_map_surface_new)          (void *data, int w, int h, int alpha);
    void (*image_map_surface_free)          (void *data, void *surface);
 
    void (*image_content_hint_set)          (void *data, void *surface, int hint);
    int  (*image_content_hint_get)          (void *data, void *surface);
+   int  (*font_pen_coords_get)             (void *data, Evas_Font_Set *font, const Evas_Text_Props *intl_props, int pos, int *cpen_x, int *cy, int *cadv, int *ch);
+   Eina_Bool (*font_text_props_info_create) (void *data __UNUSED__, Evas_Font_Instance *fi, const Eina_Unicode *text, Evas_Text_Props *intl_props, const Evas_BiDi_Paragraph_Props *par_props, size_t pos, size_t len);
+   int  (*font_right_inset_get)            (void *data, Evas_Font_Set *font, const Evas_Text_Props *text_props);
+
+#if 0 // filtering disabled
+   void (*image_draw_filtered)             (void *data, void *context, void *surface, void *image, Evas_Filter_Info *filter);
+   Filtered_Image *(*image_filtered_get)   (void *image, uint8_t *key, size_t len);
+   Filtered_Image *(*image_filtered_save)  (void *image, void *filtered, uint8_t *key, size_t len);
+   void (*image_filtered_free)             (void *image, Filtered_Image *);
+#endif
+
+   /* EFL-GL Glue Layer */
+   void *(*gl_surface_create)            (void *data, void *config, int w, int h);
+   int  (*gl_surface_destroy)            (void *data, void *surface);
+   void *(*gl_context_create)            (void *data, void *share_context);
+   int  (*gl_context_destroy)            (void *data, void *context);
+   int  (*gl_make_current)               (void *data, void *surface, void *context);
+   void *(*gl_string_query)              (void *data, int name);
+   void *(*gl_proc_address_get)          (void *data, const char *name);
+   int  (*gl_native_surface_get)         (void *data, void *surface, void *native_surface);
+   void *(*gl_api_get)                   (void *data);
+   void (*gl_img_obj_set)                (void *data, void *image, int has_alpha);
+
+   int  (*image_load_error_get)          (void *data, void *image);
+   int  (*font_run_end_get)              (void *data, Evas_Font_Set *font, Evas_Font_Instance **script_fi, Evas_Font_Instance **cur_fi, Evas_Script_Type script, const Eina_Unicode *text, int run_len);
+
+   /* animated feature */
+   Eina_Bool (*image_animated_get)       (void *data, void *image);
+   int (*image_animated_frame_count_get) (void *data, void *image);
+   Evas_Image_Animated_Loop_Hint  (*image_animated_loop_type_get) (void *data, void *image);
+   int (*image_animated_loop_count_get)  (void *data, void *image);
+   double (*image_animated_frame_duration_get) (void *data, void *image, int start_frame, int frame_num);
+   Eina_Bool (*image_animated_frame_set) (void *data, void *image, int frame_index);
+
+   /* max size query */
+   void (*image_max_size_get)            (void *data, int *maxw, int *maxh);
 };
 
 struct _Evas_Image_Load_Func
@@ -675,6 +874,8 @@ struct _Evas_Image_Load_Func
   Eina_Bool threadable;
   Eina_Bool (*file_head) (Image_Entry *ie, const char *file, const char *key, int *error);
   Eina_Bool (*file_data) (Image_Entry *ie, const char *file, const char *key, int *error);
+  double    (*frame_duration) (Image_Entry *ie, const char *file, const int start, const int frame_num);
+  Eina_Bool do_region;
 };
 
 struct _Evas_Image_Save_Func
@@ -718,7 +919,7 @@ void evas_object_clip_across_check(Evas_Object *obj);
 void evas_object_clip_across_clippees_check(Evas_Object *obj);
 void evas_object_mapped_clip_across_mark(Evas_Object *obj);
 void evas_event_callback_call(Evas *e, Evas_Callback_Type type, void *event_info);
-void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info);
+void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info, int event_id);
 Eina_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
 int evas_mem_free(int mem_required);
 int evas_mem_degrade(int mem_required);
@@ -734,6 +935,12 @@ void evas_smart_cb_descriptions_fix(Evas_Smart_Cb_Description_Array *a) EINA_ARG
 Eina_Bool evas_smart_cb_descriptions_resize(Evas_Smart_Cb_Description_Array *a, unsigned int size) EINA_ARG_NONNULL(1);
 const Evas_Smart_Cb_Description *evas_smart_cb_description_find(const Evas_Smart_Cb_Description_Array *a, const char *name) EINA_ARG_NONNULL(1, 2) EINA_PURE;
 
+Eina_Bool _evas_object_image_preloading_get(const Evas_Object *obj);
+void _evas_object_image_preloading_set(Evas_Object *obj, Eina_Bool preloading);
+void _evas_object_image_preloading_check(Evas_Object *obj);
+Evas_Object *_evas_object_image_video_parent_get(Evas_Object *obj);
+void _evas_object_image_video_overlay_show(Evas_Object *obj);
+void _evas_object_image_video_overlay_hide(Evas_Object *obj);
 void evas_object_smart_del(Evas_Object *obj);
 void evas_object_smart_cleanup(Evas_Object *obj);
 void evas_object_smart_member_raise(Evas_Object *member);
@@ -781,9 +988,16 @@ void evas_font_dir_available_list_free(Eina_List *available);
 void evas_font_free(Evas *evas, void *font);
 void evas_fonts_zero_free(Evas *evas);
 void evas_fonts_zero_presure(Evas *evas);
-void *evas_font_load(Evas *evas, const char *name, const char *source, int size);
+void evas_font_name_parse(Evas_Font_Description *fdesc, const char *name);
+int evas_font_style_find(const char *start, const char *end, Evas_Font_Style style);
+Evas_Font_Description *evas_font_desc_new(void);
+Evas_Font_Description *evas_font_desc_dup(const Evas_Font_Description *fdesc);
+void evas_font_desc_unref(Evas_Font_Description *fdesc);
+int evas_font_desc_cmp(const Evas_Font_Description *a, const Evas_Font_Description *b);
+Evas_Font_Description *evas_font_desc_ref(Evas_Font_Description *fdesc);
+void * evas_font_load(Evas *evas, Evas_Font_Description *fdesc, const char *source, Evas_Font_Size size);
 void evas_font_load_hinting_set(Evas *evas, void *font, int hinting);
-void evas_object_smart_member_cache_invalidate(Evas_Object *obj);
+void evas_object_smart_member_cache_invalidate(Evas_Object *obj, Eina_Bool pass_events, Eina_Bool freeze_events);
 void evas_text_style_pad_get(Evas_Text_Style_Type style, int *l, int *r, int *t, int *b);
 void _evas_object_text_rehint(Evas_Object *obj);
 void _evas_object_textblock_rehint(Evas_Object *obj);
@@ -808,30 +1022,43 @@ int evas_async_target_del(const void *target);
 void _evas_preload_thread_init(void);
 void _evas_preload_thread_shutdown(void);
 Evas_Preload_Pthread *evas_preload_thread_run(void (*func_heavy)(void *data),
-					     void (*func_end)(void *data),
-					     void (*func_cancel)(void *data),
-					     const void *data);
+                                              void (*func_end)(void *data),
+                                              void (*func_cancel)(void *data),
+                                              const void *data);
 Eina_Bool evas_preload_thread_cancel(Evas_Preload_Pthread *thread);
 
 void _evas_walk(Evas *e);
 void _evas_unwalk(Evas *e);
 
+// expose for use in engines
 EAPI int _evas_module_engine_inherit(Evas_Func *funcs, char *name);
-
+EAPI const char *_evas_module_libdir_get(void);
+         
+Eina_Bool evas_render_mapped(Evas *e, Evas_Object *obj, 
+                             void *context, void *surface,
+                             int off_x, int off_y, int mapped,
+                             int ecx, int ecy, int ecw, int ech);
 void evas_render_invalidate(Evas *e);
 void evas_render_object_recalc(Evas_Object *obj);
 
 Eina_Bool evas_map_inside_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y);
 Eina_Bool evas_map_coords_get(const Evas_Map *m, Evas_Coord x, Evas_Coord y, Evas_Coord *mx, Evas_Coord *my, int grab);
 
+Eina_List *evas_module_engine_list(void);
+
+/* for updating touch point list */
+void _evas_touch_point_append(Evas *e, int id, Evas_Coord x, Evas_Coord y);
+void _evas_touch_point_update(Evas *e, int id, Evas_Coord x, Evas_Coord y, Evas_Touch_Point_State state);
+void _evas_touch_point_remove(Evas *e, int id);
+
 /****************************************************************************/
 /*****************************************/
 /********************/
-//#define MPOOL 1
+#define MPOOL 1
 
 #ifdef MPOOL 
 typedef struct _Evas_Mempool Evas_Mempool;
-  
+
 struct _Evas_Mempool
 {
   int           count;
@@ -877,13 +1104,15 @@ struct _Evas_Mempool
    calloc(1, sizeof(siz))
 # define EVAS_MEMPOOL_FREE(x, p) \
    free(p)
-#endif  
+#endif
 /********************/
 /*****************************************/
 /****************************************************************************/
-  
+
 #define EVAS_API_OVERRIDE(func, api, prefix) \
      (api)->func = prefix##func
+#define EVAS_API_RESET(func, api) \
+     (api)->func = NULL
 
 #include "evas_inline.x"
 
