@@ -14,15 +14,42 @@ extern "C" {
 #define EVAS_INLINE_ARRAY_H  // We dont need that and it is buggy
 
 #include "evas_common.h"
-#include "evas_private.h"
 
 #ifdef __cplusplus
 }
 #endif
 
+
 //#define ENABLE_LOG_PRINTF
+
+extern int _evas_engine_direct3d_log_dom ;
+#ifdef ERR
+# undef ERR
+#endif
+#define ERR(...) EINA_LOG_DOM_ERR(_evas_engine_direct3d_log_dom, __VA_ARGS__)
+
+#ifdef DBG
+# undef DBG
+#endif
+#define DBG(...) EINA_LOG_DOM_DBG(_evas_engine_direct3d_log_dom, __VA_ARGS__)
+
+#ifdef INF
+# undef INF
+#endif
+#define INF(...) EINA_LOG_DOM_INFO(_evas_engine_direct3d_log_dom, __VA_ARGS__)
+
+#ifdef WRN
+# undef WRN
+#endif
+#define WRN(...) EINA_LOG_DOM_WARN(_evas_engine_direct3d_log_dom, __VA_ARGS__)
+
+#ifdef CRIT
+# undef CRIT
+#endif
+#define CRIT(...) EINA_LOG_DOM_CRIT(_evas_engine_direct3d_log_dom, __VA_ARGS__)
+
 #ifdef ENABLE_LOG_PRINTF
-#define Log(str, ...) printf("D3D "str"\n", __VA_ARGS__)
+#define Log(str, ...) INF("D3D "str, __VA_ARGS__)
 #else
 #define Log(str, ...)
 #endif
@@ -31,6 +58,7 @@ typedef void * Direct3DDeviceHandler;
 typedef void * Direct3DImageHandler;
 typedef void * Direct3DFontGlyphHandler;
 
+
 #ifdef __cplusplus
 extern "C" {
 #endif
