@@ -1,8 +1,9 @@
 #include "evas_gl_private.h"
 
-Evas_GL_Texture *
-evas_gl_font_texture_new(Evas_GL_Context *gc, RGBA_Font_Glyph *fg)
+void *
+evas_gl_font_texture_new(void *context, RGBA_Font_Glyph *fg)
 {
+   Evas_Engine_GL_Context *gc = context;
    Evas_GL_Texture *tex;
    DATA8 *data;
    int w, h, j, nw;
@@ -10,7 +11,7 @@ evas_gl_font_texture_new(Evas_GL_Context *gc, RGBA_Font_Glyph *fg)
    int fh;
 
    if (fg->ext_dat) return fg->ext_dat; // FIXME: one engine at a time can do this :(
-   
+
    w = fg->glyph_out->bitmap.width;
    h = fg->glyph_out->bitmap.rows;
    if ((w == 0) || (h == 0)) return NULL;
@@ -44,13 +45,13 @@ evas_gl_font_texture_new(Evas_GL_Context *gc, RGBA_Font_Glyph *fg)
 	DATA8 *tmpbuf = NULL, *dp, *tp, bits;
 	int bi, bj, end;
 	const DATA8 bitrepl[2] = {0x0, 0xff};
-	
+
 	tmpbuf = alloca(w);
 	if (tmpbuf)
 	  {
 	     int x, y;
 	     DATA8 *p1, *p2;
-	     
+
 	     for (y = 0; y < h; y++)
 	       {
 		  p1 = tmpbuf;
@@ -89,15 +90,17 @@ evas_gl_font_texture_new(Evas_GL_Context *gc, RGBA_Font_Glyph *fg)
 }
 
 void
-evas_gl_font_texture_free(Evas_GL_Texture *tex)
+evas_gl_font_texture_free(void *tex)
 {
    if (!tex) return;
    evas_gl_common_texture_free(tex);
 }
 
 void
-evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Draw_Context *dc, RGBA_Font_Glyph *fg, int x, int y)
+evas_gl_font_texture_draw(void *context, void *surface __UNUSED__, void *draw_context, RGBA_Font_Glyph *fg, int x, int y)
 {
+   Evas_Engine_GL_Context *gc = context;
+   RGBA_Draw_Context *dc = draw_context;
    Evas_GL_Texture *tex;
    Cutout_Rects *rects;
    Cutout_Rect  *rct;
@@ -106,7 +109,7 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
    int c, cx, cy, cw, ch;
    int i;
    int sx, sy, sw, sh;
-   
+
    if (dc != gc->dc) return;
    tex = fg->ext_dat;
    if (!tex) return;
@@ -116,14 +119,14 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
    g = (dc->col.col >> 8 ) & 0xff;
    b = (dc->col.col      ) & 0xff;
    sx = 0; sy = 0; sw = tex->w, sh = tex->h;
-   if ((!gc->dc->cutout.rects) 
-//       || (gc->dc->cutout.active > 32)
-       )
+   if ((!gc->dc->cutout.rects) ||
+       ((gc->shared->info.tune.cutout.max > 0) &&
+           (gc->dc->cutout.active > gc->shared->info.tune.cutout.max)))
      {
         if (gc->dc->clip.use)
           {
              int nx, ny, nw, nh;
-             
+
              nx = x; ny = y; nw = tex->w; nh = tex->h;
              RECTS_CLIP_TO_RECT(nx, ny, nw, nh,
                                 gc->dc->clip.x, gc->dc->clip.y,
@@ -142,14 +145,14 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
              ssy = (double)sy + ((double)(sh * (ny - y)) / (double)(tex->h));
              ssw = ((double)sw * (double)(nw)) / (double)(tex->w);
              ssh = ((double)sh * (double)(nh)) / (double)(tex->h);
-             evas_gl_common_context_font_push(gc, tex, 
+             evas_gl_common_context_font_push(gc, tex,
                                               ssx, ssy, ssw, ssh,
                                               nx, ny, nw, nh,
                                               r, g, b, a);
           }
         else
           {
-             evas_gl_common_context_font_push(gc, tex, 
+             evas_gl_common_context_font_push(gc, tex,
                                               0.0, 0.0, 0.0, 0.0,
 //                                              sx, sy, sw, sh,
                                               x, y, tex->w, tex->h,
@@ -159,7 +162,7 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
      }
    /* save out clip info */
    c = gc->dc->clip.use; cx = gc->dc->clip.x; cy = gc->dc->clip.y; cw = gc->dc->clip.w; ch = gc->dc->clip.h;
-   evas_common_draw_context_clip_clip(gc->dc, 0, 0, gc->w, gc->h);
+   evas_common_draw_context_clip_clip(gc->dc, 0, 0, gc->shared->w, gc->shared->h);
    evas_common_draw_context_clip_clip(gc->dc, x, y, tex->w, tex->h);
    /* our clip is 0 size.. abort */
    if ((gc->dc->clip.w <= 0) || (gc->dc->clip.h <= 0))
@@ -171,7 +174,7 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
    for (i = 0; i < rects->active; ++i)
      {
         int nx, ny, nw, nh;
-        
+
         rct = rects->rects + i;
         nx = x; ny = y; nw = tex->w; nh = tex->h;
         RECTS_CLIP_TO_RECT(nx, ny, nw, nh, rct->x, rct->y, rct->w, rct->h);
@@ -189,7 +192,7 @@ evas_gl_font_texture_draw(Evas_GL_Context *gc, void *surface __UNUSED__, RGBA_Dr
         ssy = (double)sy + ((double)(sh * (ny - y)) / (double)(tex->h));
         ssw = ((double)sw * (double)(nw)) / (double)(tex->w);
         ssh = ((double)sh * (double)(nh)) / (double)(tex->h);
-        evas_gl_common_context_font_push(gc, tex, 
+        evas_gl_common_context_font_push(gc, tex,
                                          ssx, ssy, ssw, ssh,
                                          nx, ny, nw, nh,
                                          r, g, b, a);
