@@ -1,11 +1,11 @@
 #include "evas_gl_private.h"
 
 void
-evas_gl_common_image_all_unload(Evas_GL_Context *gc)
+evas_gl_common_image_all_unload(Evas_Engine_GL_Context *gc)
 {
    Eina_List *l;
    Evas_GL_Image *im;
-   
+
    EINA_LIST_FOREACH(gc->shared->images, l, im)
      {
         if (im->im) evas_cache_image_unload_data(&im->im->cache_entry);
@@ -20,8 +20,84 @@ evas_gl_common_image_all_unload(Evas_GL_Context *gc)
      }
 }
 
+static void
+_evas_gl_image_cache_trim(Evas_Engine_GL_Context *gc)
+{
+   int size = evas_common_image_get_cache();
+
+   while (gc->shared->images_size > size)
+     {
+        Evas_GL_Image *im2;
+        Eina_List *l = NULL;
+
+        EINA_LIST_REVERSE_FOREACH(gc->shared->images, l, im2)
+          {
+             if (im2->references == 0)
+               {
+                  im2->cached = 0;
+                  im2->gc->shared->images =
+                     eina_list_remove_list(im2->gc->shared->images, l);
+                  im2->gc->shared->images_size -= (im2->csize);
+                  evas_gl_common_image_free(im2);
+                  l = NULL;
+                  break;
+               }
+          }
+        if ((gc->shared->images_size > size) && (l))
+          {
+//             printf("EEK %i > %i, no 0 ref imgs\n",
+//                    gc->shared->images_size, size);
+             break;
+          }
+        if (!gc->shared->images)
+          {
+//             printf("EEK %i > %i, no imgs\n",
+//                    gc->shared->images_size, size);
+             break;
+          }
+     }
+}
+
+static Eina_Bool
+_evas_gl_image_cache_add(Evas_GL_Image *im)
+{
+   if (im->references == 0)
+     {
+        im->csize = im->w * im->h * 4;
+        im->gc->shared->images_size += im->csize;
+        _evas_gl_image_cache_trim(im->gc);
+        return EINA_TRUE;
+     }
+   else
+     {
+        im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
+        im->cached = 0;
+     }
+   return EINA_FALSE;
+}
+
+void
+evas_gl_common_image_ref(Evas_GL_Image *im)
+{
+   if (im->references == 0)
+     {
+        im->gc->shared->images_size -= (im->csize);
+     }
+   im->references++;
+}
+
+void
+evas_gl_common_image_unref(Evas_GL_Image *im)
+{
+   im->references--;
+   if (im->references == 0)
+     {
+        _evas_gl_image_cache_add(im);
+     }
+}
+
 Evas_GL_Image *
-evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error)
+evas_gl_common_image_load(Evas_Engine_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error)
 {
    Evas_GL_Image        *im;
    RGBA_Image           *im_im;
@@ -30,14 +106,29 @@ evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key
    im_im = evas_common_load_image_from_file(file, key, lo, error);
    if (!im_im) return NULL;
 
+   /* i'd LOVe to do this, but we can't because we load to load header
+    * to get image size to know if its too big or not! so this disallows
+    * us to know that - photocam thus suffers
+   if (((int)im_im->cache_entry.w > gc->shared->info.max_texture_size) ||
+       ((int)im_im->cache_entry.h > gc->shared->info.max_texture_size))
+     {
+        evas_cache_image_drop(&(im_im->cache_entry));
+        *error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+        return NULL;
+     }
+    */
+   
+   // FIXME: keep unreffed shared images around
    EINA_LIST_FOREACH(gc->shared->images, l, im)
      {
 	if (im->im == im_im)
 	  {
-             evas_cache_image_drop(&im_im->cache_entry);
+// why did i put this here? i think to free the rgba pixel data once a texture
+// exists.
+//             evas_cache_image_drop(&(im_im->cache_entry));
 	     gc->shared->images = eina_list_remove_list(gc->shared->images, l);
 	     gc->shared->images = eina_list_prepend(gc->shared->images, im);
-	     im->references++;
+             evas_gl_common_image_ref(im);
 	     *error = EVAS_LOAD_ERROR_NONE;
 	     return im;
 	  }
@@ -46,13 +137,13 @@ evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im)
      {
+        evas_cache_image_drop(&(im_im->cache_entry));
 	*error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
 	return NULL;
      }
    im->references = 1;
    im->im = im_im;
    im->gc = gc;
-   im->references = 1;
    im->cached = 1;
    im->cs.space = EVAS_COLORSPACE_ARGB8888;
    im->alpha = im->im->cache_entry.flags.alpha;
@@ -64,11 +155,15 @@ evas_gl_common_image_load(Evas_GL_Context *gc, const char *file, const char *key
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new_from_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
+evas_gl_common_image_new_from_data(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
 {
    Evas_GL_Image *im;
    Eina_List *l;
 
+   if (((int)w > gc->shared->info.max_texture_size) ||
+       ((int)h > gc->shared->info.max_texture_size))
+     return NULL;
+     
    if (data)
      {
         EINA_LIST_FOREACH(gc->shared->images, l, im)
@@ -79,7 +174,7 @@ evas_gl_common_image_new_from_data(Evas_GL_Context *gc, unsigned int w, unsigned
                {
                   gc->shared->images = eina_list_remove_list(gc->shared->images, l);
                   gc->shared->images = eina_list_prepend(gc->shared->images, im);
-                  im->references++;
+                  evas_gl_common_image_ref(im);
                   return im;
                }
           }
@@ -118,10 +213,14 @@ evas_gl_common_image_new_from_data(Evas_GL_Context *gc, unsigned int w, unsigned
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
+evas_gl_common_image_new_from_copied_data(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, int cspace)
 {
    Evas_GL_Image *im;
 
+   if (((int)w > gc->shared->info.max_texture_size) ||
+       ((int)h > gc->shared->info.max_texture_size))
+     return NULL;
+   
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im) return NULL;
    im->references = 1;
@@ -159,10 +258,14 @@ evas_gl_common_image_new_from_copied_data(Evas_GL_Context *gc, unsigned int w, u
 }
 
 Evas_GL_Image *
-evas_gl_common_image_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha, int cspace)
+evas_gl_common_image_new(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, int alpha, int cspace)
 {
    Evas_GL_Image *im;
 
+   if (((int)w > gc->shared->info.max_texture_size) ||
+       ((int)h > gc->shared->info.max_texture_size))
+     return NULL;
+   
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im) return NULL;
    im->references = 1;
@@ -188,6 +291,9 @@ evas_gl_common_image_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, in
 	break;
       case EVAS_COLORSPACE_YCBCR422P601_PL:
       case EVAS_COLORSPACE_YCBCR422P709_PL:
+      case EVAS_COLORSPACE_YCBCR422601_PL:
+      case EVAS_COLORSPACE_YCBCR420NV12601_PL:
+      case EVAS_COLORSPACE_YCBCR420TM12601_PL:
 //        if (im->tex) evas_gl_common_texture_free(im->tex);
 	im->tex = NULL;
 	im->cs.no_free = 0;
@@ -230,6 +336,8 @@ evas_gl_common_image_native_enable(Evas_GL_Image *im)
    im->cs.no_free = 0;
    if (im->cached)
      {
+        if (im->references == 0)
+           im->gc->shared->images_size -= (im->csize);
         im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
         im->cached = 0;
      }
@@ -243,7 +351,7 @@ evas_gl_common_image_native_enable(Evas_GL_Image *im)
         evas_gl_common_texture_free(im->tex);
         im->tex = NULL;
      }
-   
+
    im->cs.space = EVAS_COLORSPACE_ARGB8888;
    im->tex = evas_gl_common_texture_native_new(im->gc, im->w, im->h, im->alpha, im);
    im->tex_only = 1;
@@ -263,7 +371,7 @@ evas_gl_common_image_native_disable(Evas_GL_Image *im)
         im->tex = NULL;
      }
    im->tex_only = 0;
-   
+
    im->im = (RGBA_Image *)evas_cache_image_empty(evas_common_image_cache_get());
    im->im->cache_entry.flags.alpha = im->alpha;
    im->cs.space = EVAS_COLORSPACE_ARGB8888;
@@ -302,6 +410,8 @@ evas_gl_common_image_content_hint_set(Evas_GL_Image *im, int hint)
         im->cs.no_free = 0;
         if (im->cached)
           {
+             if (im->references == 0)
+                im->gc->shared->images_size -= im->csize;
              im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
              im->cached = 0;
           }
@@ -331,7 +441,7 @@ evas_gl_common_image_content_hint_set(Evas_GL_Image *im, int hint)
              im->tex = NULL;
           }
         im->tex_only = 0;
-        
+
         im->im = (RGBA_Image *)evas_cache_image_empty(evas_common_image_cache_get());
         im->im->cache_entry.flags.alpha = im->alpha;
         im->cs.space = EVAS_COLORSPACE_ARGB8888;
@@ -342,30 +452,58 @@ evas_gl_common_image_content_hint_set(Evas_GL_Image *im, int hint)
      }
 }
 
+void
+evas_gl_common_image_cache_flush(Evas_Engine_GL_Context *gc)
+{
+   _evas_gl_image_cache_trim(gc);
+}
+
 void
 evas_gl_common_image_free(Evas_GL_Image *im)
 {
+#if 0 // filtering disabled
+   Filtered_Image *fi;
+#endif
+
+   evas_gl_common_context_flush(im->gc);
    im->references--;
    if (im->references > 0) return;
-   
+
    if (im->native.func.free)
      im->native.func.free(im->native.func.data, im);
-   
+
    if (im->cs.data)
      {
 	if (!im->cs.no_free) free(im->cs.data);
      }
-   if (im->cached) im->gc->shared->images = eina_list_remove(im->gc->shared->images, im);
+   if (im->cached)
+     {
+        if (_evas_gl_image_cache_add(im)) return;
+     }
    if (im->im) evas_cache_image_drop(&im->im->cache_entry);
    if (im->tex) evas_gl_common_texture_free(im->tex);
+
+#if 0 // filtering disabled
+   EINA_LIST_FREE(im->filtered, fi)
+     {
+	free(fi->key);
+	evas_gl_common_image_free((Evas_GL_Image *)fi->image);
+	free(fi);
+     }
+#endif
+
    free(im);
 }
 
 Evas_GL_Image *
-evas_gl_common_image_surface_new(Evas_GL_Context *gc, unsigned int w, unsigned int h, int alpha)
+evas_gl_common_image_surface_new(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, int alpha)
 {
    Evas_GL_Image *im;
 
+   if (((int)w > gc->shared->info.max_texture_size) ||
+       ((int)h > gc->shared->info.max_texture_size))
+     return NULL;
+   
    im = calloc(1, sizeof(Evas_GL_Image));
    if (!im) return NULL;
    im->references = 1;
@@ -394,11 +532,13 @@ evas_gl_common_image_dirty(Evas_GL_Image *im, unsigned int x, unsigned int y, un
    im->dirty = 1;
 }
 
-static void
-_evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
+void
+evas_gl_common_image_update(Evas_Engine_GL_Context *gc, Evas_GL_Image *im)
 {
+   Image_Entry *ie;
    if (!im->im) return;
-/*   
+   ie = (Image_Entry *)(im->im);
+/*
    if ((im->cs.space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
        (im->cs.space == EVAS_COLORSPACE_YCBCR422P709_PL))
      {
@@ -410,7 +550,7 @@ _evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
                   free(im->im->image.data);
                   im->im->image.data = malloc(im->im->cache_entry.w * im->im->cache_entry.h * sizeof(DATA32));
                   if (im->im->image.data)
-                    evas_common_convert_yuv_420p_601_rgba(im->cs.data, 
+                    evas_common_convert_yuv_420p_601_rgba(im->cs.data,
                                                           (void *)im->im->image.data,
                                                           im->im->cache_entry.w, im->im->cache_entry.h);
                }
@@ -422,7 +562,8 @@ _evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
    switch (im->cs.space)
      {
       case EVAS_COLORSPACE_ARGB8888:
-	if ((im->tex) && (im->dirty))
+         if ((im->tex) &&
+             ((im->dirty) || (ie->flags.animated)))
           {
              evas_cache_image_load_data(&im->im->cache_entry);
              evas_gl_common_texture_update(im->tex, im->im);
@@ -442,34 +583,84 @@ _evas_gl_common_image_update(Evas_GL_Context *gc, Evas_GL_Image *im)
         if ((im->tex) && (im->dirty))
           {
              evas_gl_common_texture_yuv_update(im->tex, im->cs.data,
-                                               im->im->cache_entry.w, 
+                                               im->im->cache_entry.w,
                                                im->im->cache_entry.h);
              im->dirty = 0;
           }
         if ((!im->tex) && (im->cs.data) && (*((unsigned char **)im->cs.data)))
           {
              im->tex = evas_gl_common_texture_yuv_new(gc, im->cs.data,
-                                                      im->im->cache_entry.w, 
+                                                      im->im->cache_entry.w,
                                                       im->im->cache_entry.h);
              im->dirty = 0;
           }
         if (!im->tex) return;
         break;
+      case EVAS_COLORSPACE_YCBCR422601_PL:
+        if ((im->tex) && (im->dirty))
+          {
+             evas_gl_common_texture_yuy2_update(im->tex, im->cs.data,
+                                                im->im->cache_entry.w,
+                                                im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if ((!im->tex) && (im->cs.data) && (*((unsigned char **)im->cs.data)))
+          {
+             im->tex = evas_gl_common_texture_yuy2_new(gc, im->cs.data,
+                                                       im->im->cache_entry.w,
+                                                       im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if (!im->tex) return;
+        break;
+      case EVAS_COLORSPACE_YCBCR420NV12601_PL:
+        if ((im->tex) && (im->dirty))
+          {
+             evas_gl_common_texture_nv12_update(im->tex, im->cs.data,
+                                                im->im->cache_entry.w,
+                                                im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if ((!im->tex) && (im->cs.data) && (*((unsigned char **)im->cs.data)))
+          {
+             im->tex = evas_gl_common_texture_nv12_new(gc, im->cs.data,
+                                                       im->im->cache_entry.w,
+                                                       im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if (!im->tex) return;
+        break;
+      case EVAS_COLORSPACE_YCBCR420TM12601_PL:
+        if ((im->tex) && (im->dirty))
+          {
+             evas_gl_common_texture_nv12tiled_update(im->tex, im->cs.data,
+                                                     im->im->cache_entry.w,
+                                                     im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if ((!im->tex) && (im->cs.data) && (*((unsigned char **)im->cs.data)))
+          {
+             im->tex = evas_gl_common_texture_nv12tiled_new(gc, im->cs.data,
+                                                            im->im->cache_entry.w,
+                                                            im->im->cache_entry.h);
+             im->dirty = 0;
+          }
+        if (!im->tex) return;
+        break;
       default:
-         ERR("unhandled img format colorspace=%d", im->cs.space);
-	break;
+        ERR("unhandled img format colorspace=%d", im->cs.space);
+        break;
     }
 }
 
 void
-evas_gl_common_image_map4_draw(Evas_GL_Context *gc, Evas_GL_Image *im, 
-                               RGBA_Map_Point *p, int smooth, int level __UNUSED__)
+evas_gl_common_image_map_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im,
+                              int npoints, RGBA_Map_Point *p, int smooth, int level __UNUSED__)
 {
    RGBA_Draw_Context *dc;
    int r, g, b, a;
    int c, cx, cy, cw, ch;
-   Eina_Bool yuv = 0;
-   
+
    dc = gc->dc;
    if (dc->mul.use)
      {
@@ -482,37 +673,40 @@ evas_gl_common_image_map4_draw(Evas_GL_Context *gc, Evas_GL_Image *im,
      {
         r = g = b = a = 255;
      }
-   
-   _evas_gl_common_image_update(gc, im);
 
-   c = gc->dc->clip.use; 
-   cx = gc->dc->clip.x; cy = gc->dc->clip.y; 
+   evas_gl_common_image_update(gc, im);
+
+   c = gc->dc->clip.use;
+   cx = gc->dc->clip.x; cy = gc->dc->clip.y;
    cw = gc->dc->clip.w; ch = gc->dc->clip.h;
    im->tex->im = im;
-   if ((im->cs.space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
-       (im->cs.space == EVAS_COLORSPACE_YCBCR422P709_PL))
-      yuv = 1;
-   evas_gl_common_context_image_map4_push(gc, im->tex, p, 
-                                          c, cx, cy, cw, ch, 
-                                          r, g, b, a, smooth, im->tex_only,
-                                          yuv);
+
+   evas_gl_common_context_image_map_push(gc, im->tex, npoints, p,
+                                         c, cx, cy, cw, ch,
+                                         r, g, b, a, smooth, im->tex_only,
+                                         im->cs.space);
 }
 
 void
-evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh, int smooth)
+evas_gl_common_image_draw(Evas_Engine_GL_Context *gc, Evas_GL_Image *im, int sx, int sy, int sw, int sh, int dx, int dy, int dw, int dh, int smooth)
 {
    RGBA_Draw_Context *dc;
+   Evas_GL_Image *imm;
    int r, g, b, a;
    double ssx, ssy, ssw, ssh;
+   double mssx, mssy, mssw, mssh;
    Cutout_Rects *rects;
    Cutout_Rect  *rct;
    int c, cx, cy, cw, ch;
    int i;
    int yuv = 0;
-   
+   int yuy2 = 0;
+   int nv12 = 0;
+
    if (sw < 1) sw = 1;
    if (sh < 1) sh = 1;
    dc = gc->dc;
+   imm = (Evas_GL_Image *)dc->mask.mask;
    if (dc->mul.use)
      {
 	a = (dc->mul.col >> 24) & 0xff;
@@ -524,33 +718,45 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
      {
 	r = g = b = a = 255;
      }
-   
-   _evas_gl_common_image_update(gc, im);
+
+   evas_gl_common_image_update(gc, im);
    if (!im->tex)
      {
         evas_gl_common_rect_draw(gc, dx, dy, dw, dh);
         return;
      }
+   if (imm)
+     {
+        evas_gl_common_image_update(gc, imm);
+        if (!imm->tex) imm = NULL; /* Turn of mask on error */
+     }
 
    if ((im->cs.space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
        (im->cs.space == EVAS_COLORSPACE_YCBCR422P709_PL))
      yuv = 1;
-   
+   if (im->cs.space == EVAS_COLORSPACE_YCBCR422601_PL)
+     yuy2 = 1;
+   if ((im->cs.space == EVAS_COLORSPACE_YCBCR420NV12601_PL) ||
+       (im->cs.space == EVAS_COLORSPACE_YCBCR420TM12601_PL))
+     nv12 = 1;
+
    im->tex->im = im;
-   if ((!gc->dc->cutout.rects) || 
+   if (imm) imm->tex->im = imm;
+   if ((!gc->dc->cutout.rects) ||
        ((gc->shared->info.tune.cutout.max > 0) &&
            (gc->dc->cutout.active > gc->shared->info.tune.cutout.max)))
      {
         if (gc->dc->clip.use)
           {
              int nx, ny, nw, nh;
-             
+             double scalex,scaley;
+
              nx = dx; ny = dy; nw = dw; nh = dh;
-             RECTS_CLIP_TO_RECT(nx, ny, nw, nh, 
+             RECTS_CLIP_TO_RECT(nx, ny, nw, nh,
                                 gc->dc->clip.x, gc->dc->clip.y,
                                 gc->dc->clip.w, gc->dc->clip.h);
              if ((nw < 1) || (nh < 1)) return;
-             if ((nx == dx) && (ny == dy) && (nw == dw) && (nh == dh))
+             if ((!imm) && (nx == dx) && (ny == dy) && (nw == dw) && (nh == dh))
                {
                   if (yuv)
                     evas_gl_common_context_yuv_push(gc,
@@ -559,7 +765,22 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                     dx, dy, dw, dh,
                                                     r, g, b, a,
                                                     smooth);
+                  else if (yuy2)
+                    evas_gl_common_context_yuy2_push(gc,
+						     im->tex,
+						     sx, sy, sw, sh,
+						     dx, dy, dw, dh,
+						     r, g, b, a,
+						     smooth);
+                  else if (nv12)
+                    evas_gl_common_context_nv12_push(gc,
+						     im->tex,
+						     sx, sy, sw, sh,
+						     dx, dy, dw, dh,
+						     r, g, b, a,
+						     smooth);
                   else
+
                     evas_gl_common_context_image_push(gc,
                                                       im->tex,
                                                       sx, sy, sw, sh,
@@ -568,18 +789,53 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                       smooth, im->tex_only);
                   return;
                }
-             
+
              ssx = (double)sx + ((double)(sw * (nx - dx)) / (double)(dw));
              ssy = (double)sy + ((double)(sh * (ny - dy)) / (double)(dh));
              ssw = ((double)sw * (double)(nw)) / (double)(dw);
              ssh = ((double)sh * (double)(nh)) / (double)(dh);
-             if (yuv)
+             if (imm)
+               {
+                  /* Correct ones here */
+                  scalex = imm->w / (double)dc->mask.w;
+                  scaley = imm->h / (double)dc->mask.h;
+                  mssx = scalex * (nx - dc->mask.x);
+                  mssy = scaley * (ny - dc->mask.y);
+                  mssw = scalex * nw;
+                  mssh = scaley * nh;
+
+                  /* No yuv + imm I'm afraid */
+                  evas_gl_common_context_image_mask_push(gc,
+                                               im->tex,
+                                               imm->tex,
+                                               ssx, ssy, ssw, ssh,
+                                               mssx, mssy, mssw, mssh,
+                                               //dc->mask.x, dc->mask.y, dc->mask.w, dc->mask.h,
+                                               nx, ny, nw, nh,
+                                               r, g, b, a,
+                                               smooth);
+               }
+             else if (yuv)
                evas_gl_common_context_yuv_push(gc,
                                                im->tex,
                                                ssx, ssy, ssw, ssh,
                                                nx, ny, nw, nh,
                                                r, g, b, a,
                                                smooth);
+             else if (yuy2)
+               evas_gl_common_context_yuy2_push(gc,
+						im->tex,
+						ssx, ssy, ssw, ssh,
+						nx, ny, nw, nh,
+						r, g, b, a,
+						smooth);
+             else if (nv12)
+               evas_gl_common_context_nv12_push(gc,
+						im->tex,
+						ssx, ssy, ssw, ssh,
+						nx, ny, nw, nh,
+						r, g, b, a,
+						smooth);
              else
                evas_gl_common_context_image_push(gc,
                                                  im->tex,
@@ -597,6 +853,20 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                dx, dy, dw, dh,
                                                r, g, b, a,
                                                smooth);
+             else if (yuy2)
+               evas_gl_common_context_yuy2_push(gc,
+						im->tex,
+						sx, sy, sw, sh,
+						dx, dy, dw, dh,
+						r, g, b, a,
+						smooth);
+             else if (nv12)
+               evas_gl_common_context_nv12_push(gc,
+						im->tex,
+						sx, sy, sw, sh,
+						dx, dy, dw, dh,
+						r, g, b, a,
+						smooth);
              else
                evas_gl_common_context_image_push(gc,
                                                  im->tex,
@@ -607,10 +877,10 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
           }
         return;
      }
-   
+
    /* save out clip info */
    c = gc->dc->clip.use; cx = gc->dc->clip.x; cy = gc->dc->clip.y; cw = gc->dc->clip.w; ch = gc->dc->clip.h;
-   evas_common_draw_context_clip_clip(gc->dc, 0, 0, gc->w, gc->h);
+   evas_common_draw_context_clip_clip(gc->dc, 0, 0, gc->shared->w, gc->shared->h);
    evas_common_draw_context_clip_clip(gc->dc, dx, dy, dw, dh);
    /* our clip is 0 size.. abort */
    if ((gc->dc->clip.w <= 0) || (gc->dc->clip.h <= 0))
@@ -636,6 +906,20 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                                dx, dy, dw, dh,
                                                r, g, b, a,
                                                smooth);
+             else if (yuy2)
+               evas_gl_common_context_yuy2_push(gc,
+						im->tex,
+						sx, sy, sw, sh,
+						dx, dy, dw, dh,
+						r, g, b, a,
+						smooth);
+             else if (nv12)
+               evas_gl_common_context_nv12_push(gc,
+						im->tex,
+						sx, sy, sw, sh,
+						dx, dy, dw, dh,
+						r, g, b, a,
+						smooth);
              else
                evas_gl_common_context_image_push(gc,
                                                  im->tex,
@@ -656,6 +940,20 @@ evas_gl_common_image_draw(Evas_GL_Context *gc, Evas_GL_Image *im, int sx, int sy
                                           nx, ny, nw, nh,
                                           r, g, b, a,
                                           smooth);
+        else if (yuy2)
+          evas_gl_common_context_yuy2_push(gc,
+					   im->tex,
+					   ssx, ssy, ssw, ssh,
+					   nx, ny, nw, nh,
+					   r, g, b, a,
+					   smooth);
+        else if (nv12)
+          evas_gl_common_context_nv12_push(gc,
+					   im->tex,
+					   ssx, ssy, ssw, ssh,
+					   nx, ny, nw, nh,
+					   r, g, b, a,
+					   smooth);
         else
           evas_gl_common_context_image_push(gc,
                                             im->tex,
