@@ -13,7 +13,7 @@ static GLXFBConfig rgba_fbconf = 0;
 #endif
 
 // fixme: something is up/wrong here - dont know what tho...
-// #define NEWGL 1
+//#define NEWGL 1
 
 static XVisualInfo *_evas_gl_x11_vi = NULL;
 static XVisualInfo *_evas_gl_x11_rgba_vi = NULL;
@@ -41,15 +41,15 @@ eng_window_new(Display *disp,
    int config_attrs[40];
    int major_version, minor_version;
    int num_config, n = 0;
-#endif   
+#endif
    XVisualInfo *vi_use;
    const GLubyte *vendor, *renderer, *version;
-   
+
    if (!_evas_gl_x11_vi) return NULL;
-   
+
    gw = calloc(1, sizeof(Evas_GL_X11_Window));
    if (!gw) return NULL;
-   
+
    win_count++;
    gw->disp = disp;
    gw->win = win;
@@ -71,16 +71,16 @@ eng_window_new(Display *disp,
              vi_use = _evas_gl_x11_rgba_vi;
           }
 #else
-#ifdef NEWGL
+//#ifdef NEWGL
         if (_evas_gl_x11_rgba_vi)
           {
              vi_use = _evas_gl_x11_rgba_vi;
           }
-#endif        
-#endif        
+//#endif
+#endif
      }
    gw->visualinfo = vi_use;
-   
+
 // EGL / GLES
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
    context_attrs[0] = EGL_CONTEXT_CLIENT_VERSION;
@@ -137,7 +137,7 @@ eng_window_new(Display *disp,
    config_attrs[n++] = 1;
    config_attrs[n++] = EGL_BLUE_SIZE;
    config_attrs[n++] = 1;
-// FIXME: end n900 breakage   
+// FIXME: end n900 breakage
 #endif
    if (gw->alpha)
      {
@@ -155,7 +155,7 @@ eng_window_new(Display *disp,
    config_attrs[n++] = 0;
    config_attrs[n++] = EGL_NONE;
 # endif
-   
+
    gw->egl_disp = eglGetDisplay((EGLNativeDisplayType)(gw->disp));
    if (!gw->egl_disp)
      {
@@ -176,7 +176,7 @@ eng_window_new(Display *disp,
 	eng_window_free(gw);
         return NULL;
      }
-   
+
    num_config = 0;
    if (!eglChooseConfig(gw->egl_disp, config_attrs, &gw->egl_config,
                         1, &num_config) || (num_config != 1))
@@ -196,7 +196,7 @@ eng_window_new(Display *disp,
         return NULL;
      }
    if (context == EGL_NO_CONTEXT)
-     context = eglCreateContext(gw->egl_disp, gw->egl_config, NULL, 
+     context = eglCreateContext(gw->egl_disp, gw->egl_config, NULL,
                                 context_attrs);
    gw->egl_context[0] = context;
    if (gw->egl_context[0] == EGL_NO_CONTEXT)
@@ -205,8 +205,8 @@ eng_window_new(Display *disp,
 	eng_window_free(gw);
         return NULL;
      }
-   if (eglMakeCurrent(gw->egl_disp, 
-                      gw->egl_surface[0], 
+   if (eglMakeCurrent(gw->egl_disp,
+                      gw->egl_surface[0],
                       gw->egl_surface[0],
                       gw->egl_context[0]) == EGL_FALSE)
      {
@@ -214,33 +214,32 @@ eng_window_new(Display *disp,
 	eng_window_free(gw);
         return NULL;
      }
-    _evas_gl_x11_window = gw;
 
    vendor = glGetString(GL_VENDOR);
    renderer = glGetString(GL_RENDERER);
    version = glGetString(GL_VERSION);
-   if (!vendor) vendor = "-UNKNOWN-";
-   if (!renderer) renderer = "-UNKNOWN-";
-   if (!version) version = "-UNKNOWN-";
+   if (!vendor)   vendor   = (unsigned char *)"-UNKNOWN-";
+   if (!renderer) renderer = (unsigned char *)"-UNKNOWN-";
+   if (!version)  version  = (unsigned char *)"-UNKNOWN-";
    if (getenv("EVAS_GL_INFO"))
      {
         fprintf(stderr, "vendor: %s\n", vendor);
         fprintf(stderr, "renderer: %s\n", renderer);
         fprintf(stderr, "version: %s\n", version);
      }
-// GLX   
+// GLX
 #else
    if (!context)
      {
-#ifdef NEWGL        
+#ifdef NEWGL
         if (indirect)
-          context = glXCreateNewContext(gw->disp, fbconf, 
-                                        GLX_RGBA_TYPE, NULL, 
-                                        GL_TRUE);
-        else
-          context = glXCreateNewContext(gw->disp, fbconf, 
-                                        GLX_RGBA_TYPE, NULL, 
+          context = glXCreateNewContext(gw->disp, fbconf,
+                                        GLX_RGBA_TYPE, NULL,
                                         GL_FALSE);
+        else
+          context = glXCreateNewContext(gw->disp, fbconf,
+                                        GLX_RGBA_TYPE, NULL,
+                                        GL_TRUE);
 #else
         if (indirect)
           context = glXCreateContext(gw->disp, gw->visualinfo, NULL, GL_FALSE);
@@ -252,13 +251,13 @@ eng_window_new(Display *disp,
    if ((gw->alpha) && (!rgba_context))
      {
         if (indirect)
-          rgba_context = glXCreateNewContext(gw->disp, rgba_fbconf, 
-                                             GLX_RGBA_TYPE, context, 
-                                             GL_TRUE);
-        else
-          rgba_context = glXCreateNewContext(gw->disp, rgba_fbconf, 
-                                             GLX_RGBA_TYPE, context, 
+          rgba_context = glXCreateNewContext(gw->disp, rgba_fbconf,
+                                             GLX_RGBA_TYPE, context,
                                              GL_FALSE);
+        else
+          rgba_context = glXCreateNewContext(gw->disp, rgba_fbconf,
+                                             GLX_RGBA_TYPE, context,
+                                             GL_TRUE);
      }
    if (gw->alpha)
      gw->glxwin = glXCreateWindow(gw->disp, rgba_fbconf, gw->win, NULL);
@@ -269,10 +268,10 @@ eng_window_new(Display *disp,
 	eng_window_free(gw);
         return NULL;
      }
-   
+
    if (gw->alpha) gw->context = rgba_context;
    else gw->context = context;
-#else   
+#else
    gw->context = context;
 #endif
 
@@ -285,13 +284,14 @@ eng_window_new(Display *disp,
      {
         int i, j,  num;
         GLXFBConfig *fbc;
+        int blacklist = 0;
 
         if (gw->glxwin)
           {
-             if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin, 
+             if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin,
                                         gw->context))
                {
-                  printf("Error: glXMakeContextCurrent(%p, %p, %p, %p)\n", (void *)gw->disp, (void *)gw->win, (void *)gw->win, (void *)gw->context);
+                  printf("Error: glXMakeContextCurrent(%p, %p, %p, %p)\n", (void *)gw->disp, (void *)gw->glxwin, (void *)gw->glxwin, (void *)gw->context);
                   eng_window_free(gw);
                   return NULL;
                }
@@ -305,7 +305,7 @@ eng_window_new(Display *disp,
                   return NULL;
                }
           }
-        
+
         // FIXME: move this up to context creation
 
         vendor = glGetString(GL_VENDOR);
@@ -317,26 +317,105 @@ eng_window_new(Display *disp,
              fprintf(stderr, "renderer: %s\n", renderer);
              fprintf(stderr, "version: %s\n", version);
           }
-        if (strstr((const char*)vendor, "NVIDIA"))
-           // FIXME: also same as tegra2 - maybe check renderer too
-           // 
-           // vendor: NVIDIA Corporation
-           // renderer: NVIDIA Tegra
-           // version: OpenGL ES 2.0
-           // 
-           // vs (for example)
-           // 
-           // vendor: NVIDIA Corporation
-           // renderer: GeForce GT 220/PCI/SSE2
-           // version: 3.2.0 NVIDIA 195.36.24
+        //   examples:
+        // vendor: NVIDIA Corporation
+        // renderer: NVIDIA Tegra
+        // version: OpenGL ES 2.0
+        //   or
+        // vendor: Imagination Technologies
+        // renderer: PowerVR SGX 540
+        // version: OpenGL ES 2.0
+        //   or
+        // vendor: NVIDIA Corporation
+        // renderer: GeForce GT 330M/PCI/SSE2
+        // version: 3.3.0 NVIDIA 256.53
+        //   or
+        // vendor: NVIDIA Corporation
+        // renderer: GeForce GT 220/PCI/SSE2
+        // version: 3.2.0 NVIDIA 195.36.24
+        //   or
+        // vendor: NVIDIA Corporation
+        // renderer: GeForce 8600 GTS/PCI/SSE2
+        // version: 3.3.0 NVIDIA 260.19.36
+        //   or
+        // vendor: ATI Technologies Inc.
+        // renderer: ATI Mobility Radeon HD 4650
+        // version: 3.2.9756 Compatibility Profile Context
+        //   or
+        // vendor: Tungsten Graphics, Inc
+        // renderer: Mesa DRI Mobile Intel® GM45 Express Chipset GEM 20100330 DEVELOPMENT x86/MMX/SSE2
+        // version: 2.1 Mesa 7.9-devel
+        //   or
+        // vendor: Advanced Micro Devices, Inc.
+        // renderer: Mesa DRI R600 (RS780 9610) 20090101  TCL DRI2
+        // version: 2.1 Mesa 7.9-devel
+        //   or
+        // vendor: NVIDIA Corporation
+        // renderer: GeForce 9600 GT/PCI/SSE2
+        // version: 3.3.0 NVIDIA 260.19.29
+        //   or
+        // vendor: ATI Technologies Inc.
+        // renderer: ATI Radeon HD 4800 Series
+        // version: 3.3.10237 Compatibility Profile Context
+        //   or
+        // vendor: Advanced Micro Devices, Inc.
+        // renderer: Mesa DRI R600 (RV770 9442) 20090101  TCL DRI2
+        // version: 2.0 Mesa 7.8.2
+        //   or
+        // vendor: Tungsten Graphics, Inc
+        // renderer: Mesa DRI Mobile Intel® GM45 Express Chipset GEM 20100330 DEVELOPMENT
+        // version: 2.1 Mesa 7.9-devel
+        //   or (bad - software renderer)
+        // vendor: Mesa Project
+        // renderer: Software Rasterizer
+        // version: 2.1 Mesa 7.9-devel
+        //   or (bad - software renderer)
+        // vendor: VMware, Inc.
+        // renderer: Gallium 0.4 on softpipe
+        // version: 2.1 Mesa 7.9-devel
+
+        if (strstr((const char *)vendor, "Mesa Project"))
           {
-             gw->detected.loose_binding = 1;
+             if (strstr((const char *)renderer, "Software Rasterizer"))
+                blacklist = 1;
+          }
+        if (strstr((const char *)renderer, "softpipe"))
+           blacklist = 1;
+        if (blacklist)
+          {
+             ERR("OpenGL Driver blacklisted:");
+             ERR("Vendor: %s", (const char *)vendor);
+             ERR("Renderer: %s", (const char *)renderer);
+             ERR("Version: %s", (const char *)version);
+             eng_window_free(gw);
+             return NULL;
+          }
+        if (strstr((const char *)vendor, "NVIDIA"))
+          {
+             if (!strstr((const char *)renderer, "NVIDIA Tegra"))
+               {
+                  int v1 = 0, v2 = 0, v3 = 0;
+
+                  if (sscanf((const char *)version,
+                             "%*s %*s %i.%i.%i",
+                             &v1, &v2, &v3) != 3)
+                    {
+                       v1 = v2 = v3 = 0;
+                       if (sscanf((const char *)version,
+                                  "%*s %*s %i.%i",
+                                  &v1, &v2) != 2)
+                          v1 = 0;
+                    }
+                  // ALSO as of some nvidia driver version loose binding is
+                  // probably not needed
+                  if (v1 < 195) gw->detected.loose_binding = 1;
+               }
           }
         else
           {
              // noothing yet. add more cases and options over time
           }
-        
+
         fbc = glXGetFBConfigs(gw->disp, screen, &num);
         if (!fbc)
           {
@@ -350,24 +429,24 @@ eng_window_new(Display *disp,
                {
                   XVisualInfo *vi;
                   int vd;
-                  int alph, val, dbuf, stencil, depth;
+                  int alph, val, dbuf, stencil, tdepth;
                   int rgba;
-                  
+
                   vi = glXGetVisualFromFBConfig(gw->disp, fbc[j]);
                   if (!vi) continue;
                   vd = vi->depth;
                   XFree(vi);
-                  
+
                   if (vd != i) continue;
-                  
+
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_ALPHA_SIZE, &alph);
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BUFFER_SIZE, &val);
-                  
+
                   if ((val != i) && ((val - alph) != i)) continue;
-                  
+
                   val = 0;
                   rgba = 0;
-                  
+
                   if (i == 32)
                     {
                        glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_TEXTURE_RGBA_EXT, &val);
@@ -384,32 +463,32 @@ eng_window_new(Display *disp,
                        if (!val) continue;
                        gw->depth_cfg[i].tex_format = GLX_TEXTURE_FORMAT_RGB_EXT;
                     }
-                  
+
                   dbuf = 0x7fff;
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_DOUBLEBUFFER, &val);
                   if (val > dbuf) continue;
                   dbuf = val;
-                  
+
                   stencil = 0x7fff;
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_STENCIL_SIZE, &val);
                   if (val > stencil) continue;
                   stencil = val;
-                  
-                  depth = 0x7fff;
+
+                  tdepth = 0x7fff;
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_DEPTH_SIZE, &val);
-                  if (val > depth) continue;
-                  depth = val;
-                  
+                  if (val > tdepth) continue;
+                  tdepth = val;
+
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_MIPMAP_TEXTURE_EXT, &val);
                   if (val < 0) continue;
                   gw->depth_cfg[i].mipmap = val;
-                  
+
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_Y_INVERTED_EXT, &val);
                   gw->depth_cfg[i].yinvert = val;
-                  
+
                   glXGetFBConfigAttrib(gw->disp, fbc[j], GLX_BIND_TO_TEXTURE_TARGETS_EXT, &val);
                   gw->depth_cfg[i].tex_target = val;
-                  
+
                   gw->depth_cfg[i].fbc = fbc[j];
                }
           }
@@ -420,8 +499,7 @@ eng_window_new(Display *disp,
           }
      }
 #endif
-   _evas_gl_x11_window = gw;
-   
+
    gw->gl_context = evas_gl_common_context_new();
    if (!gw->gl_context)
      {
@@ -430,11 +508,12 @@ eng_window_new(Display *disp,
      }
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
    gw->gl_context->egldisp = gw->egl_disp;
-#endif   
-   evas_gl_common_context_use(gw->gl_context);
+#endif
+   eng_window_use(gw);
    evas_gl_common_context_resize(gw->gl_context, w, h, rot);
    gw->surf = 1;
    return gw;
+   indirect = 0;
 }
 
 void
@@ -445,20 +524,20 @@ eng_window_free(Evas_GL_X11_Window *gw)
    eng_window_use(gw);
    if (gw == _evas_gl_x11_window) _evas_gl_x11_window = NULL;
    if (gw->gl_context)
-      {
-         ref = gw->gl_context->references - 1;
-         evas_gl_common_context_free(gw->gl_context);
-      }
+     {
+        ref = gw->gl_context->references - 1;
+        evas_gl_common_context_free(gw->gl_context);
+     }
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
    if (gw->egl_surface[0] != EGL_NO_SURFACE)
-     eglDestroySurface(gw->egl_disp, gw->egl_surface[0]);
+      eglDestroySurface(gw->egl_disp, gw->egl_surface[0]);
+   eglMakeCurrent(gw->egl_disp, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
    if (ref == 0)
      {
         if (context) eglDestroyContext(gw->egl_disp, context);
         eglTerminate(gw->egl_disp);
         context = EGL_NO_CONTEXT;
      }
-   eglMakeCurrent(gw->egl_disp, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
 #else
    if (gw->glxwin) glXDestroyWindow(gw->disp, gw->glxwin);
    if (ref == 0)
@@ -478,47 +557,55 @@ void
 eng_window_use(Evas_GL_X11_Window *gw)
 {
    Eina_Bool force_use = EINA_FALSE;
-   
+
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
    if (_evas_gl_x11_window)
      {
-        if ((eglGetCurrentContext() != 
+        if ((eglGetCurrentContext() !=
              _evas_gl_x11_window->egl_context[0]) ||
-            (eglGetCurrentSurface(EGL_READ) != 
+            (eglGetCurrentSurface(EGL_READ) !=
                 _evas_gl_x11_window->egl_surface[0]) ||
-            (eglGetCurrentSurface(EGL_DRAW) != 
+            (eglGetCurrentSurface(EGL_DRAW) !=
                 _evas_gl_x11_window->egl_surface[0]))
            force_use = EINA_TRUE;
      }
-#else   
-#endif   
+#else
+   if (_evas_gl_x11_window)
+     {
+        if (glXGetCurrentContext() != _evas_gl_x11_window->context)
+           force_use = EINA_TRUE;
+     }
+#endif
    if ((_evas_gl_x11_window != gw) || (force_use))
      {
         if (_evas_gl_x11_window)
-          evas_gl_common_context_flush(_evas_gl_x11_window->gl_context);
-	_evas_gl_x11_window = gw;
-       if (gw)
-         {
+          {
+             evas_gl_common_context_use(_evas_gl_x11_window->gl_context);
+             evas_gl_common_context_flush(_evas_gl_x11_window->gl_context);
+          }
+        _evas_gl_x11_window = gw;
+        if (gw)
+          {
 // EGL / GLES
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
            if (gw->egl_surface[0] != EGL_NO_SURFACE)
              {
-                if (eglMakeCurrent(gw->egl_disp, 
-                                   gw->egl_surface[0], 
+                if (eglMakeCurrent(gw->egl_disp,
+                                   gw->egl_surface[0],
                                    gw->egl_surface[0],
                                    gw->egl_context[0]) == EGL_FALSE)
                   {
                      ERR("eglMakeCurrent() failed!");
                   }
              }
-// GLX        
+// GLX
 #else
            if (gw->glxwin)
              {
-               if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin, 
+               if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin,
                                           gw->context))
                  {
-                   ERR("glXMakeContextCurrent(%p, %p, %p, %p)", (void *)gw->disp, (void *)gw->win, (void *)gw->win, (void *)gw->context);
+                   ERR("glXMakeContextCurrent(%p, %p, %p, %p)", (void *)gw->disp, (void *)gw->glxwin, (void *)gw->glxwin, (void *)gw->context);
                  }
              }
            else
@@ -529,7 +616,7 @@ eng_window_use(Evas_GL_X11_Window *gw)
                  }
              }
 #endif
-         }
+          }
      }
    if (gw) evas_gl_common_context_use(gw->gl_context);
 }
@@ -580,8 +667,8 @@ eng_window_resurf(Evas_GL_X11_Window *gw)
             (unsigned int)gw->win, eglGetError());
         return;
      }
-   if (eglMakeCurrent(gw->egl_disp, 
-                      gw->egl_surface[0], 
+   if (eglMakeCurrent(gw->egl_disp,
+                      gw->egl_surface[0],
                       gw->egl_surface[0],
                       gw->egl_context[0]) == EGL_FALSE)
      {
@@ -593,18 +680,18 @@ eng_window_resurf(Evas_GL_X11_Window *gw)
      gw->glxwin = glXCreateWindow(gw->disp, rgba_fbconf, gw->win, NULL);
    else
      gw->glxwin = glXCreateWindow(gw->disp, fbconf, gw->win, NULL);
-   if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin, 
+   if (!glXMakeContextCurrent(gw->disp, gw->glxwin, gw->glxwin,
                               gw->context))
      {
-        ERR("glXMakeContextCurrent(%p, %p, %p, %p)", (void *)gw->disp, (void *)gw->win, (void *)gw->win, (void *)gw->context);
+        ERR("glXMakeContextCurrent(%p, %p, %p, %p)", (void *)gw->disp, (void *)gw->glxwin, (void *)gw->glxwin, (void *)gw->context);
      }
-#else   
+#else
    if (!glXMakeCurrent(gw->disp, gw->win, gw->context))
      {
         ERR("glXMakeCurrent(%p, 0x%x, %p) failed", (void *)gw->disp, (unsigned int)gw->win, (void *)gw->context);
      }
-#endif   
-#endif   
+#endif
+#endif
    gw->surf = 1;
 }
 
@@ -616,7 +703,7 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
    if (!_evas_gl_x11_vi)
      {
         int alpha;
-        
+
 // EGL / GLES
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
         for (alpha = 0; alpha < 2; alpha++)
@@ -628,7 +715,7 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
                   XVisualInfo *xvi, vi_in;
                   int nvi, i;
                   XRenderPictFormat *fmt;
-                  
+
                   vi_in.screen = einfo->info.screen;
                   vi_in.depth = 32;
                   vi_in.class = TrueColor;
@@ -639,16 +726,16 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
                   if (xvi)
                     {
                        for (i = 0; i < nvi; i++)
-                         {  
-                            fmt = XRenderFindVisualFormat(einfo->info.display, 
+                         {
+                            fmt = XRenderFindVisualFormat(einfo->info.display,
                                                           xvi[i].visual);
-                            if ((fmt->type == PictTypeDirect) && 
+                            if ((fmt->type == PictTypeDirect) &&
                                 (fmt->direct.alphaMask))
                               {
-                                 _evas_gl_x11_rgba_vi = 
+                                 _evas_gl_x11_rgba_vi =
                                    calloc(1, sizeof(XVisualInfo));
                                  if (_evas_gl_x11_rgba_vi)
-                                   memcpy(_evas_gl_x11_rgba_vi, 
+                                   memcpy(_evas_gl_x11_rgba_vi,
                                           &(xvi[i]), sizeof(XVisualInfo));
                                  break;
                               }
@@ -671,7 +758,7 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
              int config_attrs[40];
              GLXFBConfig *configs = NULL, config = 0;
              int i, num;
-             
+
              i = 0;
              config_attrs[i++] = GLX_DRAWABLE_TYPE;
              config_attrs[i++] = GLX_WINDOW_BIT;
@@ -706,8 +793,8 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
              config_attrs[i++] = GLX_TRANSPARENT_TYPE;
              config_attrs[i++] = GLX_NONE;//GLX_NONE;//GLX_TRANSPARENT_INDEX//GLX_TRANSPARENT_RGB;
              config_attrs[i++] = 0;
-        
-             configs = glXChooseFBConfig(einfo->info.display, 
+             
+             configs = glXChooseFBConfig(einfo->info.display,
                                          einfo->info.screen,
                                          config_attrs, &num);
              if ((!configs) || (num < 1))
@@ -719,8 +806,8 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
                {
                   XVisualInfo *visinfo;
                   XRenderPictFormat *format = NULL;
-  
-                  visinfo = glXGetVisualFromFBConfig(einfo->info.display, 
+
+                  visinfo = glXGetVisualFromFBConfig(einfo->info.display,
                                                      configs[i]);
                   if (!visinfo) continue;
                   if (!alpha)
@@ -762,11 +849,11 @@ eng_best_visual_get(Evas_Engine_Info_GL_X11 *einfo)
 // EGL / GLES
 #if defined (GLES_VARIETY_S3C6410) || defined (GLES_VARIETY_SGX)
         if (_evas_gl_x11_rgba_vi) return _evas_gl_x11_rgba_vi->visual;
-#else        
-# ifdef NEWGL
+#else
+//# ifdef NEWGL
         if (_evas_gl_x11_rgba_vi) return _evas_gl_x11_rgba_vi->visual;
-# endif
-#endif        
+//# endif
+#endif
      }
    return _evas_gl_x11_vi->visual;
 }
@@ -781,24 +868,24 @@ eng_best_colormap_get(Evas_Engine_Info_GL_X11 *einfo)
    if (einfo->info.destination_alpha)
      {
         if (!_evas_gl_x11_rgba_cmap)
-          _evas_gl_x11_rgba_cmap = 
+          _evas_gl_x11_rgba_cmap =
           XCreateColormap(einfo->info.display,
                           RootWindow(einfo->info.display,
                                      einfo->info.screen),
-                          _evas_gl_x11_rgba_vi->visual, 
+                          _evas_gl_x11_rgba_vi->visual,
                           0);
         return _evas_gl_x11_rgba_cmap;
      }
    if (!_evas_gl_x11_cmap)
-     _evas_gl_x11_cmap = 
+     _evas_gl_x11_cmap =
      XCreateColormap(einfo->info.display,
                      RootWindow(einfo->info.display,
                                 einfo->info.screen),
-                     _evas_gl_x11_vi->visual, 
+                     _evas_gl_x11_vi->visual,
                      0);
    return _evas_gl_x11_cmap;
 }
-                                 
+
 int
 eng_best_depth_get(Evas_Engine_Info_GL_X11 *einfo)
 {
