@@ -54,6 +54,7 @@ eng_info(Evas *e)
    info = calloc(1, sizeof(Evas_Engine_Info_Software_16_DDraw));
    if (!info) return NULL;
    info->magic.magic = rand();
+   info->render_mode = EVAS_RENDER_MODE_BLOCKING;
    return info;
    e = NULL;
 }
@@ -125,7 +126,6 @@ eng_setup(Evas *e, void *in)
 	evas_common_convert_init();
 	evas_common_scale_init();
 	evas_common_rectangle_init();
-	evas_common_gradient_init();
 	evas_common_polygon_init();
 	evas_common_line_init();
 	evas_common_font_init();
@@ -574,10 +574,11 @@ module_open(Evas_Module *em)
    if (!em) return 0;
    /* get whatever engine module we inherit from */
    if (!_evas_module_engine_inherit(&pfunc, "software_16")) return 0;
-   _evas_engine_soft16_ddraw_log_dom = eina_log_domain_register("EvasSoft16DDraw",EINA_COLOR_BLUE);
-   if(_evas_engine_soft16_ddraw_log_dom < 0)
+   _evas_engine_soft16_ddraw_log_dom = eina_log_domain_register
+     ("evas-software_16_ddraw", EVAS_DEFAULT_LOG_COLOR);
+   if (_evas_engine_soft16_ddraw_log_dom < 0)
      {
-       EINA_LOG_ERR("Impossible to create a log domain for Soft16_DDraw engine.\n");
+        EINA_LOG_ERR("Can not create a module log domain.");
        return 0;
      }
    /* store it for later use */
