@@ -229,12 +229,19 @@ evas_engine_sdl16_output_free(void *data)
    if (re->tb) evas_common_tilebuf_free(re->tb);
    if (re->rects) evas_common_tilebuf_free_render_rects(re->rects);
    if (re->tmp_out) evas_cache_image_drop(&re->tmp_out->cache_entry);
-   if (re->soft16_engine_image) evas_cache_engine_image_drop(&re->soft16_engine_image->cache_entry);
+   if (re->soft16_engine_image)
+     evas_cache_engine_image_drop(&re->soft16_engine_image->cache_entry);
+   if (re->cache) evas_cache_engine_image_shutdown(re->cache);
+
+   if (re->update_rects)
+     free(re->update_rects);
    free(re);
 
    evas_common_font_shutdown();
    evas_common_image_shutdown();
    evas_common_soft16_image_shutdown();
+
+   SDL_QuitSubSystem(SDL_INIT_VIDEO);
 }
 
 static void
@@ -243,7 +250,7 @@ evas_engine_sdl16_output_resize(void *data, int w, int h)
    Render_Engine        *re = data;
    SDL_Surface          *surface;
 
-   if ((re->w == w) && (re->h == h)) return;
+   if ((re->tb->outbuf_w == w) && (re->tb->outbuf_h == h)) return;
 
    evas_cache_engine_image_drop(&re->soft16_engine_image->cache_entry);
 
@@ -335,6 +342,7 @@ evas_engine_sdl16_output_redraws_next_update_get(void *data,
 	re->cur_rect = re->rects;
 	if (re->rot != 0) _tmp_out_alloc(re); /* grows if required */
      }
+
    if (!re->cur_rect)
      {
 	if (re->rects) evas_common_tilebuf_free_render_rects(re->rects);
@@ -528,6 +536,8 @@ evas_engine_sdl16_output_redraws_next_update_push(void *data, void *surface __UN
      _tmp_out_process(re, rect.x, rect.y, w, h);
 
    ++re->update_rects_count;
+
+   evas_common_cpu_end_opt();
 }
 
 static void
@@ -571,7 +581,13 @@ evas_engine_sdl16_image_alpha_get(void *data __UNUSED__, void *image)
 
    if (!eim) return 1;
    im = (Soft16_Image *) eim->cache_entry.src;
-   if (im->cache_entry.flags.alpha) return 1;
+   switch (eim->cache_entry.src->space)
+     {
+     case EVAS_COLORSPACE_ARGB8888:
+        if (im->cache_entry.flags.alpha) return 1;
+     default:
+        break;
+     }
    return 0;
 }
 
@@ -581,20 +597,28 @@ evas_engine_sdl16_image_size_get(void *data __UNUSED__, void *image, int *w, int
    SDL_Engine_Image_Entry       *eim;
 
    eim = image;
-   if (w) *w = eim->cache_entry.w;
-   if (h) *h = eim->cache_entry.h;
+   if (w) *w = eim->cache_entry.src->w;
+   if (h) *h = eim->cache_entry.src->h;
 }
 
 static int
 evas_engine_sdl16_image_colorspace_get(void *data __UNUSED__, void *image __UNUSED__)
 {
-   return EVAS_COLORSPACE_RGB565_A5P;
+   SDL_Engine_Image_Entry       *eim = image;
+
+   if (!eim) return EVAS_COLORSPACE_RGB565_A5P;
+   return eim->cache_entry.src->space;
 }
 
 static void
 evas_engine_sdl16_image_colorspace_set(void *data __UNUSED__, void *image __UNUSED__, int cspace __UNUSED__)
 {
-   /* FIXME: Not implemented. */
+   SDL_Engine_Image_Entry       *eim = image;
+
+   if (!eim) return;
+   if (eim->cache_entry.src->space == cspace) return;
+
+   evas_cache_engine_image_colorspace(&eim->cache_entry, cspace, NULL);
 }
 
 static void*
@@ -666,26 +690,29 @@ evas_engine_sdl16_image_dirty_region(void *data __UNUSED__,
 
 static void*
 evas_engine_sdl16_image_data_get(void *data __UNUSED__, void *image,
-                                 int to_write, DATA32** image_data)
+                                 int to_write, DATA32** image_data, int *err)
 {
    SDL_Engine_Image_Entry       *eim = image;
    Soft16_Image                 *im;
+   int                           error;
 
    if (!eim)
      {
         *image_data = NULL;
+        if (err) *err = EVAS_LOAD_ERROR_GENERIC;
         return NULL;
      }
    im = (Soft16_Image *) eim->cache_entry.src;
-   evas_cache_image_load_data(&im->cache_entry);
 
    if (to_write)
-     eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_alone(&eim->cache_entry,
-                                                                    NULL);
+     eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_dirty(&eim->cache_entry,
+         0, 0, eim->cache_entry.src->w, eim->cache_entry.src->h);
 
+   error = evas_cache_image_load_data(&im->cache_entry);
    /* FIXME: Handle colorspace conversion correctly. */
    if (image_data) *image_data = (DATA32 *) im->pixels;
 
+   if (err) *err = error;
    return eim;
 }
 
@@ -750,10 +777,8 @@ evas_engine_sdl16_image_alpha_set(void *data __UNUSED__, void *image, int has_al
 
    if (im->cache_entry.flags.alpha == has_alpha) return eim;
 
-   eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_alone(&eim->cache_entry,
-                                                                  NULL);
-
-   im = (Soft16_Image *) eim->cache_entry.src;
+   //eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_alone(&eim->cache_entry,  NULL);
+   //im = (Soft16_Image *) eim->cache_entry.src;
 
    im->cache_entry.flags.alpha = has_alpha;
    eim = (SDL_Engine_Image_Entry *) evas_cache_engine_image_dirty(&eim->cache_entry, 0, 0, eim->cache_entry.w, eim->cache_entry.h);
@@ -808,6 +833,7 @@ evas_engine_sdl16_image_draw(void *data __UNUSED__, void *context, void *surface
                      dst_region_x, dst_region_y, dst_region_w, dst_region_h,
                      smooth);
 
+   evas_common_cpu_end_opt ();
    if (mustlock_im)
      SDL_UnlockSurface(eim->surface);
 
@@ -816,7 +842,7 @@ evas_engine_sdl16_image_draw(void *data __UNUSED__, void *context, void *surface
 }
 
 static void
-evas_engine_sdl16_image_map4_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *image __UNUSED__, RGBA_Map_Point *p __UNUSED__, int smooth __UNUSED__, int level __UNUSED__)
+evas_engine_sdl16_image_map_draw(void *data __UNUSED__, void *context __UNUSED__, void *surface __UNUSED__, void *image __UNUSED__, int npoints __UNUSED__, RGBA_Map_Point *p __UNUSED__, int smooth __UNUSED__, int level __UNUSED__)
 {
 }
 
@@ -873,17 +899,13 @@ evas_engine_sdl16_image_format_get(void *data __UNUSED__, void *image __UNUSED__
 }
 
 static void
-evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Eina_Unicode *text, const Evas_BiDi_Props *intl_props)
+evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface, void *font, int x, int y, int w __UNUSED__, int h __UNUSED__, int ow __UNUSED__, int oh __UNUSED__, const Evas_Text_Props *intl_props)
 {
    static RGBA_Image            *im = NULL;
    SDL_Engine_Image_Entry       *eim = surface;
    Soft16_Image                 *dst = (Soft16_Image *) eim->cache_entry.src;
    int                           mustlock_im = 0;
 
-   if (!im)
-     im = (RGBA_Image *) evas_cache_image_empty(evas_common_image_cache_get());
-   evas_cache_image_surface_alloc(&im->cache_entry, dst->cache_entry.w, dst->cache_entry.h);
-
    if (eim->surface && SDL_MUSTLOCK(eim->surface))
      {
         mustlock_im = 1;
@@ -895,7 +917,7 @@ evas_engine_sdl16_font_draw(void *data __UNUSED__, void *context, void *surface,
                                          evas_common_soft16_font_glyph_new,
                                          evas_common_soft16_font_glyph_free,
                                          evas_common_soft16_font_glyph_draw);
-   evas_common_font_draw(im, context, font, x, y, text, intl_props);
+   evas_common_font_draw((RGBA_Image *) eim->cache_entry.src, context, font, x, y, intl_props);
    evas_common_draw_context_font_ext_set(context,
                                          NULL,
                                          NULL,
@@ -922,6 +944,7 @@ evas_engine_sdl16_line_draw(void *data __UNUSED__, void *context, void *surface,
    evas_common_soft16_line_draw((Soft16_Image *) eim->cache_entry.src,
                     context,
                     x1, y1, x2, y2);
+   evas_common_cpu_end_opt();
 
    if (mustlock_im)
      SDL_UnlockSurface(eim->surface);
@@ -953,6 +976,7 @@ evas_engine_sdl16_rectangle_draw(void *data __UNUSED__, void *context, void *sur
              im = (Soft16_Image *) eim->cache_entry.src;
 
              evas_common_soft16_rectangle_draw(im, context, x, y, w, h);
+             evas_common_cpu_end_opt();
 
              if (mustlock_im)
                SDL_UnlockSurface(eim->surface);
@@ -1002,6 +1026,7 @@ evas_engine_sdl16_polygon_draw(void *data __UNUSED__, void *context, void *surfa
      }
 
    evas_common_soft16_polygon_draw((Soft16_Image *) eim->cache_entry.src, context, polygon, x, y);
+   evas_common_cpu_end_opt();
 
    if (mustlock_im)
      SDL_UnlockSurface(eim->surface);
@@ -1074,7 +1099,7 @@ module_open(Evas_Module *em)
    ORD(image_border_set);
    ORD(image_border_get);
    ORD(image_draw);
-   ORD(image_map4_draw);
+   ORD(image_map_draw);
    ORD(image_cache_flush);
    ORD(image_cache_set);
    ORD(image_cache_get);
@@ -1142,15 +1167,20 @@ _sdl16_image_constructor(Engine_Image_Entry *ie, void* data __UNUSED__)
 
    im = (Soft16_Image *) ie->src;
 
-   if (im->pixels)
+   if (im)
      {
-        /* FIXME: Take care of CSPACE */
-        sdl = SDL_CreateRGBSurfaceFrom(im->pixels,
-                                       ie->w, ie->h,
-                                       16, ie->w * 2,
-                                       RMASK565, GMASK565, BMASK565, AMASK565);
-        eim->surface = sdl;
-        eim->flags.engine_surface = 0;
+        evas_cache_image_load_data(&im->cache_entry);
+
+        if (im->pixels)
+          {
+            /* FIXME: Take care of CSPACE */
+            sdl = SDL_CreateRGBSurfaceFrom(im->pixels,
+                                           ie->w, ie->h,
+                                           16, ie->w * 2,
+                                           RMASK565, GMASK565, BMASK565, AMASK565);
+            eim->surface = sdl;
+            eim->flags.engine_surface = 0;
+          }
      }
 
    return EVAS_LOAD_ERROR_NONE;
